 /* ++                                                                */
 /*                                                                   */
 /* Copyright (c) 1997-1998  Microsoft Corporation                    */
 /*                                                                   */
 /* Module Name:                                                      */
 /*                                                                   */
 /*   basetsd.cpy                                                     */
 /*                                                                   */
 /* Abstract:                                                         */
 /*                                                                   */
 /*   Type definitions for the basic sized types.                     */
 /*                                                                   */
 /* Author:                                                           */
 /*                                                                   */
 /*   Jeff Havens (jhavens)   23-Oct-1997                             */
 /*                                                                   */
 /* Revision History:                                                 */
 /*                                                                   */
 /* --                                                                */
 /*                                                                   */
 /* Important PL/I note:                                              */
 /* PL/I does not currently support 64-bit pointers.                  */
 /*                                                                   */
 /* All statements indicating the equivalence of the length           */
 /* of 64-bit integers and pointers are currently incorrect.          */
 /* This file must be revised when PL/I support for 64-bit pointers   */
 /* is realized in the compiler (change pointer64 preprocessor var).  */

 %dcl #BASETSD_H_ char ext;
 %if #BASETSD_H_ = '' %then %do;
 %#BASETSD_H_ = 'Y';

 %dcl #WIN64 char ext;


 /* PL/I abstraction type: change this when 64 bit pointers realized  */
 %dcl pointer64 char ext; %pointer64 = 'pointer';


 /* The following types are guaranteed to be signed and 32 bits wide. */

 define alias  LONG32 fixed bin (31) signed;
 define alias PLONG32 pointer;
 define alias  INT32  fixed bin (31) signed;
 define alias PINT32  pointer;


 /* The following types are guaranteed to be unsigned and 32 bits     */
 /* wide.                                                             */

 define alias  ULONG32 fixed bin (32) unsigned;
 define alias PULONG32 pointer;
 define alias  DWORD32 fixed bin (32) unsigned;
 define alias PDWORD32 pointer;
 define alias  UINT32  fixed bin (32) unsigned;
 define alias PUINT32  pointer;



 /* The INT_PTR is guaranteed to be the same size as a pointer. Its */
 /* size with change with pointer size (32/64).  It should be used  */
 /* anywhere that a pointer is cast to an integer type. UINT_PTR is */
 /* the unsigned variation.                                         */


 %if #WIN64 ^= '' %then %do;

 define alias  INT_PTR  fixed bin (63);
 define alias PINT_PTR  pointer64;
 define alias  UINT_PTR fixed bin (64) unsigned;
 define alias PUINT_PTR pointer64;

 define alias  LONG_PTR  fixed bin (63);
 define alias PLONG_PTR  pointer64;
 define alias  ULONG_PTR fixed bin (64) unsigned;
 define alias PULONG_PTR pointer64;

 define alias #_int3264 fixed bin (63);

 %end; %else %do;

 define alias  INT_PTR  fixed bin (31);
 define alias PINT_PTR  pointer;
 define alias  UINT_PTR fixed bin (32) unsigned;
 define alias PUINT_PTR pointer;

 define alias  LONG_PTR  fixed bin (31);
 define alias PLONG_PTR  pointer;
 define alias  ULONG_PTR fixed bin (32) unsigned;
 define alias PULONG_PTR pointer;

 define alias #_int3264 fixed bin (31);

 %end;


 /* HALF_PTR is half the size of a pointer it intended for use      */
 /* within structures which contain a pointer and two small fields. */
 /* UHALF_PTR is the unsigned variation.                            */

 %if #WIN64 ^= '' %then %do;

 dcl ADDRESS_TAG_BIT fixed bin(64) unsigned value('40000000000'xu);

 define alias HANDLE_PTR fixed bin(64) unsigned;

 define alias  UHALF_PTR fixed bin (32) unsigned;
 define alias PUHALF_PTR pointer;
 define alias  HALF_PTR  fixed bin (31);
 define alias PHALF_PTR  pointer;

 HandleToUlong: proc (h)
                returns (fixed bin (32) unsigned byvalue)
                options (inline reorder) internal;
   dcl h pointer64 byvalue;
   dcl (iunsigned, binvalue) builtin;
   return (iunsigned (binvalue (h)));
 end HandleToUlong;

 HandleToLong: proc (h)
               returns (fixed bin (31) byvalue)
               options (inline reorder) internal;
   dcl h pointer64 byvalue;
   dcl (isigned, binvalue) builtin;
   return (isigned (binvalue (h)));
 end HandleToLong;

 LongToHandle: proc (h)
               returns (pointer64 /* type HANDLE */ byvalue)
               options (inline reorder) internal;
   dcl h fixed bin (31) byvalue;
   dcl (ptrvalue) builtin;
   return (ptrvalue (cast (: INT_PTR, h :)));
 end LongToHandle;

 PtrToUlong: proc (p)
             returns (fixed bin (32) unsigned byvalue)
             options (inline reorder) internal;
   dcl p pointer64 byvalue;
   dcl (iunsigned, binvalue) builtin;
   return (iunsigned (binvalue (p)));
 end PtrToUlong;

 PtrToUshort: proc (p)
              returns (fixed bin (16) unsigned byvalue)
              options (inline reorder) internal;
   dcl p pointer64 byvalue;
   dcl (iunsigned, binvalue) builtin;
   return (iunsigned (binvalue (p)));
 end PtrToUshort;

 PtrToLong: proc (p)
            returns (fixed bin (31) byvalue)
            options (inline reorder) internal;
   dcl p pointer64 byvalue;
   dcl (isigned, binvalue) builtin;
   return (isigned (binvalue (p)));
 end PtrToLong;

 PtrToInt: proc (p)
            returns (fixed bin (31) byvalue)
            options (inline reorder) internal;
   dcl p pointer64 byvalue;
   dcl (isigned, binvalue) builtin;
   return (isigned (binvalue (p)));
 end PtrToInt;

 PtrToShort: proc (p)
             returns (fixed bin (15) byvalue)
             options (inline reorder) internal;
   dcl p pointer64 byvalue;
   dcl (isigned, binvalue) builtin;
   return (isigned (binvalue (p)));
 end PtrToShort;

 IntToPtr: proc (i)
           returns (pointer64 byvalue)
           options (inline reorder) internal;
   /* Caution: IntToPtr() sign-extends the int value. */
   dcl i fixed bin (31) byvalue;
   dcl (ptrvalue) builtin;
   return (ptrvalue (cast (: INT_PTR, i :)));
 end IntToPtr;

 UIntToPtr: proc (ui)
            returns (pointer64 byvalue)
            options (inline reorder) internal;
   /* Caution: UIntToPtr() zero-extends the unsigned int value. */
   dcl ui fixed bin (32) unsigned byvalue;
   dcl (ptrvalue) builtin;
   return (ptrvalue (cast (: UINT_PTR, ui :)));
 end UIntToPtr;

 LongToPtr: proc (l)
            returns (pointer64 byvalue)
            options (inline reorder) internal;
   /* Caution: IntToPtr() sign-extends the long value. */
   dcl l fixed bin (31) byvalue;
   dcl (ptrvalue) builtin;
   return (ptrvalue (cast (: LONG_PTR, l :)));
 end LongToPtr;

 ULongToPtr: proc (ul)
             returns (pointer64 byvalue)
             options (inline reorder) internal;
   /* Caution: ULongToPtr() zero-extends the unsigned long value. */
   dcl ul fixed bin (31) byvalue;
   dcl (ptrvalue) builtin;
   return (ptrvalue (cast (: ULONG_PTR, ul :)));
 end ULongToPtr;

 %end; %else %do;

 dcl ADDRESS_TAG_BIT     fixed bin (32) unsigned value('80000000'xu);

 define alias HANDLE_PTR fixed bin (32) unsigned;

 define alias  UHALF_PTR fixed bin (16) unsigned;
 define alias PUHALF_PTR pointer;
 define alias  HALF_PTR  fixed bin (15);
 define alias PHALF_PTR  pointer;


 HandleToUlong: proc (h)
                returns (fixed bin (32) unsigned byvalue)
                options (inline reorder) internal;
   dcl h pointer byvalue;
   dcl (iunsigned, binvalue) builtin;
   return (iunsigned (binvalue (h)));
 end HandleToUlong;

 HandleToLong: proc (h)
               returns (fixed bin (31) byvalue)
               options (inline reorder) internal;
   dcl h pointer byvalue;
   dcl (isigned, binvalue) builtin;
   return (isigned (binvalue (h)));
 end HandleToLong;

 LongToHandle: proc (h)
               returns (pointer /* type HANDLE */ byvalue)
               options (inline reorder) internal;
   dcl h fixed bin (31) byvalue;
   dcl (ptrvalue) builtin;
   return (ptrvalue (h));
 end LongToHandle;

 PtrToUlong: proc (p)
             returns (fixed bin (32) unsigned byvalue)
             options (inline reorder) internal;
   dcl p pointer byvalue;
   dcl (iunsigned, binvalue) builtin;
   return (iunsigned (binvalue (p)));
 end PtrToUlong;

 PtrToLong: proc (p)
            returns (fixed bin (31) byvalue)
            options (inline reorder) internal;
   dcl p pointer byvalue;
   dcl (isigned, binvalue) builtin;
   return (isigned (binvalue (p)));
 end PtrToLong;

 PtrToUint: proc (p)
             returns (fixed bin (32) unsigned byvalue)
             options (inline reorder) internal;
   dcl p pointer byvalue;
   dcl (iunsigned, binvalue) builtin;
   return (iunsigned (binvalue (p)));
 end PtrToUint;

 PtrToInt: proc (p)
             returns (fixed bin (31) byvalue)
             options (inline reorder) internal;
   dcl p pointer byvalue;
   dcl (isigned, binvalue) builtin;
   return (isigned (binvalue (p)));
 end PtrToInt;

 PtrToUshort: proc (p)
             returns (fixed bin (16) unsigned byvalue)
             options (inline reorder) internal;
   dcl p pointer byvalue;
   dcl (iunsigned, binvalue) builtin;
   return (iunsigned (binvalue (p)));
 end PtrToUshort;

 PtrToShort: proc (p)
             returns (fixed bin (15) byvalue)
             options (inline reorder) internal;
   dcl p pointer byvalue;
   dcl (isigned, binvalue) builtin;
   return (isigned (binvalue (p)));
 end PtrToShort;

 IntToPtr: proc (i)
           returns (pointer byvalue)
           options (inline reorder) internal;
   dcl i fixed bin (31) byvalue;
   dcl (ptrvalue) builtin;
   return (ptrvalue (cast (: INT_PTR, i :)));
 end IntToPtr;

 UIntToPtr: proc (ui)
            returns (pointer byvalue)
            options (inline reorder) internal;
   dcl ui fixed bin (32) unsigned byvalue;
   dcl (ptrvalue) builtin;
   return (ptrvalue (cast (: LONG_PTR, ui :)));
 end UIntToPtr;

 LongToPtr: proc (l)
            returns (pointer byvalue)
            options (inline reorder) internal;
   dcl l fixed bin (31) byvalue;
   dcl (ptrvalue) builtin;
   return (ptrvalue (cast (: LONG_PTR, l :)));
 end LongToPtr;

 ULongToPtr: proc (ul)
             returns (pointer byvalue)
             options (inline reorder) internal;
   dcl ul fixed bin (31) byvalue;
   dcl (ptrvalue) builtin;
   return (ptrvalue (cast (: ULONG_PTR, ul :)));
 end ULongToPtr;

 %end; /* !_WIN64 */

 /* PL/I help constants */
 dcl ZERO_UINT_PTR  type UINT_PTR   value(0);
 dcl ZERO_ULONG_PTR type ULONG_PTR  value(0);
 dcl ZERO_UHALF_PTR type UHALF_PTR  value(0);

 dcl MAXUINT_PTR  type UINT_PTR  value(inot(ZERO_UINT_PTR));
 dcl MAXINT_PTR   type  INT_PTR  value(isrl(MAXUINT_PTR,1));
 dcl MININT_PTR   type  INT_PTR  value(inot(MAXINT_PTR));

 dcl MAXULONG_PTR type ULONG_PTR value(inot(ZERO_ULONG_PTR));
 dcl MAXLONG_PTR  type  LONG_PTR value(isrl(MAXULONG_PTR,1));
 dcl MINLONG_PTR  type  LONG_PTR value(inot(MAXLONG_PTR));

 dcl MAXUHALF_PTR type UHALF_PTR value(inot(ZERO_UHALF_PTR));
 dcl MAXHALF_PTR  type  HALF_PTR value(isrl(MAXUHALF_PTR,1));
 dcl MINHALF_PTR  type  HALF_PTR value(inot(MAXHALF_PTR));


 /* SIZE_T used for counts or ranges which need to span the range of */
 /* of a pointer. SSIZE_T is the signed variation.                  */

 define alias  SIZE_T  type  ULONG_PTR;
 define alias PSIZE_T  type PULONG_PTR;
 define alias  SSIZE_T type  LONG_PTR;
 define alias PSSIZE_T type PLONG_PTR;
 %dcl #SIZE_T_DEFINED char ext;
 %#SIZE_T_DEFINED = 'Y';


 /* Add Windows flavor DWORD_PTR types */

 define alias  DWORD_PTR type  ULONG_PTR;
 define alias PDWORD_PTR type PULONG_PTR;


 /* The following types are guaranteed to be signed and 64 bits wide. */

 define alias  LONG64 fixed bin (63) signed;
 define alias PLONG64 pointer64;
 define alias  INT64  fixed bin (63) signed;
 define alias PINT64  pointer64;


 /* The following types are guaranteed to be unsigned and 64 bits     */
 /* wide.                                                             */

 define alias  ULONG64 fixed bin (64) unsigned;
 define alias PULONG64 pointer64;
 define alias  DWORD64 fixed bin (64) unsigned;
 define alias PDWORD64 pointer64;
 define alias  UINT64  fixed bin (64) unsigned;
 define alias PUINT64  pointer64;

 %end; /* _BASETSD_H_ */
