 /*********************************************************************/
 /*                                                                   */
 /* Source File Name: db2ApiDf                                        */
 /*                                                                   */
 /* (C) COPYRIGHT International Business Machines Corp. 1987, 1997    */
 /* All Rights Reserved                                               */
 /* Licensed Materials - Property of IBM                              */
 /*                                                                   */
 /* US Government Users Restricted Rights - Use, duplication or       */
 /* disclosure restricted by GSA ADP Schedule Contract with IBM Corp. */
 /*                                                                   */
 /* Function = Include File defining:                                 */
 /*              Environment Commands - Constants                     */
 /*              Environment Commands - Structures                    */
 /*              Environment Commands - Function Prototypes           */
 /*              Environment Commands - Labels for SQLCODES           */
 /*                                                                   */
 /* Operating System:  Windows NT                                     */
 /*                                                                   */
 /*********************************************************************/
 %dcl SQL_H_DB2APIDF char ext;
 %if SQL_H_DB2APIDF = '' %then %do;
 %SQL_H_DB2APIDF = 'Y';

 /* Required Include Files */

 %dcl SQL_H_SQL char ext;
 %if SQL_H_SQL = '' %then %do;
 %include sql;     /* Includes sqlsystm & sqlca */
 %end;
 %dcl #SQLDASIZE char ext;
 %if #SQLDASIZE = '' %then %do;
 %include sqlda_t;
 %end;
 %dcl SQL_H_SQLMON char ext;
 %if SQL_H_SQLMON = '' %then %do;
 %include sqlmon;
 %end;
 %dcl SQL_H_SQLAPREP char ext;
 %if SQL_H_SQLAPREP = '' %then %do;
 %include sqlaprep;
 %end;
 %dcl SQL_H_SQLUTIL char ext;
 %if SQL_H_SQLUTIL = '' %then %do;
 %include sqlutil;
 %end;

 /* Version, Release PTF identifier */
 dcl db2Version6Prior   fixed bin (31) value(SQL_RELPRE6);
     /* Pre Version 6.1.0.0   */
 dcl db2Version610      fixed bin (31) value(SQL_REL6100);
     /* Version 6.1.0.0       */
 dcl db2Version710      fixed bin (31) value(SQL_REL7100);
     /* Version 7.1.0.0       */

 /* Type definition names */
 define alias db2Uint32 type sqluint32; /* 32 bit unsigned int   */
 define alias db2int32  type sqlint32;  /* 32 bit signed int     */
 define alias db2Uint16 type sqluint16; /* 16 bit unsigned int   */
 define alias db2int16  type sqlint16;  /* 16 bit signed int     */

 /*********************************************************************/
 /* Structure db2Char                                                 */
 /*                                                                   */
 /* Description:                                                      */
 /*     pioData  -- Pointer to character data.                        */
 /*              -- If NULL, no data will be returned.                */
 /*                                                                   */
 /*     iLength  -- Input:  Size of the pData buffer.                 */
 /*                                                                   */
 /*     oLength  -- Output: Length of valid data in pData.            */
 /*              -- If the output data was truncated, this will be    */
 /*                 set to the complete size before truncation        */
 /*********************************************************************/

 define structure
   1 db2Char,
     2 pioData pointer,              /* Character data                */
     2 iLength type db2Uint32,       /* I: Length of pioData          */
     2 oLength type db2Uint32;       /* O: Untruncated length of data */

 /*********************************************************************/
 /* Load Query Structures and Constants                               */
 /*********************************************************************/


 /* Possible values for "iStringType" */
 dcl DB2LOADQUERY_TABLENAME              fixed bin(31) value(0);
                                         /* Tablename specified       */
 dcl DB2LOADQUERY_TEMPPATHNAME           fixed bin(31) value(1);

 /* Possible values for "iShowLoadMessages" */
 dcl DB2LOADQUERY_SHOW_ALL_MSGS          fixed bin(31) value(0);
                                         /* Show ALL LOAD messages    */
 dcl DB2LOADQUERY_SHOW_NO_MSGS           fixed bin(31) value(1);
                                         /* Show NO LOAD messages     */
 dcl DB2LOADQUERY_SHOW_NEW_MSGS          fixed bin(31) value(2);
                                         /* Show only NEW LOAD        */
                                         /* messages                  */

 /* Possible values for "oWhichPhase" */
 dcl DB2LOADQUERY_LOAD_PHASE             fixed bin(31) value(1);
                                         /* Load Phase                */
 dcl DB2LOADQUERY_BUILD_PHASE            fixed bin(31) value(2);
                                         /* Build Phase               */
 dcl DB2LOADQUERY_DELETE_PHASE           fixed bin(31) value(3);
                                         /* Delete Phase              */

 /* Load Query Output Structure */
 define structure
   1 db2LoadQueryOutputStruct,
     2 oRowsRead        type db2Uint32, /* Rows Read                  */
     2 oRowsSkipped     type db2Uint32, /* Rows Skipped               */
     2 oRowsCommitted   type db2Uint32, /* Rows Committed             */
     2 oRowsLoaded      type db2Uint32, /* Rows Loaded                */
     2 oRowsRejected    type db2Uint32, /* Rows Rejected              */
     2 oRowsDeleted     type db2Uint32, /* Rows Deleted               */
     2 oCurrentIndex    type db2Uint32, /* Current Index (BUILD PHASE)*/
     2 oNumTotalIndexes type db2Uint32, /* Total # of Indexes to build*/
                                        /* (BUILD PHASE)              */
     2 oCurrentMPPNode  type db2Uint32, /* Node being queried         */
                                        /* (MPP only)                 */
     2 oLoadRestarted   type db2Uint32, /* Load Restart indicator     */
     2 oWhichPhase      type db2Uint32, /* Phase of queried Load      */
     2 oWarningCount    type db2Uint32; /* Warning Count              */


 /* Load Query Parameter Structure */
 define structure
   1 db2LoadQueryStruct,
     2 iStringType       type db2Uint32,/* Type of piString           */
     2 piString          pointer,       /* Name to query              */
     2 iShowLoadMessages type db2Uint32,/* Level of Load message      */
                                        /* reporting                  */
     2 poOutputStruct    handle db2LoadQueryOutputStruct, /* Load     */
                                        /* Query Output Structure     */
     2 piLocalMessageFile pointer;      /* Message File               */


 dcl db2LoadQuery                       /* Load Query                 */
     entry (type db2Uint32,             /* Database version number    */
            pointer,                    /* In/out parameters          */
            pointer)                    /* SQLCA                      */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2LoadQuery');

 /* Generic Load Query Parameter Structure */
 define structure
   1 db2gLoadQueryStruct,
     2 iStringType       type db2Uint32,/* Type of piString           */
     2 iStringLen        type db2Uint32,/* Length in bytes of piString*/
     2 piString          pointer,       /* Name to query              */
     2 iShowLoadMessages type db2Uint32,/* Level of Load message      */
                                        /* reporting                  */
     2 poOutputStruct handle db2LoadQueryOutputStruct,/* Load Query   */
                                        /* Output Structure           */
     2 iLocalMessageFileLen type db2Uint32,/* Length in bytes of      */
                                        /* message file variable      */
     2 piLocalMessageFile pointer;      /* Message File               */

 dcl db2gLoadQuery                      /* Generic Load Query         */
     entry (type db2Uint32,             /* Database version number    */
            pointer,                    /* In/out parameters          */
            pointer)                    /* SQLCA                      */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2gLoadQuery');

 /*********************************************************************/
 /* Open History Scan Definitions                                     */
 /*********************************************************************/

 /* Values for db2HistoryOpen - iCallerAction */
 dcl DB2HISTORY_LIST_HISTORY        fixed bin(31) value(0);
 dcl DB2HISTORY_LIST_BACKUP         fixed bin(31) value(1);
 dcl DB2HISTORY_LIST_ROLLFORWARD    fixed bin(31) value(2);
 dcl DB2HISTORY_LIST_RUNSTATS       fixed bin(31) value(3);
 dcl DB2HISTORY_LIST_REORG          fixed bin(31) value(4);
 dcl DB2HISTORY_LIST_ALT_TABLESPACE fixed bin(31) value(5);
 dcl DB2HISTORY_LIST_DROPPED_TABLE  fixed bin(31) value(6);
 dcl DB2HISTORY_LIST_LOAD           fixed bin(31) value(7);
 dcl DB2HISTORY_LIST_REN_TABLESPACE fixed bin(31) value(8);
 dcl DB2HISTORY_LIST_CRT_TABLESPACE fixed bin(31) value(9);

 /* db2HistoryOpen input struct */
 define structure
   1 db2HistoryOpenStruct,
     2 piDatabaseAlias pointer, /* DB to fetch history from           */
     2 piTimestamp     pointer, /* Since this timestamp               */
     2 piObjectName    pointer, /* Entries containing this object     */
     2 oNumRows        type db2Uint32,/* # of entries matching search */
     2 oMaxTbspaces    type db2Uint32,/* Max. # of tablespace names   */
                                      /* stored with ANY history entry*/
     2 iCallerAction   type db2Uint16,/* Caller action                */
     2 oHandle         type db2Uint16;/* Handle for this scan         */

 /* db2HistoryOpenScan - API */
 dcl db2HistoryOpenScan                 /* Open a history file scan   */
     entry (type db2Uint32,             /* Database version number    */
            pointer,                    /* In/out parameters          */
            pointer)                    /* SQLCA                      */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2HistoryOpenScan');

 /* db2gHistoryOpen input struct */
 define structure
   1 db2gHistoryOpenStruct,
     2 piDatabaseAlias pointer, /* DB to fetch history from           */
     2 piTimestamp     pointer, /* Since this timestamp               */
     2 piObjectName    pointer, /* Entries containing this object     */
     2 iAliasLen       type db2Uint32,/* Length of database alias     */
                                      /* string                       */
     2 iTimestampLen   type db2Uint32,/* Length of timestamp string   */
     2 iObjectNameLen  type db2Uint32,/* Length of object name string */
     2 oNumRows        type db2Uint32,/* # of entries matching search */
     2 oMaxTbspaces    type db2Uint32,/* Max. # of tablespace names   */
                                      /* stored with ANY history entry*/
     2 iCallerAction   type db2Uint16,/* Caller action                */
     2 oHandle         type db2Uint16;/* Handle for this scan         */

 /* db2gHistoryOpenScan - Generic API */
 dcl db2gHistoryOpenScan                /* Open a history file scan   */
     entry (type db2Uint32,             /* Database version number    */
            pointer,                    /* In/out parameters          */
            pointer)                    /* SQLCA                      */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2gHistoryOpenScan');

 /*********************************************************************/
 /* Fetch Next History Entry Definititions                            */
 /*********************************************************************/

 /* Structure db2HistoryEID */
 define structure
   1 db2HistoryEID,
     2 ioNode type SQL_PDB_NODE_TYPE, /* Node number                  */
     2 ioHID  type db2Uint32;         /* Local history file entry ID  */

 /* Max Sizes of db2HistoryDataStruct db2Char fields */
 /* NOTE: These lengths do not include NULL terminators */
 dcl DB2HISTORY_TIMESTAMP_SZ  fixed bin(31) value(14);
 dcl DB2HISTORY_SEQNUM_SZ     fixed bin(31) value(3);
 dcl DB2HISTORY_OBJPART_SZ    fixed bin(31)
     value(DB2HISTORY_TIMESTAMP_SZ + DB2HISTORY_SEQNUM_SZ);
 dcl DB2HISTORY_LOGFILE_SZ    fixed bin(31) value(12);
 dcl DB2HISTORY_ID_SZ         fixed bin(31) value(24);
 dcl DB2HISTORY_TABLE_QUAL_SZ fixed bin(31) value(30);
 dcl DB2HISTORY_TABLE_NAME_SZ fixed bin(31) value(128);
 dcl DB2HISTORY_LOCATION_SZ   fixed bin(31) value(255);
 dcl DB2HISTORY_COMMENT_SZ    fixed bin(31) value(30);
 dcl DB2HISTORY_COMMAND_SZ    fixed bin(31) value(SQL_MAX_STMT_SIZ);

 /* Structure db2HistoryData                                          */
 /*                                                                   */
 /* Description:                                                      */
 /*      ioHistDataID     -- Eye catcher, must be set to "SQLUHINF"   */
 /*                                                                   */
 /*      oObjectPart      -- Start time + sequence number             */
 /*                       -- DB2HISTORY_OBJPART_SZ                    */
 /*                                                                   */
 /*      oEndTime         -- Completion timestamp                     */
 /*                       -- DB2HISTORY_TIMESTAMP_SZ                  */
 /*                                                                   */
 /*      oFirstLog        -- First log file referrenced by event      */
 /*                       -- DB2HISTORY_LOGFILE_SZ                    */
 /*                                                                   */
 /*      oLastLog         -- Last log file referrenced by event       */
 /*                       -- DB2HISTORY_LOGFILE_SZ                    */
 /*                                                                   */
 /*      oID              -- Backup ID (timestamp), or unique table ID*/
 /*                       -- DB2HISTORY_ID_SZ                         */
 /*                                                                   */
 /*      oTableQualifier  -- Table qualifier                          */
 /*                       -- DB2HISTORY_TABLE_QUAL_SZ                 */
 /*                                                                   */
 /*      oTableName       -- Table name                               */
 /*                       -- DB2HISTORY_TABLE_NAME_SZ                 */
 /*                                                                   */
 /*      oLocation        -- Location of object used or produced by   */
 /*                          event                                    */
 /*                       -- DB2HISTORY_LOCATION_SZ                   */
 /*                                                                   */
 /*      oComment         -- Text comment                             */
 /*                       -- DB2HISTORY_COMMENT_SZ                    */
 /*                                                                   */
 /*      oCommandText     -- Command text, or DDL                     */
 /*                       -- DB2HISTORY_COMMAND_SZ                    */
 /*                                                                   */
 /*      oLastLSN         -- Last Log Sequence Number                 */
 /*      poEventSQLCA     -- SQLCA returned at event completion       */
 /*      poTablespace     -- LIST of tablespace names                 */
 /*      iNumTablespaces  -- # of available entries in the            */
 /*                          poTablespace list                        */
 /*      oNumTablespaces  -- # of used entries in the                 */
 /*                          poTablespace list                        */
 /*      oEID             -- Unique history entry ID                  */
 /*      oOperation       -- Event type:        DB2HISTORY_TYPE_*     */
 /*      oObject          -- Event granularity: DB2HISTORY_GRAN_*     */
 /*      oOptype          -- Event details:     DB2HISTORY_OPTYPE_*   */
 /*      oStatus          -- Entry status:      DB2HISTORY_STATUS_*   */
 /*      oDeviceType      -- Type of oLocation: DB2_MEDIA_*           */

 define structure
   1 db2HistoryData,
     2 ioHistDataID    char(8),      /* Eye catcher = 'SQLUHINF'      */
     2 oObjectPart     type db2Char, /* Start time + sequence number  */
     2 oEndTime        type db2Char, /* Completion time of event      */
     2 oFirstLog       type db2Char, /* First log file                */
     2 oLastLog        type db2Char, /* Last log file                 */
     2 oID             type db2Char, /* Backup ID, or unique table ID */
     2 oTableQualifier type db2Char, /* Table qualifier               */
     2 oTableName      type db2Char, /* Table name                    */
     2 oLocation       type db2Char, /* Location of referrenced object*/
     2 oComment        type db2Char, /* Text comment                  */
     2 oCommandText    type db2Char, /* Command text                  */
     2 oLastLSN        type SQLU_LSN,/* Last log sequence number      */
     2 oEID            type db2HistoryEID,/* Unique entry ID          */
     2 poEventSQLCA    pointer,      /* Resulting SQLCA of event      */
     2 poTablespace    pointer,      /* List of tablespace names      */
     2 iNumTablespaces type db2Uint32,
                             /* I: # of available poTablespace fields */
     2 oNumTablespaces type db2Uint32,
                             /* O: # of used poTablespace fields      */
     2 oOperation      char(1),      /* Event type                    */
     2 oObject         char(1),      /* Operation granularity         */
     2 oOptype         char(1),      /* Operation details             */
     2 oStatus         char(1),      /* Status of entry               */
     2 oDeviceType     char(1);      /* Device type of 'location'     */

 /* Values for db2HistoryGetEntry - iCallerAction */
 dcl DB2HISTORY_GET_ENTRY  fixed bin(31) value(0);
                           /* Next entry, without command data        */
 dcl DB2HISTORY_GET_DDL    fixed bin(31) value(1);
                           /* ONLY command data from previous fetch   */
 dcl DB2HISTORY_GET_ALL    fixed bin(31) value(2);
                           /* Next entry, with all data               */

 /* db2HistoryGetEntry input struct */
 define structure
   1 db2HistoryGetEntryStruct,
     2 pioHistData   handle db2HistoryData,/* History entry data area */
     2 iHandle       type db2Uint16,       /* History scan handle     */
     2 iCallerAction type db2Uint16;       /* Caller action           */

 /* db2HistoryGetEntry - API */
 dcl db2HistoryGetEntry        /* Fetch the next history file entry   */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2HistoryGetEntry');

 /* db2gHistoryGetEntry - Generic API */
 dcl db2gHistoryGetEntry       /* Fetch the next history file entry   */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2gHistoryGetEntry');

 /* History Operations */
 dcl DB2HISTORY_OP_CRT_TABLESPACE   char value('A');
 dcl DB2HISTORY_OP_BACKUP           char value('B');
 dcl DB2HISTORY_OP_LOAD_COPY        char value('C');
 dcl DB2HISTORY_OP_DROPPED_TABLE    char value('D');
 dcl DB2HISTORY_OP_ROLLFWD          char value('F');
 dcl DB2HISTORY_OP_REORG            char value('G');
 dcl DB2HISTORY_OP_LOAD             char value('L');
 dcl DB2HISTORY_OP_REN_TABLESPACE   char value('N');
 dcl DB2HISTORY_OP_DROP_TABLESPACE  char value('O');
 dcl DB2HISTORY_OP_QUIESCE          char value('Q');
 dcl DB2HISTORY_OP_RESTORE          char value('R');
 dcl DB2HISTORY_OP_RUNSTATS         char value('S');
 dcl DB2HISTORY_OP_ALT_TABLESPACE   char value('T');
 dcl DB2HISTORY_OP_UNLOAD           char value('U');

 /* History Objects */
 dcl DB2HISTORY_GRAN_DB             char value('D');
 dcl DB2HISTORY_GRAN_TBS            char value('P');
 dcl DB2HISTORY_GRAN_TABLE          char value('T');

 /* Backup Optypes */
 dcl DB2HISTORY_OPTYPE_OFFLINE      char value('F');
 dcl DB2HISTORY_OPTYPE_ONLINE       char value('N');
 dcl DB2HISTORY_OPTYPE_INCR_OFFLINE char value('I');
 dcl DB2HISTORY_OPTYPE_INCR_ONLINE  char value('O');
 dcl DB2HISTORY_OPTYPE_DELT_OFFLINE char value('D');
 dcl DB2HISTORY_OPTYPE_DELT_ONLINE  char value('E');

 /* Quiesce Optypes */
 dcl DB2HISTORY_OPTYPE_SHARE        char value('S');
 dcl DB2HISTORY_OPTYPE_UPDATE       char value('U');
 dcl DB2HISTORY_OPTYPE_EXCL         char value('X');
 dcl DB2HISTORY_OPTYPE_RESET        char value('Z');

 /* Rollforward Optypes */
 dcl DB2HISTORY_OPTYPE_EOL          char value('E');
 dcl DB2HISTORY_OPTYPE_PIT          char value('P');

 /* Load Optypes */
 dcl DB2HISTORY_OPTYPE_INSERT       char value('I');
 dcl DB2HISTORY_OPTYPE_REPLACE      char value('R');

 /* Alter Tablespace Optypes */
 dcl DB2HISTORY_OPTYPE_ADD_CONT     char value('C');
 dcl DB2HISTORY_OPTYPE_REB          char value('R');

 /* History Entry Status Flags */
 dcl DB2HISTORY_STATUS_ACTIVE       char value('A');
 dcl DB2HISTORY_STATUS_INACTIVE     char value('I');
 dcl DB2HISTORY_STATUS_EXPIRED      char value('E');
 dcl DB2HISTORY_STATUS_DELETED      char value('D');
 dcl DB2HISTORY_STATUS_NC           char value('N');
 dcl DB2HISTORY_STATUS_INCMP_ACTV   char value('a');
 dcl DB2HISTORY_STATUS_INCMP_INACTV char value('i');

 /*********************************************************************/
 /* Close History Scan Definititions                                  */
 /*********************************************************************/

 /* db2HistoryCloseScan - API */
 dcl db2HistoryCloseScan       /* Close a history file scan           */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* History scan handle                 */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2HistoryCloseScan');


 /* db2gHistoryCloseScan - Generic API */
 dcl db2gHistoryCloseScan      /* Close a history file scan           */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* History scan handle                 */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2gHistoryCloseScan');

 /*********************************************************************/
 /* Update History Entry Definititions
 /*********************************************************************/

 /* Update history API input struct */
 define structure
   1 db2HistoryUpdateStruct,
     2 piNewLocation   pointer,
     2 piNewDeviceType pointer,
     2 piNewComment    pointer,
     2 iEID      type db2HistoryEID; /* ID of the entry to be updated */

 /* db2HistoryUpdate - API */
 dcl db2HistoryUpdate          /* Update history entry data           */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* Input parameters                    */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2HistoryUpdate');

 /* Update history API input struct */
 define structure
   1 db2gHistoryUpdateStruct,
     2 piNewLocation   pointer,
     2 piNewDeviceType pointer,
     2 piNewComment    pointer,
     2 iNewLocationLen type db2Uint32,
     2 iNewDeviceLen   type db2Uint32,
     2 iNewCommentLen  type db2Uint32,
     2 iEID      type db2HistoryEID; /* ID of the entry to be updated */

 /* db2HistoryUpdate - Generic API */
 dcl db2gHistoryUpdate         /* Update history entry data           */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* Input parameters                    */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2gHistoryUpdate');

 /*********************************************************************/
 /* Precompiler API Related Definitions                               */
 /*********************************************************************/

 /* Compile SQL Parmeter Structure */
 define structure
   1 db2CompileSqlStruct,
     2 piSqlStmtLen    pointer, /* SQL statement text length          */
     2 piSqlStmt       pointer, /* SQL statement text                 */
     2 piLineNum       pointer, /* Source Line Number                 */
     2 pioFlagInfo     handle sqla_flaginfo,/* SQL Flagger information*/
     2 pioTokenIdArray handle sqla_tokens,  /* Token ID Array         */
     2 poTaskArray     handle sqla_tasks,   /* Task Array             */
     2 poSectionNum    pointer, /* Section Number                     */
     2 poSqlStmtType   pointer, /* Type of SQL statement              */
     2 poBuffer1       pointer, /* 256 byte string buffer 1           */
     2 poBuffer2       pointer, /* 256 byte string buffer 2           */
     2 poBuffer3       pointer, /* 256 byte string buffer 3           */
     2 pioReserved     pointer; /* Reserved                           */

 /* Generic Compile SQL Parmeter Structure */
 define structure
   1 db2gCompileSqlStruct,
     2 piSqlStmtLen    pointer, /* SQL statement text length          */
     2 piSqlStmt       pointer, /* SQL statement text                 */
     2 piLineNum       pointer, /* Source Line Number                 */
     2 pioFlagInfo     handle sqla_flaginfo,/* SQL Flagger information*/
     2 pioTokenIdArray handle sqla_tokens,  /* Token ID Array         */
     2 poTaskArray     handle sqla_tasks,   /* Task Array             */
     2 poSectionNum    pointer, /* Section Number                     */
     2 poSqlStmtType   pointer, /* Type of SQL statement              */
     2 poBuffer1       pointer, /* 256 byte string buffer 1           */
     2 poBuffer2       pointer, /* 256 byte string buffer 2           */
     2 poBuffer3       pointer, /* 256 byte string buffer 3           */
     2 pioReserved     pointer; /* Reserved                           */

 /* Compile SQL API */
 dcl db2CompileSql             /* Compile an SQL statement            */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2CompileSql');

 /* Generic Compile SQL API */
 dcl db2gCompileSql            /* Compile an SQL statement            */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2gCompileSql');

 /*********************************************************************/
 /* Prune Database History Definititions                              */
 /*********************************************************************/

 /* Values for db2Prune - iCallerAction */
 dcl DB2PRUNE_ACTION_HISTORY type db2Uint32 value(1);
                             /* Remove history file entries           */
 dcl DB2PRUNE_ACTION_LOG     type db2Uint32 value(2);
                             /* Remove log files from active log path */

 /* Values for db2Prune - iOptions */
 dcl DB2PRUNE_OPTION_NONE    type db2Uint32 value('0'xn);
                             /* Force removal of last backup          */
 dcl DB2PRUNE_OPTION_FORCE   type db2Uint32 value('1'xn);
                             /* Force removal of last backup          */
 dcl DB2PRUNE_OPTION_LSNSTRING type db2Uint32 value('2'xn);
                             /* piString is an LSN string, used by    */
                             /* DB2PRUNE ACTION_LOG                   */

 /* Prune API input struct */
 define structure
   1 db2PruneStruct,
     2 piString pointer,            /* Timestamp, or lastLSN          */
     2 iEID     type db2HistoryEID, /* Prune history since this EID   */
     2 iAction  type db2Uint32,     /* What to prune                  */
     2 iOptions type db2Uint32;     /* Options relevent to particular */
                                    /* iAction                        */

 dcl db2Prune                  /* Prune history / log                 */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* Input parameters                    */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2Prune');

 /* Generic Prune API input struct */
 define structure
   1 db2gPruneStruct,
     2 iStringLen type db2Uint32,    /* length in bytes of piString   */
     2 piString   pointer,           /* Timestamp, or lastLSN         */
     2 iEID       type db2HistoryEID,/* Prune history since this EID  */
     2 iAction    type db2Uint32,    /* What to prune                 */
     2 iOptions   type db2Uint32;    /* Options relevent to particular*/
                                     /* iAction                       */

 dcl db2gPrune                 /* Generic Prune history / log         */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* Input parameters                    */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2gPrune');

 /*********************************************************************/
 /* Archive Active Log Definitions                                    */
 /*********************************************************************/

 /* Values for db2ArchiveLog - iAllNodeFlag */
 dcl DB2ARCHIVELOG_NODE_LIST        fixed bin (31) value(0);
                                         /* Apply to nodes specified  */
                                         /* in piNodeList             */
 dcl DB2ARCHIVELOG_ALL_NODES        fixed bin (31) value(1);
                                         /* Apply to all nodes        */
 dcl DB2ARCHIVELOG_ALL_EXCEPT       fixed bin (31) value(2);
                                         /* Apply to all nodes        */
                                         /* except those in           */
                                         /* piNodeList                */

 /* Archive Active Log API Input Structure */
 define structure
   1 db2ArchiveLogStruct,
     2 piDatabaseAlias pointer, /* Alias of database to archive log   */
                                /* for                                */
     2 piUserName      pointer, /* User name to execute command under */
     2 piPassword      pointer, /* Password for username              */
     2 iAllNodeFlag type db2Uint16, /* Flag indicating how the        */
                                 /* iNodeList should be used          */
     2 iNumNodes type db2Uint16, /* Number of nodes in piNodeList     */
                                 /* array                             */
     2 piNodeList      pointer,  /* Node list              */
     2 iOptions type db2Uint32;  /* Future use                        */

 dcl db2ArchiveLog             /* Archive Active Log                  */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* Input parameters                    */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2ArchiveLog');

 /* Generic Archive Active Log API Input Struct */
 define structure
   1 db2gArchiveLogStruct,
     2 iAliasLen     type db2Uint32, /* Length of database alias field*/
     2 iUserNameLen  type db2Uint32, /* Length of username field      */
     2 iPasswordLen  type db2Uint32, /* Length of password field      */
     2 piDatabaseAlias pointer, /* Alias of database to archive log   */
                                /* for                                */
     2 piUserName      pointer, /* User name to execute command under */
     2 piPassword      pointer, /* Password for username              */
     2 iAllNodeFlag type db2Uint16, /* Flag indicating how the        */
                                 /* iNodeList should be used          */
     2 iNumNodes type db2Uint16, /* Number of nodes in piNodeList     */
                                 /* array                             */
     2 piNodeList      pointer,  /* Node list              */
     2 iOptions type db2Uint32;  /* Future use                        */

 dcl db2gArchiveLog            /* Generic Archive Active Log          */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* Input parameters                    */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2gArchiveLog');


 /* Ping Database Parameter Structure */
 define structure
   1 db2DatabasePingStruct,
     2 iDbAlias char (SQL_ALIAS_SZ) varz, /* Input: Database alias -  */
                                          /* reserved                 */
     2 iNumIterations type db2Uint16, /* Input: Number of iterations  */
     2 poElapsedTime  pointer;  /* Output: Array of elapsed times     */
                                /* (microsec), contains               */
                                /* iNumIterations elements            */

 dcl db2DatabasePing           /* Ping Database                       */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2DatabasePing');

 /* Generic Ping Database Parameter Structure */
 define structure
   1 db2gDatabasePingStruct,
     2 iDbAliasLength type db2Uint16,  /* Input: Db alias name length */
                                       /* - reserved                  */
     2 iDbAlias char (8),              /* Input: Database alias       */
                                       /* - reserved                  */
     2 iNumIterations type db2Uint16, /* Input: Number of iterations  */
     2 poElapsedTime  pointer;  /* Output: Array of elapsed times     */
                                /* (microsec), contains               */
                                /* iNumIterations elements            */

 dcl db2gDatabasePing          /* Generic Ping Database               */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2gDatabasePing');

 /* Convert LSN from internal representation to ascii hex string.     */
 /*      _inLsn  - SQLU_LSN * that points to the LSN in db2 internal  */
 /*                b-byte binary format.                              */
 /*      _outStr - char * to store the output in hex string format.   */
 /*                It should be big enough to store at least 13 bytes.*/

 db2LsnToString: proc (_inLsn, _outStr) options (inline) internal;
   dcl _inLsn  type SQLU_LSN byaddr,
       _outStr char (*) varz byaddr;
   dcl hex builtin;
   _outstr = hex (_inlsn.lsnChar);
 end db2LsnToString;


 /* Convert Monitor Data Interface Structure */
 define structure
   1 db2ConvMonStreamData,
     2 poTarget    pointer,        /* Pointer to target structure     */
     2 piSource    handle sqlm_header_info,/* Pointer to V6 stream    */
     2 iTargetType type db2Uint32, /* Target structure type           */
     2 iTargetSize type db2Uint32, /* Space allocated for target      */
     2 iSourceType type db2Uint32; /* Type of source stream           */

 dcl db2ConvMonStream          /* Convert monitor stream              */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2ConvMonStream');

 dcl db2gConvMonStream         /* Convert monitor stream              */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2gConvMonStream');

 /* Get Snapshot Data Interface Structure */
 define structure
   1 db2GetSnapshotData,
     2 piSqlmaData     handle sqlma,   /* Pointer to monitor area     */
     2 poCollectedData handle sqlm_collected,
                                       /* Pointer to collected data   */
     2 poBuffer        pointer,        /* Pointer to output buffer    */
     2 iVersion        type db2Uint32, /* Snapshot version            */
     2 iBufferSize     type db2Uint32, /* Size of output buffer       */
     2 iStoreResult    type db2Uint32, /* Write to file flag          */
     2 iNodeNumber     type db2int32,  /* Target node                 */
     2 poOutputFormat  pointer; /* Pointer to output format indicator */

 define structure
   1 db2gGetSnapshotData,
     2 piSqlmaData     handle sqlma,   /* Pointer to monitor area     */
     2 poCollectedData handle sqlm_collected,
                                       /* Pointer to collected data   */
     2 poBuffer        pointer,        /* Pointer to output buffer    */
     2 iVersion        type db2Uint32, /* Snapshot version            */
     2 iBufferSize     type db2Uint32, /* Size of output buffer       */
     2 iStoreResult    type db2Uint32, /* Write to file flag          */
     2 iNodeNumber     type db2int32,  /* Target node                 */
     2 poOutputFormat  pointer; /* Pointer to output format indicator */

 dcl db2GetSnapshot            /* Get snapshot                        */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2GetSnapshot');

 dcl db2gGetSnapshot           /* Get snapshot                        */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2gGetSnapshot');

 /* Get Snapshot Size Data Interface Structure */
 define structure
   1 db2GetSnapshotSizeData,
     2 piSqlmaData     handle sqlma,   /* Pointer to monitor area     */
     2 poBufferSize    pointer,        /* Pointer to output buffer    */
     2 iVersion        type db2Uint32, /* Snapshot version            */
     2 iNodeNumber     type db2int32;  /* Target node                 */

 define structure
   1 db2gGetSnapshotSizeData,
     2 piSqlmaData     handle sqlma,   /* Pointer to monitor area     */
     2 poBufferSize    pointer,        /* Pointer to output buffer    */
     2 iVersion        type db2Uint32, /* Snapshot version            */
     2 iNodeNumber     type db2int32;  /* Target node                 */

 dcl db2GetSnapshotSize        /* Get snapshot size                   */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2GetSnapshotSize');

 dcl db2gGetSnapshotSize       /* Get snapshot size                   */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2gGetSnapshotSize');

 /* Monitor Switches Data Interface Structure */
 define structure
   1 db2MonitorSwitchesData,
     2 piGroupStates handle sqlm_recording_group,
                                      /* Pointer to group states      */
     2 poBuffer       pointer,        /* Pointer to output buffer     */
     2 iBufferSize    type db2Uint32, /* Size of output buffer        */
     2 iReturnData    type db2Uint32, /* Return data flag             */
     2 iVersion       type db2Uint32, /* Snapshot version             */
     2 iNodeNumber    type db2int32,  /* Target node                  */
     2 poOutputFormat pointer;  /* Pointer to output format indicator */

 define structure
   1 db2gMonitorSwitchesData,
     2 piGroupStates handle sqlm_recording_group,
                                      /* Pointer to group states      */
     2 poBuffer       pointer,        /* Pointer to output buffer     */
     2 iBufferSize    type db2Uint32, /* Size of output buffer        */
     2 iReturnData    type db2Uint32, /* Return data flag             */
     2 iVersion       type db2Uint32, /* Snapshot version             */
     2 iNodeNumber    type db2int32,  /* Target node                  */
     2 poOutputFormat pointer;  /* Pointer to output format indicator */

 dcl db2MonitorSwitches        /* Get monitor switches                */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2MonitorSwitches');

 dcl db2gMonitorSwitches       /* Get monitor switches                */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2gMonitorSwitches');

 /* Monitor Reset Data Interface Structure */
 define structure
   1 db2ResetMonitorData,
     2 iResetAll      type db2Uint32, /* Reset value flag */
     2 piDbAlias      pointer,        /* Database alias */
     2 iVersion       type db2Uint32, /* Snapshot version */
     2 iNodeNumber    type db2int32;  /* Target node */

 /* Monitor Reset Data General Interface Structure */
 define structure
   1 db2gResetMonitorData,
     2 iResetAll      type db2Uint32, /* Reset value flag */
     2 piDbAlias      pointer,        /* Database alias */
     2 iDbAliasLength type db2Uint32, /* Length of DB alias   */
     2 iVersion       type db2Uint32, /* Snapshot version */
     2 iNodeNumber    type db2int32;  /* Target node */

 dcl db2ResetMonitor           /* Reset monitor                       */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2ResetMonitor');

 dcl db2gResetMonitor          /* Reset monitor                       */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2gResetMonitor');

 /* Restart Database Structure */
 define structure
   1 db2RestartDbStruct,
     2 piDatabaseName    pointer, /* Database name or alias           */
     2 piUserId          pointer, /* User Id                          */
     2 piPassword        pointer, /* Password                         */
     2 piTablespaceNames pointer; /* Tablespace name list             */

 dcl db2DatabaseRestart        /* Database Restart                    */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2DatabaseRestart');

 define structure
   1 db2gRestartDbStruct,
     2 iDatabaseNameLen    type db2Uint32,
                           /* Length in bytes of piDatabaseName       */
     2 iUserIdLen          type db2Uint32,
                           /* Length in bytes of piUserId             */
     2 iPasswordLen        type db2Uint32,
                           /* Length in bytes of piPassword           */
     2 iTablespaceNamesLen type db2Uint32,
                           /* Length in bytes of piTablespaceNames    */
     2 piDatabaseName      pointer, /* Database name or alias         */
     2 piUserId            pointer, /* User Id                        */
     2 piPassword          pointer, /* Password                       */
     2 piTablespaceNames   pointer; /* Tablespace name list           */

 dcl db2gDatabaseRestart       /* Generic Database Restart            */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2gDatabaseRestart');

 /*********************************************************************/
 /* Satellite Synchronization APIs and related definitions            */
 /*********************************************************************/


 /* Maximum length of a sync session ID */
 dcl SQL_SYNCSESSIONID_SZ fixed bin(31) value(18);

 /* Phases of a synchronization session, as            */
 /* returned in db2QuerySatelliteProgressStruct.oStep  */
 dcl DB2SYNC_SYNCMODE_PHASE_INIT            fixed bin(31) value(10);
 dcl DB2SYNC_SYNCMODE_PHASE_EXECPREVSCRIPTS fixed bin(31) value(20);
 dcl DB2SYNC_SYNCMODE_PHASE_SENDPREVRESULTS fixed bin(31) value(30);
 dcl DB2SYNC_SYNCMODE_PHASE_SENTPREVRESULTS fixed bin(31) value(40);
 dcl DB2SYNC_SYNCMODE_PHASE_RETRIEVEBATCHES fixed bin(31) value(50);
 dcl DB2SYNC_SYNCMODE_PHASE_EXECUTESCRIPTS  fixed bin(31) value(60);
 dcl DB2SYNC_SYNCMODE_PHASE_SENDRESULTS     fixed bin(31) value(70);
 dcl DB2SYNC_SYNCMODE_PHASE_SENTRESULTS     fixed bin(31) value(80);
 dcl DB2SYNC_SYNCMODE_PHASE_DONE            fixed bin(31) value(90);

 /* Phases of a synchronization test session, as      */
 /* returned in db2QuerySatelliteProgressStruct.oStep */
 dcl DB2SYNC_TESTMODE_PHASE_INIT            fixed bin(31) value(100);
 dcl DB2SYNC_TESTMODE_PHASE_CONNTODB        fixed bin(31) value(110);
 dcl DB2SYNC_TESTMODE_PHASE_DISCONNDB       fixed bin(31) value(120);
 dcl DB2SYNC_TESTMODE_PHASE_DONE            fixed bin(31) value(130);


 /* Input parameters for the Set Synchronization Session API */
 define structure
   1 db2SetSyncSessionStruct,
     2 piSyncSessionID pointer; /* ID for sync session */

 dcl db2SetSyncSession         /* Set sync session for a satellite    */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* Input parameters                    */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2SetSyncSession');

 /* Output parameters for the Get Synchronization Session API */
 define structure
   1 db2GetSyncSessionStruct,
     2 poSyncSessionID pointer; /* ID for sync session */

 dcl db2GetSyncSession         /* Get sync session ID for a satellite */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* Output parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2GetSyncSession');

 /* Output parameters for the Query Satellite Progress API */
 define structure
   1 db2QuerySatelliteProgressStruct,
     2 oStep         type db2int32, /* current step of synchronization*/
     2 oSubstep      type db2int32, /* substep of the current step    */
     2 oNumSubsteps  type db2int32, /* total number of substeps       */
     2 oScriptStep   type db2int32, /* step of current script substep */
     2 oNumScriptSteps type db2int32,/* total number of script steps  */
     2 poDescription pointer,       /* description of step            */
     2 poError       pointer,       /* error text, if available       */
     2 poProgressLog pointer;       /* contents of progress log       */

 dcl db2QuerySatelliteProgress /* Query Satellite Progress            */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* Ouput parameters                    */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2QuerySatelliteProgress');

 dcl db2SyncSatelliteTest      /* Test Satellite's ability to         */
                               /* synchronize                         */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* Input/Ouput parameters              */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2SyncSatelliteTest');

 dcl db2SyncSatellite          /* Synchronize Satellite               */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* Input/Ouput parameters              */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2SyncSatellite');

 dcl db2SyncSatelliteStop      /* Stop Satellite's Synchronization    */
                               /* Session                             */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* Input/Ouput parameters              */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2SyncSatelliteStop');

 /* Administrative Message Logging Defines/Structure */

 dcl DB2LOG_SEVERE_ERROR fixed bin(31) value(1);
 dcl DB2LOG_ERROR        fixed bin(31) value(2);
 dcl DB2LOG_WARNING      fixed bin(31) value(3);
 dcl DB2LOG_INFORMATION  fixed bin(31) value(4);

 dcl STRING_MSG          fixed bin(31) value(1);
 dcl BINARY_MSG          fixed bin(31) value(2);

 define structure
   1 db2AdminMsgWriteStruct,
     2 iMsgType     type db2Uint32, /* Message Type                   */
     2 iComponent   type db2Uint32, /* Component id                   */
     2 iFunction    type db2Uint32, /* Function id                    */
     2 iProbeID     type db2Uint32, /* Numeric probe point            */
     2 piData_title pointer,        /* Title describing data          */
     2 piData       pointer,        /* Data                           */
     2 iDataLen     type db2Uint32, /* Length of binary data          */
     2 iError_type  type db2Uint32; /* Severity of message            */

 /* Administrative Message Logging */

 dcl db2AdminMsgWrite          /* Administrative Message Logging      */
     entry (type db2Uint32,    /* Database version number             */
            pointer,           /* In/out parameters                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2AdminMsgWrite');

 define structure
   1 db2LdapProtocolInfo,
     2 iType               char(1),
     2 piHostName          pointer,
     2 piServiceName       pointer,
     2 piNetbiosName       pointer,
     2 piNetworkId         pointer,
     2 piPartnerLU         pointer,
     2 piTPName            pointer,
     2 piMode              pointer,
     2 iSecurityType       fixed bin(16) unsigned,
     2 piLanAdapterAddress pointer,
     2 piChangePasswordLU  pointer,
     2 piIpxAddress        pointer;

 define structure
   1 db2LdapRegisterStruct,
     2 piNodeName          pointer,
     2 piComputer          pointer,
     2 piInstance          pointer,
     2 iNodeType           fixed bin(16) unsigned,
     2 iOsType             fixed bin(16) unsigned,
     2 iProtocol           type db2LdapProtocolInfo,
     2 piComment           pointer,
     2 piBindDN            pointer,
     2 piPassword          pointer;

 dcl db2LdapRegister           /* Register DB2 server                 */
     entry (type db2Uint32,    /* Version Number                      */
            pointer,           /* Register Info                       */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2LdapRegister');

 define structure
   1 db2LdapUpdateStruct,
     2 piNodeName          pointer,
     2 piComment           pointer,
     2 iNodeType           fixed bin(16) unsigned,
     2 iProtocol           type db2LdapProtocolInfo,
     2 piBindDN            pointer,
     2 piPassword          pointer;

 dcl db2LdapUpdate             /* Update DB2 server                   */
     entry (type db2Uint32,    /* Version Number                      */
            pointer,           /* Update Info                         */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2LdapUpdate');

 define structure
   1 db2LdapDeregisterStruct,
     2 piNodeName          pointer,
     2 piBindDN            pointer,
     2 piPassword          pointer;

 dcl db2LdapDeregister         /* Deregister DB2 server               */
     entry (type db2Uint32,    /* Version Number                      */
            pointer,           /* DeRegister Info                     */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2LdapDeregister');

 define structure
   1 db2LdapCatalogNodeStruct,
     2 piAlias             pointer,
     2 piNodeName          pointer,
     2 piBindDN            pointer,
     2 piPassword          pointer;

 dcl db2LdapCatalogNode        /* Catalog LDAP node                   */
     entry (type db2Uint32,    /* Version Number                      */
            pointer,           /* Catalog Node Info                   */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2LdapCatalogNode');

 define structure
   1 db2LdapUncatalogNodeStruct,
     2 piAlias             pointer,
     2 piBindDN            pointer,
     2 piPassword          pointer;

 dcl db2LdapUncatalogNode      /* Uncatalog LDAP node                 */
     entry (type db2Uint32,    /* Version Number                      */
            pointer,           /* UnCatalog Node Info                 */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2LdapUncatalogNode');

 define structure
   1 db2LdapCatalogDatabaseStruct,
     2 piAlias             pointer,
     2 piDatabaseName      pointer,
     2 piComment           pointer,
     2 piNodeName          pointer,
     2 piGWNodeName        pointer,
     2 piParameters        pointer,
     2 piARLibrary         pointer,
     2 iAuthentication     fixed bin(16) unsigned,
     2 piDCEPrincipalName  pointer,
     2 piBindDN            pointer,
     2 piPassword          pointer;

 dcl db2LdapCatalogDatabase    /* Catalog LDAP database               */
     entry (type db2Uint32,    /* Version Number                      */
            pointer,           /* Catalog Database Info               */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2LdapCatalogDatabase');

 define structure
   1 db2LdapUncatalogDatabaseStruct,
     2 piAlias             pointer,
     2 piBindDN            pointer,
     2 piPassword          pointer;

 dcl db2LdapUncatalogDatabase  /* Uncatalog LDAP database             */
     entry (type db2Uint32,    /* Version Number                      */
            pointer,           /* UnCatalog Database Info             */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2LdapUncatalogDatabase');

 /* Suspend I/O Database Structure */
 define structure
   1 db2SetWriteDbStruct,
     2 iOption     type db2int32, /* Suspend or resume                */
     2 piTablespaceNames pointer; /* Suspending tablespace name list: */
                                  /* empty for all Tablespaces        */

 dcl db2SetWriteForDB          /* Set Write Suspend/Resume For DB     */
     entry (type db2Uint32,    /* Version Number                      */
            pointer,           /* UnCatalog Database Info             */
            pointer)           /* SQLCA                               */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2SetWriteForDB');

 %end; /* SQL_H_DB2APIDF */
