 /*  M A P I . C P Y                                                  */
 /*                                                                   */
 /*  Messaging Applications Programming Interface.                    */
 /*                                                                   */
 /*  Copyright 1986-1996 Microsoft Corporation. All Rights Reserved.  */
 /*                                                                   */
 /*  Purpose:                                                         */
 /*                                                                   */
 /*    This file defines the structures and constants used by that    */
 /*    subset of the Messaging Applications Programming Interface     */
 /*    which is supported under Windows by Microsoft Mail for PC      */
 /*    Networks version 3.x.                                          */

 /*********************************************************************/
 /* PL/I Note: There is no import LIB for simple MAPI, as MAPI is an  */
 /* optional component of Windows (it need not be installed).         */
 /* The application programmer is expected to use LoadLibrary and     */
 /* GetProcAddress. See the Microsoft Platform SDK for details.       */
 /* In PL/I, you can also use the FETCH statement, which is easier.   */
 /*                                                                   */
 /* The MS Platform SDK says to load MAPI32.DLL under 32-bit Windows  */
 /* To make things easier, entry declarations indicating that this be */
 /* done automatically on call are provided, so all you have to do is */
 /* call the functions.                                               */
 /* You should still test for the presence of MAPI by looking up the  */
 /* value of the MAPI variable in the registry under                  */
 /* HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows Messaging Subsystem */
 /* and checking that it is 1 (see MS Platform SDK for details).      */
 /* You can also use an ON ERROR and test for ONCODE() = 9250.        */
 /*********************************************************************/


 %dcl MAPI_H char ext;
 %if MAPI_H = '' %then %do;
 %MAPI_H = 'Y';

 %dcl #WINDEF_ char ext;
 %if #WINDEF_ = '' %then %do;
 %include windef;
 %end;

 %dcl SMAPI_API char ext;
 %SMAPI_API
   = 'options (byvalue linkage(stdcall) nodescriptor fetchable)';

 /* Types. */

 /* define alias LPULONG pointer */
 define alias FLAGS fixed bin(32) unsigned;

 %dcl #_LHANDLE char ext;
 %if #_LHANDLE = '' %then %do;
 %#_LHANDLE = 'Y';
 define alias LHANDLE   fixed bin(32) unsigned;
 define alias LPLHANDLE pointer;
 %end;

 /* define alias LPBYTE pointer */

 dcl lhSessionNull type LHANDLE value(0);

 define structure
   1 MapiFileDesc,
     2 ulReserved   type ULONG, /* Reserved for future use (must be 0)*/
     2 flFlags      type ULONG, /* Flags */
     2 nPosition    type ULONG,
       /* character in text to be replaced by attachment */
     2 lpszPathName type LPSTR, /* Full path name of attachment file */
     2 lpszFileName type LPSTR, /* Original file name (optional) */
     2 lpFileType   type LPVOID;
       /* Attachment file type (can be lpMapiFileTagExt) */
 define alias lpMapiFileDesc handle MapiFileDesc;

 dcl MAPI_OLE        type ULONG value('00000001'xu);
 dcl MAPI_OLE_STATIC type ULONG value('00000002'xu);


 define structure
   1 MapiFileTagExt,
     2 ulReserved type ULONG, /* Reserved, must be zero. */
     2 cbTag      type ULONG, /* Size (in bytes) of */
     2 lpTag      type LPBYTE,/* X.400 OID for this attachment type */
     2 cbEncoding type ULONG, /* Size (in bytes) of */
     2 lpEncoding type LPBYTE;
       /* X.400 OID for this attachment's encoding */
 define alias lpMapiFileTagExt handle MapiFileTagExt;


 define structure
   1 MapiRecipDesc,
     2 ulReserved   type ULONG,/* Reserved for future use */
     2 ulRecipClass type ULONG,/* Recipient class */
                    /* MAPI_TO, MAPI_CC, MAPI_BCC, MAPI_ORIG */
     2 lpszName     type LPSTR,/* Recipient name */
     2 lpszAddress  type LPSTR,/* Recipient address (optional) */
     2 ulEIDSize    type ULONG,/* Count in bytes of size of pEntryID */
     2 lpEntryID    type LPVOID;/* System-specific recipient reference*/

 define alias lpMapiRecipDesc handle MapiRecipDesc;

 %dcl #MAPI_ORIG char ext;
 %if #MAPI_ORIG = '' %then %do;
 %#MAPI_ORIG = 'Y';
 dcl MAPI_ORIG type ULONG value(0);
     /* Recipient is message originator   */
 dcl MAPI_TO   type ULONG value(1);
     /* Recipient is a primary recipient  */
 dcl MAPI_CC   type ULONG value(2);
     /* Recipient is a copy recipient     */
 dcl MAPI_BCC  type ULONG value(3);
     /* Recipient is blind copy recipient */
 %end;

 define structure
   1 MapiMessage,
     2 ulReserved       type ULONG,/* Reserved for future use (M.B. 0)*/
     2 lpszSubject      type LPSTR,/* Message Subject */
     2 lpszNoteText     type LPSTR,/* Message Text */
     2 lpszMessageType  type LPSTR,/* Message Class */
     2 lpszDateReceived type LPSTR,/* in YYYY/MM/DD HH:MM format */
     2 lpszConversationID type LPSTR,/* conversation thread ID */
     2 flFlags          type FLAGS,/* unread,return receipt */
     2 lpOriginator     type lpMapiRecipDesc,/* Originator descriptor */
     2 nRecipCount      type ULONG,/* Number of recipients */
     2 lpRecips         type lpMapiRecipDesc,/* Recipient descriptors */
     2 nFileCount       type ULONG,/* # of file attachments */
     2 lpFiles          type lpMapiFileDesc;/* Attachment descriptors */
 define alias lpMapiMessage handle MapiMessage;

 dcl MAPI_UNREAD            type FLAGS value('00000001'xu);
 dcl MAPI_RECEIPT_REQUESTED type FLAGS value('00000002'xu);
 dcl MAPI_SENT              type FLAGS value('00000004'xu);


 /*  Entry points. */

 /* flFlags values for Simple MAPI entry points. All documented flags */
 /* are shown for each call. Duplicates are commented out but remain  */
 /* present for every call.                                           */

 /* MAPILogon() flags.       */

 dcl MAPI_LOGON_UI        type FLAGS value('00000001'xu);
     /* Display logon UI             */
 %dcl #MAPI_PASSWORD_UI char ext;
 %if #MAPI_PASSWORD_UI = '' %then %do;
 %#MAPI_PASSWORD_UI = 'Y';
 dcl MAPI_PASSWORD_UI     type FLAGS value('00020000'xu);
     /* prompt for password only     */
 %end;
 dcl MAPI_NEW_SESSION     type FLAGS value('00000002'xu);
     /* Don't use shared session     */
 dcl MAPI_FORCE_DOWNLOAD  type FLAGS value('00001000'xu);
     /* Get new mail before return   */
 dcl MAPI_EXTENDED        type FLAGS value('00000020'xu);
     /* Extended MAPI Logon          */

 /* MAPISendMail() flags.    */

 /* dcl MAPI_LOGON_UI        type FLAGS value('00000001'xu) */
 /*     /- Display logon UI             -/                     */
 /* dcl MAPI_NEW_SESSION     type FLAGS value('00000002'xu) */
 /*     /- Don't use shared session     -/                     */

 %dcl #MAPI_DIALOG char ext;
 %if #MAPI_DIALOG = '' %then %do;
 %#MAPI_DIALOG = 'Y';
 dcl MAPI_DIALOG          type FLAGS value('00000008'xu);
     /* Display a send note UI       */
 %end;
 /* dcl MAPI_USE_DEFAULT     type FLAGS value('00000040'xu) */
 /*     /- Use default profile in logon -/                     */

 /* MAPIFindNext() flags.    */

 dcl MAPI_UNREAD_ONLY     type FLAGS value('00000020'xu);
     /* Only unread messages         */
 dcl MAPI_GUARANTEE_FIFO  type FLAGS value('00000100'xu);
     /* use date order               */
 dcl MAPI_LONG_MSGID      type FLAGS value('00004000'xu);
     /* allow 512 char returned ID   */

 /* MAPIReadMail() flags.    */

 dcl MAPI_PEEK            type FLAGS value('00000080'xu);
     /* Do not mark as read.         */
 dcl MAPI_SUPPRESS_ATTACH type FLAGS value('00000800'xu);
     /* header + body, no files      */
 dcl MAPI_ENVELOPE_ONLY   type FLAGS value('00000040'xu);
     /* Only header information      */
 dcl MAPI_BODY_AS_FILE    type FLAGS value('00000200'xu);

 /* MAPISaveMail() flags.    */

 /* dcl MAPI_LOGON_UI        type FLAGS value('00000001'xu) */
 /*     /- Display logon UI             -/                     */
 /* dcl MAPI_NEW_SESSION     type FLAGS value('00000002'xu) */
 /*     /- Don't use shared session     -/                     */
 /* dcl MAPI_LONG_MSGID      type FLAGS value('00004000'xu) */
 /*     /- allow 512 char returned ID   -/                      */

 /* MAPIAddress() flags.     */

 /* dcl MAPI_LOGON_UI        type FLAGS value('00000001'xu) */
 /*     /- Display logon UI             -/                     */
 /* dcl MAPI_NEW_SESSION     type FLAGS value('00000002'xu) */
 /*     /- Don't use shared session     -/                     */

 /* MAPIDetails() flags.     */

 /* dcl MAPI_LOGON_UI        type FLAGS value('00000001'xu) */
 /*     /- Display logon UI             -/                     */
 /* dcl MAPI_NEW_SESSION     type FLAGS value('00000002'xu) */
 /*     /- Don't use shared session     -/                     */
 dcl MAPI_AB_NOMODIFY     type FLAGS value('00000400'xu);
     /* Don't allow mods of AB entries */

 /* MAPIResolveName() flags. */

 /* dcl MAPI_LOGON_UI        type FLAGS value('00000001'xu) */
 /*     /- Display logon UI             -/                     */
 /* dcl MAPI_NEW_SESSION     type FLAGS value('00000002'xu) */
 /*     /- Don't use shared session     -/                     */
 /* dcl MAPI_DIALOG          type FLAGS value('00000008'xu) */
 /*     /- Display a send note UI       -/                     */
 /* dcl MAPI_AB_NOMODIFY     type FLAGS value('00000400'xu) */
 /*     /- Don't allow mods of AB entries -/                   */


 define alias LPMAPILOGON
        entry (type ULONG,
               char (*) varz byaddr optional,
               char (*) varz byaddr optional,
               type FLAGS,
               type ULONG,
               type LHANDLE byaddr)
        returns (type ULONG byvalue optional)
        limited PASCAL;

 dcl MAPILogon
     entry (type ULONG,
            char (*) varz byaddr optional,
            char (*) varz byaddr optional,
            type FLAGS,
            type ULONG,
            type LHANDLE byaddr)
     returns (type ULONG byvalue optional)
     SMAPI_API external('MAPI32/MAPILogon');


 define alias LPMAPILOGOFF
        entry (type LHANDLE,
               type ULONG,
               type FLAGS,
               type ULONG)
        returns (type ULONG byvalue optional)
        limited PASCAL;

 dcl MAPILogoff
     entry (type LHANDLE,
            type ULONG,
            type FLAGS,
            type ULONG)
     returns (type ULONG byvalue optional)
     SMAPI_API external('MAPI32/MAPILogoff');


 define alias LPMAPISENDMAIL
        entry (type LHANDLE,
               type ULONG,
               type MapiMessage byaddr,
               type FLAGS,
               type ULONG)
        returns (type ULONG byvalue optional)
        limited PASCAL;

 dcl MAPISendMail
     entry (type LHANDLE,
            type ULONG,
            type MapiMessage byaddr,
            type FLAGS,
            type ULONG)
     returns (type ULONG byvalue optional)
     SMAPI_API external('MAPI32/MAPISendMail');


 define alias LPMAPISENDDOCUMENTS
        entry (type ULONG,
               char (*) varz byaddr optional,
               char (*) varz byaddr optional,
               char (*) varz byaddr optional,
               type ULONG)
        returns (type ULONG byvalue optional)
        limited PASCAL;

 dcl MAPISendDocuments
     entry (type ULONG,
            char (*) varz byaddr optional,
            char (*) varz byaddr optional,
            char (*) varz byaddr optional,
            type ULONG)
     returns (type ULONG byvalue optional)
     SMAPI_API external('MAPI32/MAPISendDocuments');


 define alias LPMAPIFINDNEXT
        entry (type LHANDLE,
               type ULONG,
               char (*) varz byaddr optional,
               char (*) varz byaddr optional,
               type FLAGS,
               type ULONG,
               char (*) varz byaddr)
        returns (type ULONG byvalue optional)
        limited PASCAL;

 dcl MAPIFindNext
     entry (type LHANDLE,
            type ULONG,
            char (*) varz byaddr optional,
            char (*) varz byaddr optional,
            type FLAGS,
            type ULONG,
            char (*) varz byaddr)
     returns (type ULONG byvalue optional)
     SMAPI_API external('MAPI32/MAPIFindNext');


 define alias LPMAPIREADMAIL
        entry (type LHANDLE,
               type ULONG,
               char (*) varz byaddr,
               type FLAGS,
               type ULONG,
               type lpMapiMessage byaddr)
        returns (type ULONG byvalue optional)
        limited PASCAL;

 dcl MAPIReadMail
     entry (type LHANDLE,
            type ULONG,
            char (*) varz byaddr,
            type FLAGS,
            type ULONG,
            type lpMapiMessage byaddr)
     returns (type ULONG byvalue optional)
     SMAPI_API external('MAPI32/MAPIReadMail');


 define alias LPMAPISAVEMAIL
        entry (type LHANDLE,
               type ULONG,
               type MapiMessage byaddr,
               type FLAGS,
               type ULONG,
               char (*) varz byaddr)
        returns (type ULONG byvalue optional)
        limited PASCAL;

 dcl MAPISaveMail
     entry (type LHANDLE,
            type ULONG,
            type MapiMessage byaddr,
            type FLAGS,
            type ULONG,
            char (*) varz byaddr)
     returns (type ULONG byvalue optional)
     SMAPI_API external('MAPI32/MAPISaveMail');


 define alias LPMAPIDELETEMAIL
        entry (type LHANDLE,
               type ULONG,
               char (*) varz byaddr,
               type FLAGS,
               type ULONG)
        returns (type ULONG byvalue optional)
        limited PASCAL;

 dcl MAPIDeleteMail
     entry (type LHANDLE,
            type ULONG,
            char (*) varz byaddr,
            type FLAGS,
            type ULONG)
     returns (type ULONG byvalue optional)
     SMAPI_API external('MAPI32/MAPIDeleteMail');

 define alias LPMAPIFREEBUFFER
        entry (type LPVOID)
        returns (type ULONG byvalue optional)
        limited PASCAL;

 dcl MAPIFreeBuffer
     entry (type LPVOID)
     returns (type ULONG byvalue optional)
     SMAPI_API external('MAPI32/MAPIFreeBuffer');


 define alias LPMAPIADDRESS
        entry  (type LHANDLE,
                type ULONG,
                char (*) varz byaddr optional,
                type ULONG,
                char (*) varz byaddr optional,
                type ULONG,
                (*) type MapiRecipDesc byaddr optional,
                type FLAGS,
                type ULONG,
                type ULONG byaddr optional,
                type lpMapiRecipDesc byaddr)
             returns (type ULONG byvalue optional)
        limited PASCAL;

 dcl MAPIAddress
     entry (type LHANDLE,
            type ULONG,
            char (*) varz byaddr optional,
            type ULONG,
            char (*) varz byaddr optional,
            type ULONG,
            (*) type MapiRecipDesc byaddr optional,
            type FLAGS,
            type ULONG,
            type ULONG byaddr optional,
            type lpMapiRecipDesc byaddr)
      returns (type ULONG byvalue optional)
      SMAPI_API external('MAPI32/MAPIAddress');


 define alias LPMAPIDETAILS
        entry (type LHANDLE,
               type ULONG,
               type MapiRecipDesc byaddr,
               type FLAGS,
               type ULONG)
        returns (type ULONG byvalue optional)
        limited PASCAL;

 dcl MAPIDetails
     entry (type LHANDLE,
            type ULONG,
            type MapiRecipDesc byaddr,
            type FLAGS,
            type ULONG)
     returns (type ULONG byvalue optional)
     SMAPI_API external('MAPI32/MAPIDetails');


 define alias LPMAPIRESOLVENAME
        entry (type LHANDLE,
               type ULONG,
               char (*) varz byaddr,
               type FLAGS,
               type ULONG,
               type lpMapiRecipDesc byaddr)
        returns (type ULONG byvalue optional)
        limited PASCAL;

 dcl MAPIResolveName
     entry (type LHANDLE,
            type ULONG,
            char (*) varz byaddr,
            type FLAGS,
            type ULONG,
            type lpMapiRecipDesc byaddr)
     returns (type ULONG byvalue optional)
     SMAPI_API external('MAPI32/MAPIResolveName');

 %dcl #SUCCESS_SUCCESS char ext;
 %if #SUCCESS_SUCCESS = '' %then %do;
 %#SUCCESS_SUCCESS = 'Y';
 dcl SUCCESS_SUCCESS                 fixed bin(31) value(0);
 %end;
 dcl MAPI_USER_ABORT                 fixed bin(31) value(1);
 dcl MAPI_E_USER_ABORT               fixed bin(31)
     value(MAPI_USER_ABORT);
 dcl MAPI_E_FAILURE                  fixed bin(31) value(2);
 dcl MAPI_E_LOGON_FAILURE            fixed bin(31) value(3);
 dcl MAPI_E_LOGIN_FAILURE            fixed bin(31)
     value(MAPI_E_LOGON_FAILURE);
 dcl MAPI_E_DISK_FULL                fixed bin(31) value(4);
 dcl MAPI_E_INSUFFICIENT_MEMORY      fixed bin(31) value(5);
 dcl MAPI_E_ACCESS_DENIED            fixed bin(31) value(6);
 dcl MAPI_E_TOO_MANY_SESSIONS        fixed bin(31) value(8);
 dcl MAPI_E_TOO_MANY_FILES           fixed bin(31) value(9);
 dcl MAPI_E_TOO_MANY_RECIPIENTS      fixed bin(31) value(10);
 dcl MAPI_E_ATTACHMENT_NOT_FOUND     fixed bin(31) value(11);
 dcl MAPI_E_ATTACHMENT_OPEN_FAILURE  fixed bin(31) value(12);
 dcl MAPI_E_ATTACHMENT_WRITE_FAILURE fixed bin(31) value(13);
 dcl MAPI_E_UNKNOWN_RECIPIENT        fixed bin(31) value(14);
 dcl MAPI_E_BAD_RECIPTYPE            fixed bin(31) value(15);
 dcl MAPI_E_NO_MESSAGES              fixed bin(31) value(16);
 dcl MAPI_E_INVALID_MESSAGE          fixed bin(31) value(17);
 dcl MAPI_E_TEXT_TOO_LARGE           fixed bin(31) value(18);
 dcl MAPI_E_INVALID_SESSION          fixed bin(31) value(19);
 dcl MAPI_E_TYPE_NOT_SUPPORTED       fixed bin(31) value(20);
 dcl MAPI_E_AMBIGUOUS_RECIPIENT      fixed bin(31) value(21);
 dcl MAPI_E_AMBIG_RECIP              fixed bin(31)
     value(MAPI_E_AMBIGUOUS_RECIPIENT);
 dcl MAPI_E_MESSAGE_IN_USE           fixed bin(31) value(22);
 dcl MAPI_E_NETWORK_FAILURE          fixed bin(31) value(23);
 dcl MAPI_E_INVALID_EDITFIELDS       fixed bin(31) value(24);
 dcl MAPI_E_INVALID_RECIPS           fixed bin(31) value(25);
 dcl MAPI_E_NOT_SUPPORTED            fixed bin(31) value(26);


 %dcl MAPIX_H char ext;
 %if MAPIX_H ^= '' %then %do;
 /* Maps from a simple mapi session to */
 /* a MAPI 1.0 extended mapi session */
 dcl ScMAPIXFromSMAPI
     entry (type LHANDLE,
            type ULONG,
            type LPCIID,
            type LPMAPISESSION byaddr)
     STDMETHODIMP_(type SCODE);
 %end; /* MAPIX_H */

 %end; /* MAPI_H */
