 /* ++                                                                */
 /*                                                                   */
 /* Copyright (c) 1996-1999 Microsoft Corporation                     */
 /*                                                                   */
 /* Module Name:                                                      */
 /*                                                                   */
 /*   mq.cpy                                                          */
 /*                                                                   */
 /* Abstract:                                                         */
 /*                                                                   */
 /*   Master include file for Message Queue applications              */
 /*                                                                   */
 /* --                                                                */

 %dcl #_MQ_H__ char ext;
 %if #_MQ_H__ = '' %then %do;
 %#_MQ_H__ = 'Y';

 %dcl #_wtypes_h__ char ext;
 %if #_wtypes_h__ = '' %then %do;
 %include wtypes;
 %end;

 /*  Clients that need transactions should include transact.cpy       */
 /*  before mq.cpy                                                    */
 /*  For the clients not needing transactions - ITransaction is opaque*/

 %dcl #_transact_h__ char ext;
 %if #_transact_h__ = '' %then %do;
 define structure
   1 ITransaction;
 %end;

 %dcl TYPEDEF_CA entry;
 %TYPEDEF_CA: proc (type, name) returns (char);
   /* for primitive types */
   dcl (type, name) char;
   return ('define structure ' ||
             '1 ' || name || ', ' ||
               '2 cElems type ULONG, ' ||
               '2 pElems pointer; /* to ' || type || ' */');
 %end TYPEDEF_CA;

 %dcl TYPEDEF_CA2 entry;
 %TYPEDEF_CA2: proc (type, name) returns (char);
   /* for structure types */
   dcl (type, name) char;
   return ('define structure ' ||
             '1 ' || name || ', ' ||
               '2 cElems type ULONG, ' ||
               '2 pElems handle ' || type || ';');
 %end TYPEDEF_CA2;


 %dcl #tagCAUB_DEFINED char ext;
 %if #tagCAUB_DEFINED = '' %then %do;
 %#tagCAUB_DEFINED = 'Y';
 TYPEDEF_CA (fixed bin(8) unsigned, CAUB);
 %end;

 %dcl #tagCAI_DEFINED char ext;
 %if #tagCAI_DEFINED = '' %then %do;
 %#tagCAI_DEFINED = 'Y';
 TYPEDEF_CA (fixed bin(15), CAI);
 %end;

 %dcl #tagCAUI_DEFINED char ext;
 %if #tagCAUI_DEFINED = '' %then %do;
 %#tagCAUI_DEFINED = 'Y';
 TYPEDEF_CA (type USHORT, CAUI);
 %end;

 %dcl #tagCAL_DEFINED char ext;
 %if #tagCAL_DEFINED = '' %then %do;
 %#tagCAL_DEFINED = 'Y';
 TYPEDEF_CA (fixed bin(31), CAL);
 %end;

 %dcl #tagCAUL_DEFINED char ext;
 %if #tagCAUL_DEFINED = '' %then %do;
 %#tagCAUL_DEFINED = 'Y';
 TYPEDEF_CA (type ULONG, CAUL);
 %end;

 %dcl #tagCACLSID_DEFINED char ext;
 %if #tagCACLSID_DEFINED = '' %then %do;
 %#tagCACLSID_DEFINED = 'Y';
 TYPEDEF_CA2 (CLSID, CACLSID);
 %end;

 %dcl #tagCABSTR_DEFINED char ext;
 %if #tagCABSTR_DEFINED = '' %then %do;
 %#tagCABSTR_DEFINED = 'Y';
 TYPEDEF_CA (type LPOLESTR, CABSTR);
 %end;

 %dcl #tagCALPWSTR_DEFINED char ext;
 %if #tagCALPWSTR_DEFINED = '' %then %do;
 %#tagCALPWSTR_DEFINED = 'Y';
 TYPEDEF_CA (type LPWSTR, CALPWSTR);
 %end;

 %dcl #tagCAMQPROPVARIANT_DEFINED char ext;
 %if #tagCAMQPROPVARIANT_DEFINED = '' %then %do;
 %#tagCAMQPROPVARIANT_DEFINED = 'Y';
 TYPEDEF_CA2 (MQPROPVARIANT, CAMQPROPVARIANT);
 %end;


 define structure
   1 MQPROPVARIANT,
     2 vt          type VARTYPE,
     2 wReserved1  type WORD,
     2 wReserved2  type WORD,
     2 wReserved3  type WORD,
     2 * union,                      /*[switch_is((unsigned short)vt)]*/
                                       /*[case(VT_EMPTY, VT_NULL)];*/
       3 bVal      type UCHAR,         /*[case(VT_UI1)]*/
       3 iVal      fixed bin(15),      /*[case(VT_I2)]*/
       3 uiVal     type USHORT,        /*[case(VT_UI2)]*/
       3 boolVal   type VARIANT_BOOL,  /*[case(VT_BOOL)]*/
       3 lVal      fixed bin(31),      /*[case(VT_I4)]*/
       3 ulVal     type ULONG,         /*[case(VT_UI4)]*/
       3 scode     type SCODE,         /*[case(VT_HRESULT)]*/
       3 date      type DATE,          /*[case(VT_DATE)]*/
       3 puuid     handle CLSID,       /*[case(VT_CLSID)]*/
       3 blob      type BLOB,          /*[case(VT_BLOB)]*/
       3 bstrVal   type LPOLESTR,      /*[case(VT_BSTR)]*/
       3 pszVal    type LPSTR,         /*[case(VT_LPSTR)]*/
       3 pwszVal   type LPWSTR,        /*[case(VT_LPWSTR)]*/
       3 caub      type CAUB,          /*[case(VT_VECTOR | VT_UI1)]*/
       3 cai       type CAI,           /*[case(VT_VECTOR | VT_I2)]*/
       3 caui      type CAUI,          /*[case(VT_VECTOR | VT_UI2)]*/
       3 cal       type CAL,           /*[case(VT_VECTOR | VT_I4)]*/
       3 caul      type CAUL,          /*[case(VT_VECTOR | VT_UI4)]*/
       3 cauuid    type CACLSID,       /*[case(VT_VECTOR | VT_CLSID)]*/
       3 cabstr    type CABSTR,        /*[case(VT_VECTOR | VT_BSTR)]*/
       3 calpwstr  type CALPWSTR,      /*[case(VT_VECTOR | VT_LPWSTR)]*/
       3 capropvar type CAMQPROPVARIANT;/*[case(VT_VARIANT)]*/


 dcl PRLT fixed bin(31) value(0);
 dcl PRLE fixed bin(31) value(1);
 dcl PRGT fixed bin(31) value(2);
 dcl PRGE fixed bin(31) value(3);
 dcl PREQ fixed bin(31) value(4);
 dcl PRNE fixed bin(31) value(5);

 define structure
   1 MQPROPERTYRESTRICTION,
     2 rel   type ULONG,
     2 prop  type PROPID,
     2 prval type MQPROPVARIANT;

 define structure
   1 MQRESTRICTION,
     2 cRes      type ULONG,
     2 paPropRes handle MQPROPERTYRESTRICTION; /* [size_is] */

 define structure
   1 MQCOLUMNSET,
     2 cCol type ULONG,
     2 aCol pointer; /* to PROPID */ /* [size_is] */

 dcl QUERY_SORTASCEND  fixed bin(31) value(0);
 dcl QUERY_SORTDESCEND fixed bin(31) value(1);

 define structure
   1 MQSORTKEY,
     2 propColumn type PROPID,
     2 dwOrder    type ULONG;

 define structure
   1 MQSORTSET,
     2 cCol type ULONG,
     2 aCol handle MQSORTKEY; /* [size_is] */


 dcl MQ_MAX_Q_NAME_LEN   fixed bin(31) value(124);
     /* Maximal WCHAR length of a queue name. */
 dcl MQ_MAX_Q_LABEL_LEN   fixed bin(31) value(124);
 dcl MQ_MAX_MSG_LABEL_LEN fixed bin(31) value(250);

 define alias QUEUEHANDLE type HANDLE;


 define alias MSGPROPID   type PROPID;
 define alias QUEUEPROPID type PROPID;
 define alias QMPROPID    type PROPID;

 define structure
   1 MQMSGPROPS,
     2 cProp    type DWORD,
     2 aPropID  pointer,              /* to (cProp) MSGPROPID */
     2 aPropVar handle MQPROPVARIANT, /* to (cProp) MQPROPVARIANT */
     2 aStatus  pointer;              /* to (cProp) HRESULT */

 define structure
   1 MQQUEUEPROPS,
     2 cProp    type DWORD,
     2 aPropID  pointer,              /* to (cProp) QUEUEPROPID */
     2 aPropVar handle MQPROPVARIANT, /* to (cProp) MQPROPVARIANT */
     2 aStatus  pointer;              /* to (cProp) HRESULT */

 define structure
   1 MQQMPROPS,
     2 cProp    type DWORD,
     2 aPropID  pointer,              /* to (cProp) QMPROPID */
     2 aPropVar handle MQPROPVARIANT, /* to (cProp) MQPROPVARIANT */
     2 aStatus  pointer;              /* to (cProp) HRESULT */


 /*********************************************************************/
 /*  API FLAGS                                                        */
 /*********************************************************************/


 /*  MQOpenQueue - Access values */
 dcl MQ_RECEIVE_ACCESS      type DWORD value('00000001'xu);
 dcl MQ_SEND_ACCESS         type DWORD value('00000002'xu);
 dcl MQ_PEEK_ACCESS         type DWORD value('00000020'xu);


 /*  MQOpenQueue - Share values */
 dcl MQ_DENY_NONE           type DWORD value('00000000'xu);
 dcl MQ_DENY_RECEIVE_SHARE  type DWORD value('00000001'xu);

 /*  MQReceiveMessage - Action values */
 dcl MQ_ACTION_RECEIVE      type DWORD value('00000000'xu);
 dcl MQ_ACTION_PEEK_CURRENT type DWORD value('80000000'xu);
 dcl MQ_ACTION_PEEK_NEXT    type DWORD value('80000001'xu);

 /* MQSendMessage,  MQReceiveMessage:  special cases for the */
 /* transaction parameter                                    */

 %if 1 = 1 %then %do;
 dcl MQ_NO_TRANSACTION      handle ITransaction value(ptrvalue(0));
 dcl MQ_MTS_TRANSACTION     handle ITransaction value(ptrvalue(1));
 dcl MQ_XA_TRANSACTION      handle ITransaction value(ptrvalue(2));
 dcl MQ_SINGLE_MESSAGE      handle ITransaction value(ptrvalue(3));
 %end; %else %do;
 /* Work around - until bug fixed: */
 %dcl MQ_NO_TRANSACTION     char ext;
 %MQ_NO_TRANSACTION  = 'sysnull()';
 %dcl MQ_MTS_TRANSACTION    char ext;
 %MQ_MTS_TRANSACTION = 'bind (: ITransaction, ptrvalue(1) :)';
 %dcl MQ_XA_TRANSACTION     char ext;
 %MQ_XA_TRANSACTION  = 'bind (: ITransaction, ptrvalue(2) :)';
 %dcl MQ_SINGLE_MESSAGE     char ext;
 %MQ_SINGLE_MESSAGE  = 'bind (: ITransaction, ptrvalue(3) :)';
 %end;

 /*********************************************************************/
 /*  PRIORITY LIMITS */
 /*********************************************************************/

 /*  Message priorities */
 dcl MQ_MIN_PRIORITY        type DWORD value(0);
     /* Minimal message priority */
 dcl MQ_MAX_PRIORITY        type DWORD value(7);
     /* Maximal message priority */


 /*********************************************************************/
 /*  MESSAGE PROPERTIES */
 /*********************************************************************/
 dcl PROPID_M_BASE              fixed bin(31) value(0);
 dcl PROPID_M_CLASS             fixed bin(31) value(PROPID_M_BASE + 1);
     /* VT_UI2           */
 dcl PROPID_M_MSGID             fixed bin(31) value(PROPID_M_BASE + 2);
     /* VT_UI1|VT_VECTOR */
 dcl PROPID_M_CORRELATIONID     fixed bin(31) value(PROPID_M_BASE + 3);
     /* VT_UI1|VT_VECTOR */
 dcl PROPID_M_PRIORITY          fixed bin(31) value(PROPID_M_BASE + 4);
     /* VT_UI1           */
 dcl PROPID_M_DELIVERY          fixed bin(31) value(PROPID_M_BASE + 5);
     /* VT_UI1           */
 dcl PROPID_M_ACKNOWLEDGE       fixed bin(31) value(PROPID_M_BASE + 6);
     /* VT_UI1           */
 dcl PROPID_M_JOURNAL           fixed bin(31) value(PROPID_M_BASE + 7);
     /* VT_UI1           */
 dcl PROPID_M_APPSPECIFIC       fixed bin(31) value(PROPID_M_BASE + 8);
     /* VT_UI4           */
 dcl PROPID_M_BODY              fixed bin(31) value(PROPID_M_BASE + 9);
     /* VT_UI1|VT_VECTOR */
 dcl PROPID_M_BODY_SIZE         fixed bin(31) value(PROPID_M_BASE + 10);
     /* VT_UI4           */
 dcl PROPID_M_LABEL             fixed bin(31) value(PROPID_M_BASE + 11);
     /* VT_LPWSTR        */
 dcl PROPID_M_LABEL_LEN         fixed bin(31) value(PROPID_M_BASE + 12);
     /* VT_UI4           */
 dcl PROPID_M_TIME_TO_REACH_QUEUE
                                fixed bin(31) value(PROPID_M_BASE + 13);
     /* VT_UI4           */
 dcl PROPID_M_TIME_TO_BE_RECEIVED
                                fixed bin(31) value(PROPID_M_BASE + 14);
     /* VT_UI4           */
 dcl PROPID_M_RESP_QUEUE        fixed bin(31) value(PROPID_M_BASE + 15);
     /* VT_LPWSTR        */
 dcl PROPID_M_RESP_QUEUE_LEN    fixed bin(31) value(PROPID_M_BASE + 16);
     /* VT_UI4           */
 dcl PROPID_M_ADMIN_QUEUE       fixed bin(31) value(PROPID_M_BASE + 17);
     /* VT_LPWSTR        */
 dcl PROPID_M_ADMIN_QUEUE_LEN   fixed bin(31) value(PROPID_M_BASE + 18);
     /* VT_UI4           */
 dcl PROPID_M_VERSION           fixed bin(31) value(PROPID_M_BASE + 19);
     /* VT_UI4           */
 dcl PROPID_M_SENDERID          fixed bin(31) value(PROPID_M_BASE + 20);
     /* VT_UI1|VT_VECTOR */
 dcl PROPID_M_SENDERID_LEN      fixed bin(31) value(PROPID_M_BASE + 21);
     /* VT_UI4           */
 dcl PROPID_M_SENDERID_TYPE     fixed bin(31) value(PROPID_M_BASE + 22);
     /* VT_UI4           */
 dcl PROPID_M_PRIV_LEVEL        fixed bin(31) value(PROPID_M_BASE + 23);
     /* VT_UI4           */
 dcl PROPID_M_AUTH_LEVEL        fixed bin(31) value(PROPID_M_BASE + 24);
     /* VT_UI4           */
 dcl PROPID_M_AUTHENTICATED     fixed bin(31) value(PROPID_M_BASE + 25);
     /* VT_UI1           */
 dcl PROPID_M_HASH_ALG          fixed bin(31) value(PROPID_M_BASE + 26);
     /* VT_UI4           */
 dcl PROPID_M_ENCRYPTION_ALG    fixed bin(31) value(PROPID_M_BASE + 27);
     /* VT_UI4           */
 dcl PROPID_M_SENDER_CERT       fixed bin(31) value(PROPID_M_BASE + 28);
     /* VT_UI1|VT_VECTOR */
 dcl PROPID_M_SENDER_CERT_LEN   fixed bin(31) value(PROPID_M_BASE + 29);
     /* VT_UI4           */
 dcl PROPID_M_SRC_MACHINE_ID    fixed bin(31) value(PROPID_M_BASE + 30);
     /* VT_CLSID         */
 dcl PROPID_M_SENTTIME          fixed bin(31) value(PROPID_M_BASE + 31);
     /* VT_UI4           */
 dcl PROPID_M_ARRIVEDTIME       fixed bin(31) value(PROPID_M_BASE + 32);
     /* VT_UI4           */
 dcl PROPID_M_DEST_QUEUE        fixed bin(31) value(PROPID_M_BASE + 33);
     /* VT_LPWSTR        */
 dcl PROPID_M_DEST_QUEUE_LEN    fixed bin(31) value(PROPID_M_BASE + 34);
     /* VT_UI4           */
 dcl PROPID_M_EXTENSION         fixed bin(31) value(PROPID_M_BASE + 35);
     /* VT_UI1|VT_VECTOR */
 dcl PROPID_M_EXTENSION_LEN     fixed bin(31) value(PROPID_M_BASE + 36);
     /* VT_UI4           */
 dcl PROPID_M_SECURITY_CONTEXT  fixed bin(31) value(PROPID_M_BASE + 37);
     /* VT_UI4           */
 dcl PROPID_M_CONNECTOR_TYPE    fixed bin(31) value(PROPID_M_BASE + 38);
     /* VT_CLSID         */
 dcl PROPID_M_XACT_STATUS_QUEUE fixed bin(31) value(PROPID_M_BASE + 39);
     /* VT_LPWSTR        */
 dcl PROPID_M_XACT_STATUS_QUEUE_LEN
                                fixed bin(31) value(PROPID_M_BASE + 40);
     /* VT_UI4           */
 dcl PROPID_M_TRACE             fixed bin(31) value(PROPID_M_BASE + 41);
     /* VT_UI1           */
 dcl PROPID_M_BODY_TYPE         fixed bin(31) value(PROPID_M_BASE + 42);
     /* VT_UI4           */
 dcl PROPID_M_DEST_SYMM_KEY     fixed bin(31) value(PROPID_M_BASE + 43);
     /* VT_UI1|VT_VECTOR */
 dcl PROPID_M_DEST_SYMM_KEY_LEN fixed bin(31) value(PROPID_M_BASE + 44);
     /* VT_UI4           */
 dcl PROPID_M_SIGNATURE         fixed bin(31) value(PROPID_M_BASE + 45);
     /* VT_UI1|VT_VECTOR */
 dcl PROPID_M_SIGNATURE_LEN     fixed bin(31) value(PROPID_M_BASE + 46);
     /* VT_UI4           */
 dcl PROPID_M_PROV_TYPE         fixed bin(31) value(PROPID_M_BASE + 47);
     /* VT_UI4           */
 dcl PROPID_M_PROV_NAME         fixed bin(31) value(PROPID_M_BASE + 48);
     /* VT_LPWSTR        */
 dcl PROPID_M_PROV_NAME_LEN     fixed bin(31) value(PROPID_M_BASE + 49);
     /* VT_UI4           */


 /* Message Property Size */
 dcl PROPID_M_MSGID_SIZE         fixed bin(31) value(20);
 dcl PROPID_M_CORRELATIONID_SIZE fixed bin(31) value(20);


 /*********************************************************************/
 /*  MESSAGE CLASS VALUES                                             */
 /*********************************************************************/

 /*  Message Class Values are 16 bits layed out as follows:           */
 /*                                                                   */
 /*   1 1 1 1 1 1                                                     */
 /*   5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0                                 */
 /*  +-+-+-----------+---------------+                                */
 /*  |S|R| Reserved  |  Class code   |                                */
 /*  +-+-+-----------+---------------+                                */
 /*                                                                   */
 /*  where                                                            */
 /*                                                                   */
 /*      S - is the severity flag                                     */
 /*          0 - Normal Message/Positive Acknowledgment (ACK)         */
 /*          1 - Negative Acknowledgment (NACK)                       */
 /*                                                                   */
 /*      R - is the receive flag                                      */
 /*          0 - Arrival ACK/NACK                                     */
 /*          1 - Receive ACK/NACK                                     */

 %dcl MQCLASS_CODE entry;
 %MQCLASS_CODE: proc (s, r, code) returns (char);
   dcl (s, r, code) char;
   return ('iunsigned (ior (raise2 (' || s || ', 15), ' ||
                          'raise2 (' || r || ', 14), ' ||
                           code || '))');
 %end MQCLASS_CODE;

 MQCLASS_NACK: proc (c) returns (type USHORT byvalue)
               options (inline reorder) internal;
   dcl c type USHORT byvalue;
   dcl iand builtin;
   return (iand (c, '8000'xu));
 end MQCLASS_NACK;

 MQCLASS_RECEIVE: proc (c) returns (type USHORT byvalue)
                  options (inline reorder) internal;
   dcl c type USHORT byvalue;
   dcl iand builtin;
   return (iand (c, '4000'xu));
 end MQCLASS_RECEIVE;


 /*  Normal message */
 dcl MQMSG_CLASS_NORMAL
     type USHORT value(MQCLASS_CODE (0, 0, '00'xu));

 /*  Report message */
 dcl MQMSG_CLASS_REPORT
     type USHORT value(MQCLASS_CODE (0, 0, '01'xu));

 /*  Arrival acknowledgment.                       */
 /*  The message has reached the destination queue */
 dcl MQMSG_CLASS_ACK_REACH_QUEUE
     type USHORT value(MQCLASS_CODE (0, 0, '02'xu));

 /*  Receive acknowledgment.                          */
 /*  The message has been received by the application */
 dcl MQMSG_CLASS_ACK_RECEIVE
     type USHORT value(MQCLASS_CODE (0, 1, '00'xu));


 /*----------------------------------------------- */

 /*  Negative arrival acknowledgments */


 /*  Destination queue can not be reached, */
 /*  the queue may have been deleted       */
 dcl MQMSG_CLASS_NACK_BAD_DST_Q
     type USHORT value(MQCLASS_CODE (1, 0, '00'xu));

 /*  The message was purged before reaching the destination queue */
 dcl MQMSG_CLASS_NACK_PURGED
     type USHORT value(MQCLASS_CODE (1, 0, '01'xu));

 /*  Time to reach queue has expired */
 dcl MQMSG_CLASS_NACK_REACH_QUEUE_TIMEOUT
     type USHORT value(MQCLASS_CODE (1, 0, '02'xu));

 /*  The message has exceeded the queue quota */
 dcl MQMSG_CLASS_NACK_Q_EXCEED_QUOTA
     type USHORT value(MQCLASS_CODE (1, 0, '03'xu));

 /*  The sender does not have send access rights on the queue. */
 dcl MQMSG_CLASS_NACK_ACCESS_DENIED
     type USHORT value(MQCLASS_CODE (1, 0, '04'xu));

 /*  The message hop count exceeded */
 dcl MQMSG_CLASS_NACK_HOP_COUNT_EXCEEDED
     type USHORT value(MQCLASS_CODE (1, 0, '05'xu));

 /*  The message signature is bad.           */
 /*  The message could not be authenticated. */
 dcl MQMSG_CLASS_NACK_BAD_SIGNATURE
     type USHORT value(MQCLASS_CODE (1, 0, '06'xu));

 /*  The message could not be decrypted. */
 dcl MQMSG_CLASS_NACK_BAD_ENCRYPTION
     type USHORT value(MQCLASS_CODE (1, 0, '07'xu));

 /*  The message could not be encrypted for the destination. */
 dcl MQMSG_CLASS_NACK_COULD_NOT_ENCRYPT
     type USHORT value(MQCLASS_CODE (1, 0, '08'xu));

 /*  The message was sent to a non-transactional queue */
 /*  within a transaction.                             */
 dcl MQMSG_CLASS_NACK_NOT_TRANSACTIONAL_Q
     type USHORT value(MQCLASS_CODE (1, 0, '09'xu));

 /*  The message was sent to a transactional queue */
 /*  not within a transaction.                     */
 dcl MQMSG_CLASS_NACK_NOT_TRANSACTIONAL_MSG
     type USHORT value(MQCLASS_CODE (1, 0, '0A'xu));

 /*  The requested crypto provider for encryption */
 /*  is not supported by destination.             */
 dcl MQMSG_CLASS_NACK_UNSUPPORTED_CRYPTO_PROVIDER
     type USHORT value(MQCLASS_CODE (1, 0, '0B'xu));


 /*----------------------------------------------- */

 /*  Negative receive acknowledgments */


 /*  The queue was deleted, after the message has arrived */
 dcl MQMSG_CLASS_NACK_Q_DELETED
     type USHORT value(MQCLASS_CODE (1, 1, '00'xu));

 /*  The message was purged at the destination queue */
 dcl MQMSG_CLASS_NACK_Q_PURGED
     type USHORT value(MQCLASS_CODE (1, 1, '01'xu));

 /*  Time to receive has expired, while the message is in the queue */
 /*  (info from the destination)                                    */
 dcl MQMSG_CLASS_NACK_RECEIVE_TIMEOUT
     type USHORT value(MQCLASS_CODE (1, 1, '02'xu));

 /*  Time to receive has expired, while the message is in the queue */
 /*  (generated by sender locally)                                  */
 dcl MQMSG_CLASS_NACK_RECEIVE_TIMEOUT_AT_SENDER
     type USHORT value(MQCLASS_CODE (1, 1, '03'xu));


 /*------ PROPID_M_ACKNOWLEDGE --------------- */
 dcl MQMSG_ACKNOWLEDGMENT_NONE        fixed bin(15) value('00'xn);

 dcl MQMSG_ACKNOWLEDGMENT_POS_ARRIVAL fixed bin(15) value('01'xn);
 dcl MQMSG_ACKNOWLEDGMENT_POS_RECEIVE fixed bin(15) value('02'xn);
 dcl MQMSG_ACKNOWLEDGMENT_NEG_ARRIVAL fixed bin(15) value('04'xn);
 dcl MQMSG_ACKNOWLEDGMENT_NEG_RECEIVE fixed bin(15) value('08'xn);

 dcl MQMSG_ACKNOWLEDGMENT_NACK_REACH_QUEUE
     type UCHAR value(MQMSG_ACKNOWLEDGMENT_NEG_ARRIVAL);

 dcl MQMSG_ACKNOWLEDGMENT_FULL_REACH_QUEUE
     type UCHAR value(ior(MQMSG_ACKNOWLEDGMENT_NEG_ARRIVAL,
                          MQMSG_ACKNOWLEDGMENT_POS_ARRIVAL));

 dcl MQMSG_ACKNOWLEDGMENT_NACK_RECEIVE
     type UCHAR value(ior(MQMSG_ACKNOWLEDGMENT_NEG_ARRIVAL,
                          MQMSG_ACKNOWLEDGMENT_NEG_RECEIVE));

 dcl MQMSG_ACKNOWLEDGMENT_FULL_RECEIVE
     type UCHAR value(ior(MQMSG_ACKNOWLEDGMENT_NEG_ARRIVAL,
                          MQMSG_ACKNOWLEDGMENT_NEG_RECEIVE,
                          MQMSG_ACKNOWLEDGMENT_POS_RECEIVE));


 /*------ PROPID_M_DELIVERY ------------------ */
 dcl MQMSG_DELIVERY_EXPRESS             type DWORD value(0);
 dcl MQMSG_DELIVERY_RECOVERABLE         type DWORD value(1);

 /*----- PROPID_M_JOURNAL -------------------- */
 dcl MQMSG_JOURNAL_NONE                 type DWORD value(0);
 dcl MQMSG_DEADLETTER                   type DWORD value(1);
 dcl MQMSG_JOURNAL                      type DWORD value(2);

 /*----- PROPID_M_TRACE ---------------------- */
 dcl MQMSG_TRACE_NONE                   type DWORD value(0);
 dcl MQMSG_SEND_ROUTE_TO_REPORT_QUEUE   type DWORD value(1);

 /*----- PROPID_M_SENDERID_TYPE -------------- */
 dcl MQMSG_SENDERID_TYPE_NONE           type DWORD value(0);
 dcl MQMSG_SENDERID_TYPE_SID            type DWORD value(1);

 /*----- PROPID_M_PRIV_LEVEL ----------------- */
 dcl MQMSG_PRIV_LEVEL_NONE              type DWORD value(0);
 dcl MQMSG_PRIV_LEVEL_BODY              type DWORD value('01'xn);
 dcl MQMSG_PRIV_LEVEL_BODY_BASE         type DWORD value('01'xn);
 dcl MQMSG_PRIV_LEVEL_BODY_ENHANCED     type DWORD value('03'xn);

 /*----- PROPID_M_AUTH_LEVEL ----------------- */
 dcl MQMSG_AUTH_LEVEL_NONE              type DWORD value(0);
 dcl MQMSG_AUTH_LEVEL_ALWAYS            type DWORD value(1);

 /*----- PROPID_M_AUTHENTICATED -------------- */
 dcl MQMSG_AUTHENTICATION_NOT_REQUESTED type DWORD value(0);
 dcl MQMSG_AUTHENTICATION_REQUESTED     type DWORD value(1);


 /*********************************************************************/
 /*  QUEUE PROPERTIES */
 /*********************************************************************/
 dcl PROPID_Q_BASE              fixed bin(31) value(100);
 dcl PROPID_Q_INSTANCE          fixed bin(31) value(PROPID_Q_BASE +  1);
     /* VT_CLSID     */
 dcl PROPID_Q_TYPE              fixed bin(31) value(PROPID_Q_BASE +  2);
     /* VT_CLSID     */
 dcl PROPID_Q_PATHNAME          fixed bin(31) value(PROPID_Q_BASE +  3);
     /* VT_LPWSTR    */
 dcl PROPID_Q_JOURNAL           fixed bin(31) value(PROPID_Q_BASE +  4);
     /* VT_UI1       */
 dcl PROPID_Q_QUOTA             fixed bin(31) value(PROPID_Q_BASE +  5);
     /* VT_UI4       */
 dcl PROPID_Q_BASEPRIORITY      fixed bin(31) value(PROPID_Q_BASE +  6);
     /* VT_I2        */
 dcl PROPID_Q_JOURNAL_QUOTA     fixed bin(31) value(PROPID_Q_BASE +  7);
     /* VT_UI4       */
 dcl PROPID_Q_LABEL             fixed bin(31) value(PROPID_Q_BASE +  8);
     /* VT_LPWSTR    */
 dcl PROPID_Q_CREATE_TIME       fixed bin(31) value(PROPID_Q_BASE +  9);
     /* VT_I4        */
 dcl PROPID_Q_MODIFY_TIME       fixed bin(31) value(PROPID_Q_BASE + 10);
     /* VT_I4        */
 dcl PROPID_Q_AUTHENTICATE      fixed bin(31) value(PROPID_Q_BASE + 11);
     /* VT_UI1       */
 dcl PROPID_Q_PRIV_LEVEL        fixed bin(31) value(PROPID_Q_BASE + 12);
     /* VT_UI4       */
 dcl PROPID_Q_TRANSACTION       fixed bin(31) value(PROPID_Q_BASE + 13);
     /* VT_UI1       */
 dcl PROPID_Q_PATHNAME_DNS      fixed bin(31) value(PROPID_Q_BASE + 24);
     /* VT_LPWSTR    */


 /*----- PROPID_Q_JOURNAL ------------------ */
 dcl MQ_JOURNAL_NONE            fixed bin(8) unsigned value(0);
 dcl MQ_JOURNAL                 fixed bin(8) unsigned value(1);

 /*----- PROPID_Q_TYPE ------------------ */
 /*  {55EE8F32-CCE9-11cf-B108-0020AFD61CE9} */
 dcl 1 MQ_QTYPE_REPORT static,                 /* type GUID */
       2 Data1 type DWORD init ('55ee8f32'xu),
       2 Data2 type WORD  init ('cce9'xu),
       2 Data3 type WORD  init ('11cf'xu),
       2 Data4 (8) char   init ('b1'x, '08'x, '00'x, '20'x,
                                'af'x, 'd6'x, '1c'x, 'e9'x);

 /*  {55EE8F33-CCE9-11cf-B108-0020AFD61CE9} */
 dcl 1 MQ_QTYPE_TEST static,                   /* type GUID */
       2 Data1 type DWORD init ('55ee8f33'xu),
       2 Data2 type WORD  init ('cce9'xu),
       2 Data3 type WORD  init ('11cf'xu),
       2 Data4 (8) char   init ('b1'x, '08'x, '00'x, '20'x,
                                'af'x, 'd6'x, '1c'x, 'e9'x);

 /*----- PROPID_Q_TRANSACTION ------------------ */
 dcl MQ_TRANSACTIONAL_NONE  fixed bin(8) unsigned value(0);
 dcl MQ_TRANSACTIONAL       fixed bin(8) unsigned value(1);

 /*----- PROPID_Q_AUTHENTICATE ------------------ */
 dcl MQ_AUTHENTICATE_NONE   fixed bin(8) unsigned value(0);
 dcl MQ_AUTHENTICATE        fixed bin(8) unsigned value(1);

 /*----- PROPID_Q_PRIV_LEVEL ------------------ */
 dcl MQ_PRIV_LEVEL_NONE     fixed bin(32) unsigned value(0);
 dcl MQ_PRIV_LEVEL_OPTIONAL fixed bin(32) unsigned value(1);
 dcl MQ_PRIV_LEVEL_BODY     fixed bin(32) unsigned value(2);


 /*********************************************************************/
 /*  MACHINE PROPERTIES                                               */
 /*********************************************************************/
 dcl PROPID_QM_BASE            fixed bin(31) value(200);

 dcl PROPID_QM_SITE_ID         fixed bin(31) value(PROPID_QM_BASE +  1);
     /* VT_CLSID            */
 dcl PROPID_QM_MACHINE_ID      fixed bin(31) value(PROPID_QM_BASE +  2);
     /* VT_CLSID            */
 dcl PROPID_QM_PATHNAME        fixed bin(31) value(PROPID_QM_BASE +  3);
     /* VT_LPWSTR           */
 dcl PROPID_QM_CONNECTION      fixed bin(31) value(PROPID_QM_BASE +  4);
     /* VT_LPWSTR|VT_VECTOR */
 dcl PROPID_QM_ENCRYPTION_PK   fixed bin(31) value(PROPID_QM_BASE +  5);
     /* VT_BLOB             */
 dcl PROPID_QM_ENCRYPTION_PK_BASE
                               fixed bin(31) value(PROPID_QM_BASE + 31);
     /* VT_BLOB           */
 dcl PROPID_QM_ENCRYPTION_PK_ENHANCED
                               fixed bin(31) value(PROPID_QM_BASE + 32);
     /* VT_BLOB           */
 dcl PROPID_QM_PATHNAME_DNS    fixed bin(31) value(PROPID_QM_BASE + 33);
     /* VT_LPWSTR         */


 /* LONG_LIVED is the default for PROPID_M_TIME_TO_REACH_QUEUE.       */
 /* If a call to MQSendMessage() specifies this value, or does not    */
 /* give this property at all, then the actual timeout is taken from  */
 /* the MQIS database.                                                */
 dcl LONG_LIVED fixed bin(32) unsigned value('fffffffe'xu);

 /* Success */
 dcl MQ_OK fixed bin(31) value(0);


 /* dcl FACILITY_MSMQ        fixed bin(16) unsigned value('0e'xn)     */
 /*     -- see WinError.cpy                                           */


 /*  Errors */

 /* Values are 32 bit values layed out as follows:                    */
 /*                                                                   */
 /*  3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1                      */
 /*  1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0  */
 /* +---+-+-+-----------------------+-------------------------------+ */
 /* |Sev|C|R|     Facility          |               Code            | */
 /* +---+-+-+-----------------------+-------------------------------+ */
 /*                                                                   */
 /* where                                                             */
 /*      Sev - is the severity code                                   */
 /*          00 - Success                                             */
 /*          01 - Informational                                       */
 /*          10 - Warning                                             */
 /*          11 - Error                                               */
 /*      C - is the Customer code flag                                */
 /*      R - is a reserved bit                                        */
 /*      Facility - is the facility code                              */
 /*      Code - is the facility's status code                         */


 /* MessageId: MQ_ERROR                                               */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  GenericError                                                     */
 dcl MQ_ERROR                         fixed bin(31) value('C00E0001'xn);

 /* MessageId: MQ_ERROR_PROPERTY                                      */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  One or more of the passed properties are invalid.                */
 dcl MQ_ERROR_PROPERTY                fixed bin(31) value('C00E0002'xn);

 /* MessageId: MQ_ERROR_QUEUE_NOT_FOUND                               */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Queue not found                                                  */
 dcl MQ_ERROR_QUEUE_NOT_FOUND         fixed bin(31) value('C00E0003'xn);

 /* MessageId: MQ_ERROR_QUEUE_EXISTS                                  */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  A queue with the same pathname already exists                    */
 dcl MQ_ERROR_QUEUE_EXISTS            fixed bin(31) value('C00E0005'xn);

 /* MessageId: MQ_ERROR_INVALID_PARAMETER                             */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  An invalid parameter passed to a function.                       */
 dcl MQ_ERROR_INVALID_PARAMETER       fixed bin(31) value('C00E0006'xn);

 /* MessageId: MQ_ERROR_INVALID_HANDLE                                */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  An invalid handle passed to a function.                          */
 dcl MQ_ERROR_INVALID_HANDLE          fixed bin(31) value('C00E0007'xn);

 /* MessageId: MQ_ERROR_OPERATION_CANCELLED                           */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The operation was cancelled before it could be completed.        */
 dcl MQ_ERROR_OPERATION_CANCELLED     fixed bin(31) value('C00E0008'xn);

 /* MessageId: MQ_ERROR_SHARING_VIOLATION                             */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Sharing violation. The queue is already opened for exclusive     */
 /*  receive.                                                         */
 dcl MQ_ERROR_SHARING_VIOLATION       fixed bin(31) value('C00E0009'xn);

 /* MessageId: MQ_ERROR_SERVICE_NOT_AVAILABLE                         */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The Message Queues service is not available                      */
 dcl MQ_ERROR_SERVICE_NOT_AVAILABLE   fixed bin(31) value('C00E000B'xn);

 /* MessageId: MQ_ERROR_MACHINE_NOT_FOUND                             */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The specified machine could not be found.                        */
 dcl MQ_ERROR_MACHINE_NOT_FOUND       fixed bin(31) value('C00E000D'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_SORT                                  */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Illegal sort specified in MQLocateBegin                          */
 /*  (e.g., duplicate columns).                                       */
 dcl MQ_ERROR_ILLEGAL_SORT            fixed bin(31) value('C00E0010'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_USER                                  */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The user is an illegal user.                                     */
 dcl MQ_ERROR_ILLEGAL_USER            fixed bin(31) value('C00E0011'xn);

 /* MessageId: MQ_ERROR_NO_DS                                         */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  No connection with this site's controller(s).                    */
 /*  Please verify permissions.                                       */
 dcl MQ_ERROR_NO_DS                   fixed bin(31) value('C00E0013'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_QUEUE_PATHNAME                        */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Illegal queue path name.                                         */
 dcl MQ_ERROR_ILLEGAL_QUEUE_PATHNAME  fixed bin(31) value('C00E0014'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_PROPERTY_VALUE                        */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Illegal property value.                                          */
 dcl MQ_ERROR_ILLEGAL_PROPERTY_VALUE  fixed bin(31) value('C00E0018'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_PROPERTY_VT                           */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Invalid VARTYPE value.                                           */
 dcl MQ_ERROR_ILLEGAL_PROPERTY_VT     fixed bin(31) value('C00E0019'xn);

 /* MessageId: MQ_ERROR_BUFFER_OVERFLOW                               */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The buffer supplied to MQReceiveMessage for message body         */
 /*  retrieval was too small. The message is not removed from the     */
 /*  queue and part of the message body that fits in the buffer was   */
 /*  copied.                                                          */
 dcl MQ_ERROR_BUFFER_OVERFLOW         fixed bin(31) value('C00E001A'xn);

 /* MessageId: MQ_ERROR_IO_TIMEOUT                                    */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The MQReceiveMessage timeout has expired                         */
 dcl MQ_ERROR_IO_TIMEOUT              fixed bin(31) value('C00E001B'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_CURSOR_ACTION                         */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  MQ_ACTION_PEEK_NEXT specified to MQReceiveMessage can not be
 /*  used with the current cursor position.                           */
 dcl MQ_ERROR_ILLEGAL_CURSOR_ACTION   fixed bin(31) value('C00E001C'xn);

 /* MessageId: MQ_ERROR_MESSAGE_ALREADY_RECEIVED                      */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  A message that is currently pointed at by the cursor has been    */
 /*  removed from the queue by another process or by another call     */
 /*  to MQReceiveMessage without the use of this cursor.              */
 dcl MQ_ERROR_MESSAGE_ALREADY_RECEIVED
                                      fixed bin(31) value('C00E001D'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_FORMATNAME                            */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The given format name is invalid.                                */
 dcl MQ_ERROR_ILLEGAL_FORMATNAME      fixed bin(31) value('C00E001E'xn);

 /* MessageId: MQ_ERROR_FORMATNAME_BUFFER_TOO_SMALL                   */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The format name buffer supplied to the API was too small         */
 /*  to fit the format name                                           */
 dcl MQ_ERROR_FORMATNAME_BUFFER_TOO_SMALL
                                      fixed bin(31) value('C00E001F'xn);

 /* MessageId: MQ_ERROR_UNSUPPORTED_FORMATNAME_OPERATION              */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The requested operation for the specified format name is not     */
 /*  supported (e.g., delete a direct queue format name).             */
 dcl MQ_ERROR_UNSUPPORTED_FORMATNAME_OPERATION
                                      fixed bin(31) value('C00E0020'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_SECURITY_DESCRIPTOR                   */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The specified security descriptor is not a valid security descripto
 r. */
 dcl MQ_ERROR_ILLEGAL_SECURITY_DESCRIPTOR
                                      fixed bin(31) value('C00E0021'xn);

 /* MessageId: MQ_ERROR_SENDERID_BUFFER_TOO_SMALL                     */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The passed buffer for the user ID property is too small.         */
 dcl MQ_ERROR_SENDERID_BUFFER_TOO_SMALL
                                      fixed bin(31) value('C00E0022'xn);

 /* MessageId: MQ_ERROR_SECURITY_DESCRIPTOR_TOO_SMALL                 */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The size of the buffer passed to MQGetQueueSecurity is too small.*/
 dcl MQ_ERROR_SECURITY_DESCRIPTOR_TOO_SMALL
                                      fixed bin(31) value('C00E0023'xn);

 /* MessageId: MQ_ERROR_CANNOT_IMPERSONATE_CLIENT                     */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The RPC server can not impersonate the client application,       */
 /*  hence security credentials could not be verified.                */
 dcl MQ_ERROR_CANNOT_IMPERSONATE_CLIENT
                                      fixed bin(31) value('C00E0024'xn);

 /* MessageId: MQ_ERROR_ACCESS_DENIED                                 */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Access is denied.                                                */
 dcl MQ_ERROR_ACCESS_DENIED           fixed bin(31) value('C00E0025'xn);

 /* MessageId: MQ_ERROR_PRIVILEGE_NOT_HELD                            */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Client does not have the required privileges to perform the      */
 /*  operation.                                                       */
 dcl MQ_ERROR_PRIVILEGE_NOT_HELD      fixed bin(31) value('C00E0026'xn);

 /* MessageId: MQ_ERROR_INSUFFICIENT_RESOURCES                        */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Insufficient resources to perform operation.                     */
 dcl MQ_ERROR_INSUFFICIENT_RESOURCES  fixed bin(31) value('C00E0027'xn);

 /* MessageId: MQ_ERROR_USER_BUFFER_TOO_SMALL                         */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Request failed because user buffer is too small to hold the      */
 /*  returned information                                             */
 dcl MQ_ERROR_USER_BUFFER_TOO_SMALL   fixed bin(31) value('C00E0028'xn);

 /* MessageId: MQ_ERROR_MESSAGE_STORAGE_FAILED                        */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Could not store a recoverable or journal message.                */
 /*  Message was not sent                                             */
 dcl MQ_ERROR_MESSAGE_STORAGE_FAILED  fixed bin(31) value('C00E002A'xn);

 /* MessageId: MQ_ERROR_SENDER_CERT_BUFFER_TOO_SMALL                  */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The passed buffer for the user certificate property is too small.*/
 dcl MQ_ERROR_SENDER_CERT_BUFFER_TOO_SMALL
                                      fixed bin(31) value('C00E002B'xn);

 /* MessageId: MQ_ERROR_INVALID_CERTIFICATE                           */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The user certificate is not valid.                               */
 dcl MQ_ERROR_INVALID_CERTIFICATE     fixed bin(31) value('C00E002C'xn);

 /* MessageId: MQ_ERROR_CORRUPTED_INTERNAL_CERTIFICATE                */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The internal MSMQ certificate is corrupted.                      */
 dcl MQ_ERROR_CORRUPTED_INTERNAL_CERTIFICATE
                                      fixed bin(31) value('C00E002D'xn);


 /* MessageId: MQ_ERROR_NO_INTERNAL_USER_CERT                         */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The internal MSMQ certificate for the user does not exist.       */
 dcl MQ_ERROR_NO_INTERNAL_USER_CERT   fixed bin(31) value('C00E002F'xn);

 /* MessageId: MQ_ERROR_CORRUPTED_SECURITY_DATA                       */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  A cryptographic function has failed.                             */
 dcl MQ_ERROR_CORRUPTED_SECURITY_DATA fixed bin(31) value('C00E0030'xn);

 /* MessageId: MQ_ERROR_CORRUPTED_PERSONAL_CERT_STORE                 */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The personal certificate store is corrupted.                     */
 dcl MQ_ERROR_CORRUPTED_PERSONAL_CERT_STORE
                                      fixed bin(31) value('C00E0031'xn);

 /* MessageId: MQ_ERROR_COMPUTER_DOES_NOT_SUPPORT_ENCRYPTION          */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The computer does not support encryption operations.             */
 dcl MQ_ERROR_COMPUTER_DOES_NOT_SUPPORT_ENCRYPTION
                                      fixed bin(31) value('C00E0033'xn);

 /* MessageId: MQ_ERROR_BAD_SECURITY_CONTEXT                          */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Bad security context.                                            */
 dcl MQ_ERROR_BAD_SECURITY_CONTEXT    fixed bin(31) value('C00E0035'xn);

 /* MessageId: MQ_ERROR_COULD_NOT_GET_USER_SID                        */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Could not get the SID information out of the thread token.       */
 dcl MQ_ERROR_COULD_NOT_GET_USER_SID  fixed bin(31) value('C00E0036'xn);

 /* MessageId: MQ_ERROR_COULD_NOT_GET_ACCOUNT_INFO                    */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Could not get the account information for the user.              */
 dcl MQ_ERROR_COULD_NOT_GET_ACCOUNT_INFO
                                      fixed bin(31) value('C00E0037'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_MQCOLUMNS                             */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Invalid MQCOLUMNS parameter                                      */
 dcl MQ_ERROR_ILLEGAL_MQCOLUMNS       fixed bin(31) value('C00E0038'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_PROPID                                */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Invalid propid value                                             */
 dcl MQ_ERROR_ILLEGAL_PROPID          fixed bin(31) value('C00E0039'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_RELATION                              */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Invalid relation value in restriction                            */
 dcl MQ_ERROR_ILLEGAL_RELATION        fixed bin(31) value('C00E003A'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_PROPERTY_SIZE                         */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Illegal property buffer size                                     */
 dcl MQ_ERROR_ILLEGAL_PROPERTY_SIZE   fixed bin(31) value('C00E003B'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_RESTRICTION_PROPID                    */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Invalid propid value in MQRESTRICTION parameter                  */
 dcl MQ_ERROR_ILLEGAL_RESTRICTION_PROPID
                                      fixed bin(31) value('C00E003C'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_MQQUEUEPROPS                          */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Illegal MQQUEUEPROPS parameter, either null or with zero
 /*  properties                                                       */
 dcl MQ_ERROR_ILLEGAL_MQQUEUEPROPS    fixed bin(31) value('C00E003D'xn);

 /* MessageId: MQ_ERROR_PROPERTY_NOTALLOWED                           */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Invalid propid for the requested operation                       */
 /*  (e.g. PROPID_Q_INSTANCE in MQSetQueueProperties)                 */
 dcl MQ_ERROR_PROPERTY_NOTALLOWED     fixed bin(31) value('C00E003E'xn);

 /* MessageId: MQ_ERROR_INSUFFICIENT_PROPERTIES                       */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Not all the required properties for the operation were specified */
 /*  in the input parameters                                          */
 dcl MQ_ERROR_INSUFFICIENT_PROPERTIES fixed bin(31) value('C00E003F'xn);

 /* MessageId: MQ_ERROR_MACHINE_EXISTS                                */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Computer with the same name already exists in the site.          */
 dcl MQ_ERROR_MACHINE_EXISTS          fixed bin(31) value('C00E0040'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_MQQMPROPS                             */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Illegal MQQMPROPS parameter, either null or with zero properties */
 dcl MQ_ERROR_ILLEGAL_MQQMPROPS       fixed bin(31) value('C00E0041'xn);

 /* MessageId: MQ_ERROR_DS_IS_FULL                                    */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  DS is full                                                       */
 dcl MQ_ERROR_DS_IS_FULL              fixed bin(31) value('C00E0042'xn);

 /* MessageId: MQ_ERROR_DS_ERROR                                      */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Internal DS error.                                               */
 dcl MQ_ERROR_DS_ERROR                fixed bin(31) value('C00E0043'xn);

 /* MessageId: MQ_ERROR_INVALID_OWNER                                 */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Invalid object owner. For example MQCreateQueue failed because   */
 /*  the QM object is invalid                                         */
 dcl MQ_ERROR_INVALID_OWNER           fixed bin(31) value('C00E0044'xn);

 /* MessageId: MQ_ERROR_UNSUPPORTED_ACCESS_MODE                       */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The specified access mode is not supported.                      */
 dcl MQ_ERROR_UNSUPPORTED_ACCESS_MODE fixed bin(31) value('C00E0045'xn);

 /* MessageId: MQ_ERROR_RESULT_BUFFER_TOO_SMALL                       */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The supplied result buffer is too small                          */
 dcl MQ_ERROR_RESULT_BUFFER_TOO_SMALL fixed bin(31) value('C00E0046'xn);

 /* MessageId: MQ_ERROR_DELETE_CN_IN_USE                              */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The Connected Network can not be deleted, it is in use.          */
 dcl MQ_ERROR_DELETE_CN_IN_USE        fixed bin(31) value('C00E0048'xn);

 /* MessageId: MQ_ERROR_NO_RESPONSE_FROM_OBJECT_SERVER                */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  No response from object owner.                                   */
 dcl MQ_ERROR_NO_RESPONSE_FROM_OBJECT_SERVER
                                      fixed bin(31) value('C00E0049'xn);

 /* MessageId: MQ_ERROR_OBJECT_SERVER_NOT_AVAILABLE                   */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Object owner is not reachable.                                   */
 dcl MQ_ERROR_OBJECT_SERVER_NOT_AVAILABLE
                                      fixed bin(31) value('C00E004A'xn);

 /* MessageId: MQ_ERROR_QUEUE_NOT_AVAILABLE                           */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Error while reading from a queue residing on a remote computer   */
 dcl MQ_ERROR_QUEUE_NOT_AVAILABLE     fixed bin(31) value('C00E004B'xn);

 /* MessageId: MQ_ERROR_DTC_CONNECT                                   */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Cannot connect to MS DTC                                         */
 dcl MQ_ERROR_DTC_CONNECT             fixed bin(31) value('C00E004C'xn);

 /* MessageId: MQ_ERROR_TRANSACTION_IMPORT                            */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Cannot import the transaction                                    */
 dcl MQ_ERROR_TRANSACTION_IMPORT      fixed bin(31) value('C00E004E'xn);

 /* MessageId: MQ_ERROR_TRANSACTION_USAGE                             */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Wrong transaction usage                                          */
 dcl MQ_ERROR_TRANSACTION_USAGE       fixed bin(31) value('C00E0050'xn);

 /* MessageId: MQ_ERROR_TRANSACTION_SEQUENCE                          */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Wrong transaction operations sequence                            */
 dcl MQ_ERROR_TRANSACTION_SEQUENCE    fixed bin(31) value('C00E0051'xn);

 /* MessageId: MQ_ERROR_MISSING_CONNECTOR_TYPE                        */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Connector Type is mandatory when sending Acknowledgment or       */
 /*  secure message                                                   */
 dcl MQ_ERROR_MISSING_CONNECTOR_TYPE  fixed bin(31) value('C00E0055'xn
 );

 /* MessageId: MQ_ERROR_STALE_HANDLE                                  */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The Queue manager service has been restarted. The queue handle   */
 /*  is stale, and should be closed.                                  */
 dcl MQ_ERROR_STALE_HANDLE            fixed bin(31) value('C00E0056'xn);

 /* MessageId: MQ_ERROR_TRANSACTION_ENLIST                            */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Cannot enlist the transaction                                    */
 dcl MQ_ERROR_TRANSACTION_ENLIST      fixed bin(31) value('C00E0058'xn);

 /* MessageId: MQ_ERROR_QUEUE_DELETED                                 */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The queue was deleted. Messages can not be received anymore      */
 /*  using this queue handle. The handle should be closed             */
 dcl MQ_ERROR_QUEUE_DELETED           fixed bin(31) value('C00E005A'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_CONTEXT                               */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Invalid context parameter (MQLocateBegin).                       */
 dcl MQ_ERROR_ILLEGAL_CONTEXT         fixed bin(31) value('C00E005B'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_SORT_PROPID                           */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Invalid propid value in MQSORTSET                                */
 dcl MQ_ERROR_ILLEGAL_SORT_PROPID     fixed bin(31) value('C00E005C'xn);

 /* MessageId: MQ_ERROR_LABEL_TOO_LONG                                */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The passed label is too long. It should be less or equal to MQ_MAX_
 MSG_LABEL_LEN */
 dcl MQ_ERROR_LABEL_TOO_LONG          fixed bin(31) value('C00E005D'xn);

 /* MessageId: MQ_ERROR_LABEL_BUFFER_TOO_SMALL                        */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The label buffer supplied to the API was too small               */
 dcl MQ_ERROR_LABEL_BUFFER_TOO_SMALL  fixed bin(31) value('C00E005E'xn);

 /* MessageId: MQ_ERROR_MQIS_SERVER_EMPTY                             */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The list of MQIS servers (in registry) is empty.                 */
 dcl MQ_ERROR_MQIS_SERVER_EMPTY       fixed bin(31) value('C00E005F'xn);

 /* MessageId: MQ_ERROR_MQIS_READONLY_MODE                            */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  MQIS database is in read-only mode.                              */
 dcl MQ_ERROR_MQIS_READONLY_MODE      fixed bin(31) value('C00E0060'xn);

 /* MessageId: MQ_ERROR_SYMM_KEY_BUFFER_TOO_SMALL                     */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The passed buffer for the Symmetric key property is too small.   */
 dcl MQ_ERROR_SYMM_KEY_BUFFER_TOO_SMALL
                                      fixed bin(31) value('C00E0061'xn);

 /* MessageId: MQ_ERROR_SIGNATURE_BUFFER_TOO_SMALL                    */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The passed buffer for the Signature property is too small.       */
 dcl MQ_ERROR_SIGNATURE_BUFFER_TOO_SMALL
                                      fixed bin(31) value('C00E0062'xn);

 /* MessageId: MQ_ERROR_PROV_NAME_BUFFER_TOO_SMALL                    */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The passed buffer for the Provider name property is too small.   */
 dcl MQ_ERROR_PROV_NAME_BUFFER_TOO_SMALL
                                      fixed bin(31) value('C00E0063'xn);

 /* MessageId: MQ_ERROR_ILLEGAL_OPERATION                             */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The operation is illegal on foreign message queuing system       */
 dcl MQ_ERROR_ILLEGAL_OPERATION       fixed bin(31) value('C00E0064'xn);

 /* MessageId: MQ_ERROR_WRITE_NOT_ALLOWED                             */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Another MQIS server is being installed, write operations to the  */
 /*  database are not allowed at this stage.                          */
 dcl MQ_ERROR_WRITE_NOT_ALLOWED       fixed bin(31) value('C00E0065'xn);

 /* MessageId: MQ_ERROR_WKS_CANT_SERVE_CLIENT                         */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  MSMQ independent clients cannot serve MSMQ dependent clients.    */
 dcl MQ_ERROR_WKS_CANT_SERVE_CLIENT   fixed bin(31) value('C00E0066'xn);

 /* MessageId: MQ_ERROR_DEPEND_WKS_LICENSE_OVERFLOW                   */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The number of dependent clients served by this MSMQ server
 /*  reachedits upper limit.                                          */
 dcl MQ_ERROR_DEPEND_WKS_LICENSE_OVERFLOW
                                      fixed bin(31) value('C00E0067'xn);

 /* MessageId: MQ_CORRUPTED_QUEUE_WAS_DELETED                         */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Ini file for queue %1 in LQS was deleted because it was corrupted.
 */
 dcl MQ_CORRUPTED_QUEUE_WAS_DELETED   fixed bin(31) value('C00E0068'xn);

 /* MessageId: MQ_ERROR_REMOTE_MACHINE_NOT_AVAILABLE                  */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The remote machine is not available.                             */
 dcl MQ_ERROR_REMOTE_MACHINE_NOT_AVAILABLE
                                      fixed bin(31) value('C00E0069'xn);

 /* MessageId: MQ_ERROR_UNSUPPORTED_OPERATION                         */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Operation is not supported for WORKGROUP installation computer.  */
 dcl MQ_ERROR_UNSUPPORTED_OPERATION   fixed bin(31) value('C00E006A'xn);

 /* MessageId: MQ_ERROR_ENCRYPTION_PROVIDER_NOT_SUPPORTED             */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The encryption provider %1 is not supported by MSMQ              */
 dcl MQ_ERROR_ENCRYPTION_PROVIDER_NOT_SUPPORTED
                                      fixed bin(31) value('C00E006B'xn);

 /* MessageId: MQ_ERROR_CANNOT_SET_CRYPTO_SEC_DESCR                   */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Can't set the security descriptor of the machine keys set.       */
 dcl MQ_ERROR_CANNOT_SET_CRYPTO_SEC_DESCR
                                      fixed bin(31) value('C00E006C'xn);

 /* MessageId: MQ_ERROR_CERTIFICATE_NOT_PROVIDED                      */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  User tries to send authenticated message without providing a     */
 /*  certificate.                                                     */

 dcl MQ_ERROR_CERTIFICATE_NOT_PROVIDED
                                      fixed bin(31) value('C00E006D'xn);

 /* Informational                                                     */
 /*                                                                   */
 /* MessageId: MQ_INFORMATION_PROPERTY                                */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  One or more of the passed properties resulted in warning but the */
 /*  function completed.                                              */
 dcl MQ_INFORMATION_PROPERTY          fixed bin(31) value('400E0001'xn);

 /* MessageId: MQ_INFORMATION_ILLEGAL_PROPERTY                        */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Invalid property id.                                             */
 dcl MQ_INFORMATION_ILLEGAL_PROPERTY  fixed bin(31) value('400E0002'xn);

 /* MessageId: MQ_INFORMATION_PROPERTY_IGNORED                        */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The specified property is ignored in this operation              */
 /*  (e.g., PROPID_M_SENDERID in SendMessage().                       */
 dcl MQ_INFORMATION_PROPERTY_IGNORED  fixed bin(31) value('400E0003'xn);

 /* MessageId: MQ_INFORMATION_UNSUPPORTED_PROPERTY                    */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The specified property is not supported and is ignored in this
 /*  operation                                                        */
 dcl MQ_INFORMATION_UNSUPPORTED_PROPERTY
                                      fixed bin(31) value('400E0004'xn);

 /* MessageId: MQ_INFORMATION_DUPLICATE_PROPERTY                      */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The specified property already appeared in the propid array,     */
 /*  and is ignored in this operation                                 */
 dcl MQ_INFORMATION_DUPLICATE_PROPERTY
                                      fixed bin(31) value('400E0005'xn);

 /* MessageId: MQ_INFORMATION_OPERATION_PENDING                       */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  Asynchronous operation is currently pending.                     */
 dcl MQ_INFORMATION_OPERATION_PENDING fixed bin(31) value('400E0006'xn);

 /* MessageId: MQ_INFORMATION_FORMATNAME_BUFFER_TOO_SMALL             */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The format name buffer supplied to MQCreateQueue was too small   */
 /*  to fit the format name. Queue was created successfully           */
 dcl MQ_INFORMATION_FORMATNAME_BUFFER_TOO_SMALL
                                      fixed bin(31) value('400E0009'xn);

 /* MessageId: MQ_INFORMATION_INTERNAL_USER_CERT_EXIST                */
 /*                                                                   */
 /* MessageText:                                                      */
 /*                                                                   */
 /*  The internal MSMQ certificate already exists for this user.      */
 dcl MQ_INFORMATION_INTERNAL_USER_CERT_EXIST
                                      fixed bin(31) value('400E000A'xn);


 /*+----------------------------------------- */
 /*                                           */
 /* Flags for MQRegisterCertificate()         */
 /*                                           */
 /*+----------------------------------------- */

 dcl MQCERT_REGISTER_ALWAYS         fixed bin(31) value('01'xn);
 dcl MQCERT_REGISTER_IF_NOT_EXIST   fixed bin(31) value('02'xn);

 /*********************************************************************/
 /*  SECURITY Flags (Queue access control)                            */
 /*********************************************************************/

 dcl MQSEC_DELETE_MESSAGE           fixed bin(31) value('1'xn);
 dcl MQSEC_PEEK_MESSAGE             fixed bin(31) value('2'xn);
 dcl MQSEC_WRITE_MESSAGE            fixed bin(31) value('4'xn);
 dcl MQSEC_DELETE_JOURNAL_MESSAGE   fixed bin(31) value('8'xn);
 dcl MQSEC_SET_QUEUE_PROPERTIES     fixed bin(31) value('10'xn);
 dcl MQSEC_GET_QUEUE_PROPERTIES     fixed bin(31) value('20'xn);
 dcl MQSEC_DELETE_QUEUE             fixed bin(31) value(DELETE);
 dcl MQSEC_GET_QUEUE_PERMISSIONS    fixed bin(31) value(READ_CONTROL);
 dcl MQSEC_CHANGE_QUEUE_PERMISSIONS fixed bin(31) value(WRITE_DAC);
 dcl MQSEC_TAKE_QUEUE_OWNERSHIP     fixed bin(31) value(WRITE_OWNER);

 dcl MQSEC_RECEIVE_MESSAGE          fixed bin(31)
     value(ior(MQSEC_DELETE_MESSAGE,
               MQSEC_PEEK_MESSAGE));

 dcl MQSEC_RECEIVE_JOURNAL_MESSAGE  fixed bin(31)
     value(ior(MQSEC_DELETE_JOURNAL_MESSAGE,
               MQSEC_PEEK_MESSAGE));

 dcl MQSEC_QUEUE_GENERIC_READ       fixed bin(31)
     value(ior(MQSEC_GET_QUEUE_PROPERTIES,
               MQSEC_GET_QUEUE_PERMISSIONS,
               MQSEC_RECEIVE_MESSAGE,
               MQSEC_RECEIVE_JOURNAL_MESSAGE));

 dcl MQSEC_QUEUE_GENERIC_WRITE      fixed bin(31)
     value(ior(MQSEC_GET_QUEUE_PROPERTIES,
               MQSEC_GET_QUEUE_PERMISSIONS,
               MQSEC_WRITE_MESSAGE));


 dcl MQSEC_QUEUE_GENERIC_EXECUTE    fixed bin(31) value(0);

 dcl MQSEC_QUEUE_GENERIC_ALL        fixed bin(31)
     value(ior(MQSEC_RECEIVE_MESSAGE,
               MQSEC_RECEIVE_JOURNAL_MESSAGE,
               MQSEC_WRITE_MESSAGE,
               MQSEC_SET_QUEUE_PROPERTIES,
               MQSEC_GET_QUEUE_PROPERTIES,
               MQSEC_DELETE_QUEUE,
               MQSEC_GET_QUEUE_PERMISSIONS,
               MQSEC_CHANGE_QUEUE_PERMISSIONS,
               MQSEC_TAKE_QUEUE_OWNERSHIP));



 /*********************************************************************/
 /*  RECEIVE CALLBACK                                                 */
 /*********************************************************************/

 define alias PMQRECEIVECALLBACK
        entry (type HRESULT,
               type QUEUEHANDLE,
               type DWORD,
               type DWORD,
               type MQMSGPROPS byaddr,
               type OVERLAPPED byaddr optional,
               type HANDLE)
        limited APIENTRY;


 /*********************************************************************/
 /* MSMQ API                                                          */
 /*********************************************************************/

 dcl MQCreateQueue
     entry (type PSECURITY_DESCRIPTOR,
            type MQQUEUEPROPS byaddr,
            WCHAR(*) varz byaddr optional,
            type DWORD byaddr)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQCreateQueue');

 dcl MQDeleteQueue
     entry (WCHAR(*) varz byaddr nonasgn)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQDeleteQueue');

 dcl MQLocateBegin
     entry (WCHAR(*) varz byaddr nonasgn optional,
            type MQRESTRICTION byaddr optional,
            type MQCOLUMNSET byaddr,
            type MQSORTSET byaddr optional,
            type HANDLE byaddr)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQLocateBegin');

 dcl MQLocateNext
     entry (type HANDLE,
            type DWORD byaddr,
            (*) type MQPROPVARIANT byaddr)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQLocateNext');

 dcl MQLocateEnd
     entry (type HANDLE)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQLocateEnd');

 dcl MQOpenQueue
     entry (WCHAR(*) varz byaddr nonasgn,
            type DWORD,
            type DWORD,
            type QUEUEHANDLE byaddr)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQOpenQueue');

 dcl MQSendMessage
     entry (type QUEUEHANDLE,
            type MQMSGPROPS byaddr,
            handle ITransaction)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQSendMessage');

 dcl MQReceiveMessage
     entry (type QUEUEHANDLE,
            type DWORD,
            type DWORD,
            type MQMSGPROPS byaddr optional,
            type OVERLAPPED byaddr optional,
            type PMQRECEIVECALLBACK optional byvalue,
            type HANDLE,
            handle ITransaction)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQReceiveMessage');

 dcl MQCreateCursor
     entry (type QUEUEHANDLE,
            type HANDLE byaddr)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQCreateCursor');

 dcl MQCloseCursor
     entry (type HANDLE)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQCloseCursor');

 dcl MQCloseQueue
     entry (type HANDLE)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQCloseQueue');

 dcl MQSetQueueProperties
     entry (WCHAR(*) varz byaddr nonasgn,
            type MQQUEUEPROPS byaddr)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQSetQueueProperties');

 dcl MQGetQueueProperties
     entry (WCHAR(*) varz byaddr nonasgn,
           type MQQUEUEPROPS byaddr)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQGetQueueProperties');

 dcl MQGetQueueSecurity
     entry (WCHAR(*) varz byaddr nonasgn,
            type SECURITY_INFORMATION,
            type PSECURITY_DESCRIPTOR,
            type DWORD,
            type DWORD byaddr)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQGetQueueSecurity');

 dcl MQSetQueueSecurity
     entry (WCHAR(*) varz byaddr nonasgn,
            type SECURITY_INFORMATION,
            type PSECURITY_DESCRIPTOR)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQSetQueueSecurity');

 dcl MQPathNameToFormatName
     entry (WCHAR(*) varz byaddr nonasgn,
            WCHAR(*) varz byaddr,
            type DWORD byaddr)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQPathNameToFormatName');

 dcl MQHandleToFormatName
     entry (type QUEUEHANDLE,
            WCHAR(*) varz byaddr,
            type DWORD byaddr)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQHandleToFormatName');

 dcl MQInstanceToFormatName
     entry (type GUID byaddr nonasgn,
            WCHAR(*) varz byaddr,
            type DWORD byaddr)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQInstanceToFormatName');

 dcl MQFreeMemory
     entry (type PVOID)
     APIENTRY external('MQFreeMemory');

 dcl MQGetMachineProperties
     entry (WCHAR(*) varz byaddr nonasgn optional,
            type GUID byaddr nonasgn optional,
            type MQQMPROPS byaddr)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQGetMachineProperties');

 dcl MQGetSecurityContext
     entry (type PVOID,
            type DWORD,
            type HANDLE byaddr)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQGetSecurityContext');

 dcl MQFreeSecurityContext
     entry (type HANDLE)
     APIENTRY external('MQFreeSecurityContext');

 dcl MQRegisterCertificate
     entry (type DWORD,
            type PVOID,
            type DWORD)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQRegisterCertificate');

 dcl MQBeginTransaction
     entry (handle ITransaction)
     returns (type HRESULT byvalue optional)
     APIENTRY external('MQBeginTransaction');

 %end; /* __MQ_H__ */
