 /*********************************************************************/
 /*                                                                   */
 /* MsiQuery.cpy - Interface to running installer for custom actions  */
 /* and tools                                                         */
 /*                                                                   */
 /* Version 1.0                                                       */
 /*                                                                   */
 /* NOTES:  All buffers sizes are TCHAR count, null included only on  */
 /*         input                                                     */
 /*         Return argument pointers may be null if not interested in */
 /*         value                                                     */
 /*         Returned handles of all types must be closed:             */
 /*         MsiCloseHandle(h)                                         */
 /*         Functions with UINT return type return a system error code*/
 /*         Designated functions will set or clear the last error     */
 /*         record, which is then accessible with                     */
 /*         MsiGetLastErrorRecord. However, the following argument    */
 /*         errors do not register an error record:                   */
 /*         ERROR_INVALID_HANDLE, ERROR_INVALID_PARAMETER,            */
 /*         ERROR_MORE_DATA.                                          */
 /*                                                                   */
 /* Copyright (c) 1999, Microsoft Corp.      All rights reserved.     */
 /*                                                                   */
 /*********************************************************************/

 %dcl #MSIQUERY_H_ char ext;
 %if #MSIQUERY_H_ = '' %then %do;
 %#MSIQUERY_H_ = 'Y';

 %dcl #WINDEF_ char ext;
 %if #WINDEF_ = '' %then %do;
 %include winbase;
 %end;
 %dcl #MSI_H_ char ext;
 %if #MSI_H_ = '' %then %do;
 %include msi; /* INSTALLSTATE */
 %end;

 %dcl UNICODE char ext;

 dcl MSI_NULL_INTEGER        fixed bin(32) unsigned value('80000000'xu);
 /* integer value reserved for null */

 /* MsiOpenDatabase persist predefine values, */
 /* otherwise output database path is used    */
 %dcl MSIDBOPEN_READONLY     char;
     %MSIDBOPEN_READONLY     = pli_b2z(0);
     /* database open read-only, no persistent changes */
 %dcl MSIDBOPEN_TRANSACT     char;
     %MSIDBOPEN_TRANSACT     = pli_b2z(1);
     /* database read/write in transaction mode */
 %dcl MSIDBOPEN_DIRECT       char;
     %MSIDBOPEN_DIRECT       = pli_b2z(2);
     /* database direct read/write without transaction */
 %dcl MSIDBOPEN_CREATE       char;
     %MSIDBOPEN_CREATE       = pli_b2z(3);
     /* create new database, transact mode read/write */
 %dcl MSIDBOPEN_CREATEDIRECT char;
     %MSIDBOPEN_CREATEDIRECT = pli_b2z(4);
     /* create new database, direct mode read/write */
 dcl MSIDBOPEN_PATCHFILE TCHAR value ('*MSIDBOPEN_READONLY');
     /* add flag to indicate patch file */

 define ordinal MSIDBSTATE
   ( MSIDBSTATE_ERROR value(-1),/* invalid database handle */
     MSIDBSTATE_READ  value(0),
        /* database open read-only, no persistent changes */
     MSIDBSTATE_WRITE value(1)  /* database readable and updatable */
   ) prec(31);

 define ordinal MSIMODIFY
   ( MSIMODIFY_SEEK             value(-1),
        /* reposition to current record primary key */
     MSIMODIFY_REFRESH          value(0),
        /* refetch current record data */
     MSIMODIFY_INSERT           value(1),
        /* insert new record, fails if matching key exists */
     MSIMODIFY_UPDATE           value(2),
        /* update existing non-key data of fetched record */
     MSIMODIFY_ASSIGN           value(3),
        /* insert record, replacing any existing record */
     MSIMODIFY_REPLACE          value(4),
        /* update record, delete old if primary key edit */
     MSIMODIFY_MERGE            value(5),
        /* fails if record with duplicate key not identical */
     MSIMODIFY_DELETE           value(6),
        /* remove row referenced by this record from table */
     MSIMODIFY_INSERT_TEMPORARY value(7), /* insert a temporary record*/
     MSIMODIFY_VALIDATE         value(8), /* validate a fetched record*/
     MSIMODIFY_VALIDATE_NEW     value(9), /* validate a new record */
     MSIMODIFY_VALIDATE_FIELD   value(10),
        /* validate field(s) of an incomplete record */
     MSIMODIFY_VALIDATE_DELETE  value(11)
        /* validate before deleting record */
   ) prec(31);

 define ordinal MSICOLINFO
   ( MSICOLINFO_NAMES   value(0), /* return column names */
     MSICOLINFO_TYPES   value(1)
        /* return column definitions, datatype code followed by width */
   ) prec(31);

 define ordinal MSICONDITION
   ( MSICONDITION_FALSE value(0), /* expression evaluates to False */
     MSICONDITION_TRUE  value(1), /* expression evaluates to True */
     MSICONDITION_NONE  value(2), /* no expression present */
     MSICONDITION_ERROR value(3)  /* syntax error in expression */
   ) prec(31);

 define ordinal MSICOSTTREE
   ( MSICOSTTREE_SELFONLY value(0),
     MSICOSTTREE_CHILDREN value(1),
     MSICOSTTREE_PARENTS  value(2),
     MSICOSTTREE_RESERVED value(3) /* Reserved for future use */
   ) prec(31);

 define ordinal MSIDBERROR
   ( MSIDBERROR_INVALIDARG      value(-3),/* invalid argument */
     MSIDBERROR_MOREDATA        value(-2),/* buffer too small */
     MSIDBERROR_FUNCTIONERROR   value(-1),/* function error */
     MSIDBERROR_NOERROR         value(0), /* no error */
     MSIDBERROR_DUPLICATEKEY    value(1), /* new record duplicates */
                       /* primary keys of existing record in table */
     MSIDBERROR_REQUIRED        value(2),
        /* non-nullable column, no null values allowed */
     MSIDBERROR_BADLINK         value(3),
        /* corresponding record in foreign table not found */
     MSIDBERROR_OVERFLOW        value(4),
        /* data greater than maximum value allowed */
     MSIDBERROR_UNDERFLOW       value(5),
        /* data less than minimum value allowed */
     MSIDBERROR_NOTINSET        value(6),
        /* data not a member of the values permitted in the set */
     MSIDBERROR_BADVERSION      value(7), /* invalid version string */
     MSIDBERROR_BADCASE         value(8),
        /* invalid case, must be all upper-case or all lower-case */
     MSIDBERROR_BADGUID         value(9), /*  invalid GUID */
     MSIDBERROR_BADWILDCARD     value(10),
        /* invalid wildcardfilename or use of wildcards */
     MSIDBERROR_BADIDENTIFIER   value(11),/* bad identifier */
     MSIDBERROR_BADLANGUAGE     value(12),/* bad language Id(s) */
     MSIDBERROR_BADFILENAME     value(13),/* bad filename */
     MSIDBERROR_BADPATH         value(14),/* bad path */
     MSIDBERROR_BADCONDITION    value(15),/* bad conditional statement*/
     MSIDBERROR_BADFORMATTED    value(16),/* bad format string */
     MSIDBERROR_BADTEMPLATE     value(17),/* bad template string */
     MSIDBERROR_BADDEFAULTDIR   value(18),
        /* bad string in Default Dir column of Directory table */
     MSIDBERROR_BADREGPATH      value(19),/* bad registry path string */
     MSIDBERROR_BADCUSTOMSOURCE value(20),
        /* bad string in CustomSource column of CustomAction table */
     MSIDBERROR_BADPROPERTY     value(21),/* bad property string */
     MSIDBERROR_MISSINGDATA     value(22),
        /* _Validation table missing reference to column */
     MSIDBERROR_BADCATEGORY     value(23),
        /* Category column of _Validation table for column is invalid */
     MSIDBERROR_BADKEYTABLE     value(24),/* table in KeyTable column */
        /* of _Validation table could not be found/loaded */
     MSIDBERROR_BADMAXMINVALUES value(25),/* value in MaxValue column */
        /* of _Validation table is less than value in MinValue column */
     MSIDBERROR_BADCABINET      value(26),/* bad cabinet name */
     MSIDBERROR_BADSHORTCUT     value(27),/* bad shortcut target */
     MSIDBERROR_STRINGOVERFLOW  value(28),
        /* string overflow (greater than length allowed in column def)*/
     MSIDBERROR_BADLOCALIZEATTRIB value(29) /* invalid localization */
        /* attribute (primary keys cannot be localized) */
   ) prec(31);

 define ordinal MSIRUNMODE
   ( MSIRUNMODE_ADMIN             value(0),
        /* admin mode install, else product install */
     MSIRUNMODE_ADVERTISE         value(1), /* installing */
        /* advertisements, else installing or updating product */
     MSIRUNMODE_MAINTENANCE       value(2), /* modifying an existing */
        /* installation, else new installation */
     MSIRUNMODE_ROLLBACKENABLED   value(3), /* rollback is enabled */
     MSIRUNMODE_LOGENABLED        value(4),
        /* log file active, enabled prior to install session */
     MSIRUNMODE_OPERATIONS        value(5),
        /* spooling execute operations, else in determination phase */
     MSIRUNMODE_REBOOTATEND       value(6),
        /* reboot needed after successful installation (settable) */
     MSIRUNMODE_REBOOTNOW         value(7),
        /* reboot needed to continue installation (settable) */
     MSIRUNMODE_CABINET           value(8),
        /* installing files from cabinets and files using Media table */
     MSIRUNMODE_SOURCESHORTNAMES  value(9), /* source LongFileNames */
        /* suppressed via PID_MSISOURCE summary property */
     MSIRUNMODE_TARGETSHORTNAMES  value(10),
        /* target LongFileNames suppressed via SHORTFILENAMES property*/
     MSIRUNMODE_RESERVED11        value(11),/* future use */
     MSIRUNMODE_WINDOWS9X         value(12),
        /* operating systems is Windows9?, else Windows NT */
     MSIRUNMODE_ZAWENABLED        value(13),
        /* operating system supports demand installation */
     MSIRUNMODE_RESERVED14        value(14),/* future use */
     MSIRUNMODE_RESERVED15        value(15),/* future use */
     MSIRUNMODE_SCHEDULED         value(16),
        /* custom action call from install script execution */
     MSIRUNMODE_ROLLBACK          value(17),
        /* custom action call from rollback execution script */
     MSIRUNMODE_COMMIT            value(18)
        /* custom action call from commit execution script */
   ) prec(31);

 dcl INSTALLMESSAGE_TYPEMASK type INSTALLMESSAGE    value('FF000000'xu);
     /* mask for type code */

 /* Note: INSTALLMESSAGE_ERROR, INSTALLMESSAGE_WARNING,               */
 /* INSTALLMESSAGE_USER are to or'd with a message box style          */
 /* to indicate the buttons to display and return:                    */
 /* MB_OK,MB_OKCANCEL,MB_ABORTRETRYIGNORE,MB_YESNOCANCEL,MB_YESNO,    */
 /* MB_RETRYCANCEL                                                    */
 /* the default button (MB_DEFBUTTON1 is normal default):             */
 /* MB_DEFBUTTON1, MB_DEFBUTTON2, MB_DEFBUTTON3                       */
 /* and optionally an icon style:                                     */
 /* MB_ICONERROR, MB_ICONQUESTION, MB_ICONWARNING, MB_ICONINFORMATION */

 define alias MSITRANSFORM_ERROR type DWORD;
 dcl ( MSITRANSFORM_ERROR_ADDEXISTINGROW            value('00000001'xu),
       MSITRANSFORM_ERROR_DELMISSINGROW             value('00000002'xu),
       MSITRANSFORM_ERROR_ADDEXISTINGTABLE          value('00000004'xu),
       MSITRANSFORM_ERROR_DELMISSINGTABLE           value('00000008'xu),
       MSITRANSFORM_ERROR_UPDATEMISSINGROW          value('00000010'xu),
       MSITRANSFORM_ERROR_CHANGECODEPAGE            value('00000020'xu),
       MSITRANSFORM_ERROR_VIEWTRANSFORM             value('00000100'xu)
     ) type MSITRANSFORM_ERROR;

 define alias MSITRANSFORM_VALIDATE type DWORD;
 dcl ( MSITRANSFORM_VALIDATE_LANGUAGE               value('00000001'xu),
       MSITRANSFORM_VALIDATE_PRODUCT                value('00000002'xu),
       MSITRANSFORM_VALIDATE_PLATFORM               value('00000004'xu),
       MSITRANSFORM_VALIDATE_MAJORVERSION           value('00000008'xu),
       MSITRANSFORM_VALIDATE_MINORVERSION           value('00000010'xu),
       MSITRANSFORM_VALIDATE_UPDATEVERSION          value('00000020'xu),
       MSITRANSFORM_VALIDATE_NEWLESSBASEVERSION     value('00000040'xu),
       MSITRANSFORM_VALIDATE_NEWLESSEQUALBASEVERSION
                                                    value('00000080'xu),
       MSITRANSFORM_VALIDATE_NEWEQUALBASEVERSION    value('00000100'xu),
       MSITRANSFORM_VALIDATE_NEWGREATEREQUALBASEVERSION
                                                    value('00000200'xu),
       MSITRANSFORM_VALIDATE_NEWGREATERBASEVERSION  value('00000400'xu),
       MSITRANSFORM_VALIDATE_UPGRADECODE            value('00000800'xu)
     ) type MSITRANSFORM_VALIDATE;


 /* ----------------------------------------------------------------- */
 /* Installer database access functions                               */
 /* ----------------------------------------------------------------- */

 /* Prepare a database query, creating a view object                  */
 /* Returns ERROR_SUCCESS if successful, and the view handle is       */
 /* returned, else ERROR_INVALID_HANDLE, ERROR_INVALID_HANDLE_STATE,  */
 /* ERROR_BAD_QUERY_SYNTAX, ERROR_GEN_FAILURE                         */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiDatabaseOpenViewA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn,  /* SQL query to be prepared */
            type MSIHANDLE byaddr)        /* returned view if TRUE */
     returns (type UINT byvalue optional)
     WINAPI external('MsiDatabaseOpenViewA');
 dcl MsiDatabaseOpenViewW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn, /* SQL query to be prepared */
            type MSIHANDLE byaddr)        /* returned view if TRUE */
     returns (type UINT byvalue optional)
     WINAPI external('MsiDatabaseOpenViewW');

 %dcl MsiDatabaseOpenView char ext;
 %if UNICODE ^= '' %then %do;
   %MsiDatabaseOpenView='MsiDatabaseOpenViewW';
 %end; %else %do;
   %MsiDatabaseOpenView='MsiDatabaseOpenViewA';
 %end; /* !UNICODE */

 /* Returns the MSIDBERROR enum and name of the column corresponding  */
 /* to the error                                                      */
 /* Similar to a GetLastError function, but for the view.             */
 /* NOT the same as MsiGetLastErrorRecord                             */
 /* Returns errors of MsiViewModify.                                  */

 dcl MsiViewGetErrorA
     entry (type MSIHANDLE,
            char(*) varz byaddr,  /* buffer to hold column name */
            type DWORD byaddr)    /* size of buffer */
      returns (type MSIDBERROR byvalue optional)
     WINAPI external('MsiViewGetErrorA');
 dcl MsiViewGetErrorW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr, /* buffer to hold column name */
            type DWORD byaddr)    /* size of buffer */
     returns (type MSIDBERROR byvalue optional)
     WINAPI external('MsiViewGetErrorW');

 %dcl MsiViewGetError char ext;
 %if UNICODE ^= '' %then %do;
   %MsiViewGetError='MsiViewGetErrorW';
 %end; %else %do;
   %MsiViewGetError='MsiViewGetErrorA';
 %end; /* !UNICODE */

 /* Exectute the view query, supplying parameters as required         */
 /* Returns ERROR_SUCCESS, ERROR_INVALID_HANDLE,                      */
 /* ERROR_INVALID_HANDLE_STATE, ERROR_GEN_FAILURE                     */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiViewExecute
     entry (type MSIHANDLE,
            type MSIHANDLE) /* optional parameter record, or 0 if none*/
     returns (type UINT byvalue optional)
     WINAPI external('MsiViewExecute');

 /* Fetch the next sequential record from the view                    */
 /* Result is ERROR_SUCCESS if a row is found, and its handle is      */
 /* returned else ERROR_NO_DATA if no records remain, and a null      */
 /* handle is returned else result is error:                          */
 /* ERROR_INVALID_HANDLE_STATE, ERROR_INVALID_HANDLE,                 */
 /* ERROR_GEN_FAILURE                                                 */

 dcl MsiViewFetch
     entry (type MSIHANDLE,
            type MSIHANDLE byaddr)
                 /* returned data record if fetch succeeds */
     returns (type UINT byvalue optional)
     WINAPI external('MsiViewFetch');

 /* Modify a database record, parameters must match types in query    */
 /* columns                                                           */
 /* Returns ERROR_SUCCESS, ERROR_INVALID_HANDLE,                      */
 /* ERROR_INVALID_HANDLE_STATE, ERROR_GEN_FAILURE, ERROR_ACCESS_DENIED*/
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiViewModify
     entry (type MSIHANDLE,
            type MSIMODIFY, /* modify action to perform */
            type MSIHANDLE)
                 /* record obtained from fetch, or new record */
     returns (type UINT byvalue optional)
     WINAPI external('MsiViewModify');

 /* Return the column names or specifications for the current view    */
 /* Returns ERROR_SUCCESS, ERROR_INVALID_HANDLE,                      */
 /* ERROR_INVALID_PARAMETER, or ERROR_INVALID_HANDLE_STATE            */

 dcl MsiViewGetColumnInfo
     entry (type MSIHANDLE,
            type MSICOLINFO, /* retrieve columns names or definitions */
            type MSIHANDLE byaddr)
          /* returned data record containing all names or definitions */
     returns (type UINT byvalue optional)
     WINAPI external('MsiViewGetColumnInfo');

 /* Release the result set for an executed view, to allow re-execution*/
 /* Only needs to be called if not all records have been fetched      */
 /* Returns ERROR_SUCCESS, ERROR_INVALID_HANDLE,                      */
 /* ERROR_INVALID_HANDLE_STATE                                        */

 dcl MsiViewClose
     entry (type MSIHANDLE)
     returns (type UINT byvalue optional)
     WINAPI external('MsiViewClose');

 /* Return a record containing the names of all primary key columns   */
 /* for a given table                                                 */
 /* Returns an MSIHANDLE for a record containing the name of each     */
 /* column.                                                           */
 /* The field count of the record corresponds to the number of        */
 /* primary key columns.                                              */
 /* Field [0] of the record contains the table name.                  */
 /* Returns ERROR_SUCCESS, ERROR_INVALID_HANDLE, ERROR_INVALID_TABLE  */

 dcl MsiDatabaseGetPrimaryKeysA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn,
                    /* the name of a specific table <case-sensitive> */
            type MSIHANDLE byaddr) /* returned record if ERROR_SUCCESS*/
     returns (type UINT byvalue optional)
     WINAPI external('MsiDatabaseGetPrimaryKeysA');
 dcl MsiDatabaseGetPrimaryKeysW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn,
                    /* the name of a specific table <case-sensitive> */
            type MSIHANDLE byaddr) /* returned record if ERROR_SUCCESS*/
     returns (type UINT byvalue optional)
     WINAPI external('MsiDatabaseGetPrimaryKeysW');

 %dcl MsiDatabaseGetPrimaryKeys char ext;
 %if UNICODE ^= '' %then %do;
   %MsiDatabaseGetPrimaryKeys='MsiDatabaseGetPrimaryKeysW';
 %end; %else %do;
   %MsiDatabaseGetPrimaryKeys='MsiDatabaseGetPrimaryKeysA';
 %end; /* !UNICODE */

 /* Return an enum defining the state of the table (temporary,        */
 /* unknown, or persistent).                                          */
 /* Returns MSICONDITION_ERROR, MSICONDITION_FALSE, MSICONDITION_TRUE,*/
 /* MSICONDITION_NONE                                                 */

 dcl MsiDatabaseIsTablePersistentA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn)
                    /* the name of a specific table */
     returns (type MSICONDITION byvalue optional)
     WINAPI external('MsiDatabaseIsTablePersistentA');
 dcl MsiDatabaseIsTablePersistentW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn)
                    /* the name of a specific table */
     returns (type MSICONDITION byvalue optional)
     WINAPI external('MsiDatabaseIsTablePersistentW');

 %dcl MsiDatabaseIsTablePersistent char ext;
 %if UNICODE ^= '' %then %do;
   %MsiDatabaseIsTablePersistent='MsiDatabaseIsTablePersistentW';
 %end; %else %do;
   %MsiDatabaseIsTablePersistent='MsiDatabaseIsTablePersistentA';
 %end; /* !UNICODE */

 /* ----------------------------------------------------------------- */
 /* Summary information stream management functions                   */
 /* ----------------------------------------------------------------- */

 /* Integer Property IDs:    1, 14, 15, 16, 19                        */
 /* DateTime Property IDs:   10, 11, 12, 13                           */
 /* Text Property IDs:       2, 3, 4, 5, 6, 7, 8, 9, 18               */
 /* Unsupported Propery IDs: 0 (PID_DICTIONARY), 17 (PID_THUMBNAIL)   */

 /* Obtain a handle for the _SummaryInformation stream for an MSI     */
 /* database                                                          */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiGetSummaryInformationA
     entry (type MSIHANDLE, /* 0 if not open */
            char(*) varz byaddr nonasgn,
                   /* path to database, 0 if database handle supplied */
            type UINT,
            /* maximium number of updated values, 0 to open read-only */
            type MSIHANDLE byaddr)
                   /* returned handle to summary information data */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetSummaryInformationA');
 dcl MsiGetSummaryInformationW
     entry (type MSIHANDLE, /* 0 if not open */
            WCHAR(*) varz byaddr nonasgn,
                   /* path to database, 0 if database handle supplied */
            /* maximium number of updated values, 0 to open read-only */
            type UINT,
                   /* returned handle to summary information data */
            type MSIHANDLE byaddr)
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetSummaryInformationW');

 %dcl MsiGetSummaryInformation char ext;
 %if UNICODE ^= '' %then %do;
   %MsiGetSummaryInformation='MsiGetSummaryInformationW';
 %end; %else %do;
   %MsiGetSummaryInformation='MsiGetSummaryInformationA';
 %end; /* !UNICODE */

 /* Obtain the number of existing properties in the                   */
 /* SummaryInformation stream                                         */

 dcl MsiSummaryInfoGetPropertyCount
     entry (type MSIHANDLE,
            type UINT byaddr)
                 /* pointer to location to return total property count*/
     returns (type UINT byvalue optional)
     WINAPI external('MsiSummaryInfoGetPropertyCount');

 /* Set a single summary information property                         */
 /* Returns ERROR_SUCCESS, ERROR_INVALID_HANDLE,                      */
 /* ERROR_UNKNOWN_PROPERTY                                            */

 dcl MsiSummaryInfoSetPropertyA
     entry (type MSIHANDLE,
            type UINT,
        /* property ID, one of allowed values for summary information */
            type UINT, /* VT_I4, VT_LPSTR, VT_FILETIME, or VT_EMPTY */
            type INT, /* integer value, used only if integer property */
            type FILETIME byaddr optional,
         /* pointer to filetime value, used only if datetime property */
           char(*) varz byaddr nonasgn optional)
                      /* text value, used only if string property */
     returns (type UINT byvalue optional)
     WINAPI external('MsiSummaryInfoSetPropertyA');
 dcl MsiSummaryInfoSetPropertyW
     entry (type MSIHANDLE,
            type UINT,
        /* property ID, one of allowed values for summary information */
            type UINT, /* VT_I4, VT_LPSTR, VT_FILETIME, or VT_EMPTY */
            type INT, /* integer value, used only if integer property */
            type FILETIME byaddr optional,
         /* pointer to filetime value, used only if datetime property */
            WCHAR(*) varz byaddr nonasgn optional)
                      /* text value, used only if string property */
     returns (type UINT byvalue optional)
     WINAPI external('MsiSummaryInfoSetPropertyW');

 %dcl MsiSummaryInfoSetProperty char ext;
 %if UNICODE ^= '' %then %do;
   %MsiSummaryInfoSetProperty='MsiSummaryInfoSetPropertyW';
 %end; %else %do;
   %MsiSummaryInfoSetProperty='MsiSummaryInfoSetPropertyA';
 %end; /* !UNICODE */

 /* Get a single property from the summary information                */
 /* Returns ERROR_SUCCESS, ERROR_INVALID_HANDLE,                      */
 /* ERROR_UNKNOWN_PROPERTY                                            */

 dcl MsiSummaryInfoGetPropertyA
     entry (type MSIHANDLE,
            type UINT,
        /* property ID, one of allowed values for summary information */
            type UINT byaddr,
             /* returned type: VT_I4, VT_LPSTR, VT_FILETIME, VT_EMPTY */
            type INT byaddr,      /* returned integer property data   */
            type FILETIME byaddr, /* returned datetime property data  */
            char(*) varz byaddr,
                             /* buffer to return string property data */
            type DWORD byaddr) /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiSummaryInfoGetPropertyA');
 dcl MsiSummaryInfoGetPropertyW
     entry (type MSIHANDLE,
            type UINT,
        /* property ID, one of allowed values for summary information */
            type UINT byaddr,
             /* returned type: VT_I4, VT_LPSTR, VT_FILETIME, VT_EMPTY */
            type INT byaddr,      /* returned integer property data   */
            type FILETIME byaddr, /* returned datetime property data  */
            WCHAR(*) varz byaddr,
                             /* buffer to return string property data */
            type DWORD byaddr) /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiSummaryInfoGetPropertyW');

 %dcl MsiSummaryInfoGetProperty char ext;
 %if UNICODE ^= '' %then %do;
   %MsiSummaryInfoGetProperty='MsiSummaryInfoGetPropertyW';
 %end; %else %do;
   %MsiSummaryInfoGetProperty='MsiSummaryInfoGetPropertyA';
 %end; /* !UNICODE */

 /* Write back changed information to summary information stream */

 dcl MsiSummaryInfoPersist
     entry (type MSIHANDLE)
     returns (type UINT byvalue optional)
     WINAPI external('MsiSummaryInfoPersist');

 /* ----------------------------------------------------------------- */
 /* Installer database management functions                           */
 /*   - not used by custom actions                                    */
 /* ----------------------------------------------------------------- */

 /* Open an installer database, specifying the persistance mode,      */
 /* which is a pointer.                                               */
 /* Predefined persist values are reserved pointer values, requiring  */
 /* pointer arithmetic.                                               */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiOpenDatabaseA
     entry (char(*) varz byaddr nonasgn optional,
                  /* path to database, 0 to create temporary database */
            char(*) varz byaddr nonasgn,
                  /* output database path or one of predefined values */
            type MSIHANDLE byaddr)
                  /* location to return database handle */
     returns (type UINT byvalue optional)
     WINAPI external('MsiOpenDatabaseA');
 dcl MsiOpenDatabaseW
     entry (WCHAR(*) varz byaddr nonasgn,
                  /* path to database, 0 to create temporary database */
            WCHAR(*) varz byaddr nonasgn,
                  /* output database path or one of predefined values */
            type MSIHANDLE byaddr)
                  /* location to return database handle */
     returns (type UINT byvalue optional)
     WINAPI external('MsiOpenDatabaseW');

 %dcl MsiOpenDatabase char ext;
 %if UNICODE ^= '' %then %do;
   %MsiOpenDatabase='MsiOpenDatabaseW';
 %end; %else %do;
   %MsiOpenDatabase='MsiOpenDatabaseA';
 %end; /* !UNICODE */

 /* Import an MSI text archive table into an open database            */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiDatabaseImportA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn,
                                 /* folder containing archive files */
            char(*) varz byaddr nonasgn)
                                 /* table archive file to be imported */
     returns (type UINT byvalue optional)
     WINAPI external('MsiDatabaseImportA');
 dcl MsiDatabaseImportW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn,
                                 /* folder containing archive files */
            WCHAR(*) varz byaddr nonasgn)
                                 /* table archive file to be imported */
     returns (type UINT byvalue optional)
     WINAPI external('MsiDatabaseImportW');

 %dcl MsiDatabaseImport char ext;
 %if UNICODE ^= '' %then %do;
   %MsiDatabaseImport='MsiDatabaseImportW';
 %end; %else %do;
   %MsiDatabaseImport='MsiDatabaseImportA';
 %end; /* !UNICODE */

 /* Export an MSI table from an open database to a text archive file  */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiDatabaseExportA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn,
                        /* name of table in database <case-sensitive> */
            char(*) varz byaddr nonasgn,
                        /* folder containing archive files */
            char(*) varz byaddr nonasgn)
                        /* name of exported table archive file */
     returns (type UINT byvalue optional)
     WINAPI external('MsiDatabaseExportA');
 dcl MsiDatabaseExportW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn,
                        /* name of table in database <case-sensitive> */
            WCHAR(*) varz byaddr nonasgn,
                        /* folder containing archive files */
            WCHAR(*) varz byaddr nonasgn)
                        /* name of exported table archive file */
     returns (type UINT byvalue optional)
     WINAPI external('MsiDatabaseExportW');

 %dcl MsiDatabaseExport char ext;
 %if UNICODE ^= '' %then %do;
   %MsiDatabaseExport='MsiDatabaseExportW';
 %end; %else %do;
   %MsiDatabaseExport='MsiDatabaseExportA';
 %end; /* !UNICODE */

 /* Merge two database together, allowing duplicate rows              */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiDatabaseMergeA
     entry (type MSIHANDLE,
            type MSIHANDLE, /* database to be merged into hDatabase */
            char(*) varz byaddr nonasgn)
                    /* name of non-persistent table to receive errors */
     returns (type UINT byvalue optional)
     WINAPI external('MsiDatabaseMergeA');
 dcl MsiDatabaseMergeW
     entry (type MSIHANDLE,
            type MSIHANDLE, /* database to be merged into hDatabase */
            WCHAR(*) varz byaddr nonasgn)
                    /* name of non-persistent table to receive errors */
     returns (type UINT byvalue optional)
     WINAPI external('MsiDatabaseMergeW');

 %dcl MsiDatabaseMerge char ext;
 %if UNICODE ^= '' %then %do;
   %MsiDatabaseMerge='MsiDatabaseMergeW';
 %end; %else %do;
   %MsiDatabaseMerge='MsiDatabaseMergeA';
 %end; /* !UNICODE */

 /* Generate a transform file of differences between two databases    */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiDatabaseGenerateTransformA
     entry (type MSIHANDLE,
            type MSIHANDLE, /* base database to reference changes */
            char(*) varz byaddr nonasgn optional,
                            /* name of generated transform file */
            fixed bin(31),  /* reserved argument, not used */
            fixed bin(31))  /* reserved argument, not used */
     returns (type UINT byvalue optional)
     WINAPI external('MsiDatabaseGenerateTransformA');
 dcl MsiDatabaseGenerateTransformW
     entry (type MSIHANDLE,
            type MSIHANDLE, /* base database to reference changes */
            WCHAR(*) varz byaddr nonasgn optional,
                            /* name of generated transform file */
            fixed bin(31),  /* reserved argument, not used */
            fixed bin(31))  /* reserved argument, not used */
     returns (type UINT byvalue optional)
     WINAPI external('MsiDatabaseGenerateTransformW');

 %dcl MsiDatabaseGenerateTransform char ext;
 %if UNICODE ^= '' %then %do;
   %MsiDatabaseGenerateTransform='MsiDatabaseGenerateTransformW';
 %end; %else %do;
   %MsiDatabaseGenerateTransform='MsiDatabaseGenerateTransformA';
 %end; /* !UNICODE */

 /* Apply a transform file containing database difference             */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiDatabaseApplyTransformA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn,    /* name of transform file */
            fixed bin(31))
                  /* errors to suppress, bits from MSITRANSFORM_ERROR */
     returns (type UINT byvalue optional)
     WINAPI external('MsiDatabaseApplyTransformA');
 dcl MsiDatabaseApplyTransformW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn,   /* name of transform file */
            fixed bin(31))
                  /* errors to suppress, bits from MSITRANSFORM_ERROR */
     returns (type UINT byvalue optional)
     WINAPI external('MsiDatabaseApplyTransformW');

 %dcl MsiDatabaseApplyTransform char ext;
 %if UNICODE ^= '' %then %do;
   %MsiDatabaseApplyTransform='MsiDatabaseApplyTransformW';
 %end; %else %do;
   %MsiDatabaseApplyTransform='MsiDatabaseApplyTransformA';
 %end; /* !UNICODE */

 /* Create summary information of existing transform to include       */
 /* validation and error conditions                                   */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiCreateTransformSummaryInfoA
     entry (type MSIHANDLE,
            type MSIHANDLE,     /* base database to reference changes */
            char(*) varz byaddr nonasgn,
                                /* name of generated transform file   */
            fixed bin(31),
          /* errors to suppress when applied, from MSITRANSFORM_ERROR */
            fixed bin(31))
          /* properties validated when applied, MSITRANSFORM_VALIDATE */
     returns (type UINT byvalue optional)
     WINAPI external('MsiCreateTransformSummaryInfoA');
 dcl MsiCreateTransformSummaryInfoW
     entry (type MSIHANDLE,
            type MSIHANDLE,     /* base database to reference changes */
            WCHAR(*) varz byaddr nonasgn,
                                /* name of generated transform file   */
            fixed bin(31),
          /* errors to suppress when applied, from MSITRANSFORM_ERROR */
            fixed bin(31))
          /* properties validated when applied, MSITRANSFORM_VALIDATE */
     returns (type UINT byvalue optional)
     WINAPI external('MsiCreateTransformSummaryInfoW');

 %dcl MsiCreateTransformSummaryInfo char ext;
 %if UNICODE ^= '' %then %do;
   %MsiCreateTransformSummaryInfo='MsiCreateTransformSummaryInfoW';
 %end; %else %do;
   %MsiCreateTransformSummaryInfo='MsiCreateTransformSummaryInfoA';
 %end; /* !UNICODE */

 /* Write out all persistent table data, ignored if database opened   */
 /* read-only                                                         */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiDatabaseCommit
     entry (type MSIHANDLE)
     returns (type UINT byvalue optional)
     WINAPI external('MsiDatabaseCommit');

 /* Return the update state of a database */

 dcl MsiGetDatabaseState
     entry (type MSIHANDLE)
     returns (type MSIDBSTATE byvalue optional)
     WINAPI external('MsiGetDatabaseState');

 /* ----------------------------------------------------------------- */
 /* Record object functions                                           */
 /* ----------------------------------------------------------------- */

 /* Create a new record object with the requested number of fields    */
 /* Field 0, not included in count, is used for format strings and op */
 /* codes. All fields are initialized to null                         */
 /* Returns a handle to the created record, or 0 if memory could not  */
 /* be allocated                                                      */

 dcl MsiCreateRecord
     entry (type UINT) /* the number of data fields */
     returns (type MSIHANDLE byvalue optional)
     WINAPI external('MsiCreateRecord');

 /* Report whether a record field is NULL                             */
 /* Returns TRUE if the field is null or does not exist               */
 /* Returns FALSE if the field contains data, or the handle is invalid*/

 dcl MsiRecordIsNull
     entry (type MSIHANDLE,
            type UINT)
     returns (type BOOL byvalue optional)
     WINAPI external('MsiRecordIsNull');

 /* Return the length of a record field                               */
 /* Returns 0 if field is NULL or non-existent                        */
 /* Returns sizeof(int) if integer data                               */
 /* Returns character count if string data (not counting null         */
 /*                                         terminator)               */
 /* Returns bytes count if stream data                                */

 dcl MsiRecordDataSize
     entry (type MSIHANDLE,
            type UINT)
     returns (type UINT byvalue optional)
     WINAPI external('MsiRecordDataSize');

 /* Set a record field to an integer value                            */
 /* Returns ERROR_SUCCESS, ERROR_INVALID_HANDLE, ERROR_INVALID_FIELD  */

 dcl MsiRecordSetInteger
     entry (type MSIHANDLE,
            type UINT,
            fixed bin(31))
     returns (type UINT byvalue optional)
     WINAPI external('MsiRecordSetInteger');

 /* Copy a string into the designated field                           */
 /* A null string pointer and an empty string both set the field to   */
 /* null                                                              */
 /* Returns ERROR_SUCCESS, ERROR_INVALID_HANDLE, ERROR_INVALID_FIELD  */

 dcl MsiRecordSetStringA
     entry (type MSIHANDLE,
            type UINT,
            char(*) varz byaddr nonasgn optional)
     returns (type UINT byvalue optional)
     WINAPI external('MsiRecordSetStringA');
 dcl MsiRecordSetStringW
     entry (type MSIHANDLE,
            type UINT,
            WCHAR(*) varz byaddr nonasgn optional)
     returns (type UINT byvalue optional)
     WINAPI external('MsiRecordSetStringW');

 %dcl MsiRecordSetString char ext;
 %if UNICODE ^= '' %then %do;
   %MsiRecordSetString='MsiRecordSetStringW';
 %end; %else %do;
   %MsiRecordSetString='MsiRecordSetStringA';
 %end; /* !UNICODE */

 /* Return the integer value from a record field                      */
 /* Returns the value MSI_NULL_INTEGER if the field is null           */
 /* or if the field is a string that cannot be converted to an integer*/

 dcl MsiRecordGetInteger
     entry (type MSIHANDLE,
            type UINT)
     returns (fixed bin(31) byvalue optional)
     WINAPI external('MsiRecordGetInteger');

 /* Return the string value of a record field                         */
 /* Integer fields will be converted to a string                      */
 /* Null and non-existent fields will report a value of 0             */
 /* Fields containing stream data will return ERROR_INVALID_DATATYPE  */
 /* Returns ERROR_SUCCESS, ERROR_MORE_DATA, ERROR_INVALID_HANDLE,     */
 /*         ERROR_INVALID_FIELD, ERROR_BAD_ARGUMENTS                  */

 dcl MsiRecordGetStringA
     entry (type MSIHANDLE,
            type UINT,
            char (*) varz byaddr, /* buffer for returned value */
            type DWORD byaddr) /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiRecordGetStringA');
 dcl MsiRecordGetStringW
     entry (type MSIHANDLE,
            type UINT,
            WCHAR (*) varz byaddr, /* buffer for returned value */
            type DWORD byaddr) /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiRecordGetStringW');

 %dcl MsiRecordGetString char ext;
 %if UNICODE ^= '' %then %do;
   %MsiRecordGetString='MsiRecordGetStringW';
 %end; %else %do;
   %MsiRecordGetString='MsiRecordGetStringA';
 %end; /* !UNICODE */

 /* Returns the number of fields allocated in the record              */
 /* Does not count field 0, used for formatting and op codes          */

 dcl MsiRecordGetFieldCount
     entry (type MSIHANDLE)
     returns (type UINT byvalue optional)
     WINAPI external('MsiRecordGetFieldCount');

 /* Set a record stream field from a file                             */
 /* The contents of the specified file will be read into a stream     */
 /* object. The stream will be persisted if the record is inserted    */
 /* into the database                                                 */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiRecordSetStreamA
     entry (type MSIHANDLE,
            type UINT,
            char(*) varz byaddr nonasgn)
                               /* path to file containing stream data */
     returns (type UINT byvalue optional)
     WINAPI external('MsiRecordSetStreamA');
 dcl MsiRecordSetStreamW
     entry (type MSIHANDLE,
            type UINT,
            WCHAR(*) varz byaddr nonasgn)
                               /* path to file containing stream data */
     returns (type UINT byvalue optional)
     WINAPI external('MsiRecordSetStreamW');

 %dcl MsiRecordSetStream char ext;
 %if UNICODE ^= '' %then %do;
   %MsiRecordSetStream='MsiRecordSetStreamW';
 %end; %else %do;
   %MsiRecordSetStream='MsiRecordSetStreamA';
 %end; /* !UNICODE */

 /* Read bytes from a record stream field into a buffer               */
 /* Must set the in/out argument to the requested byte count to read  */
 /* The number of bytes transferred is returned through the argument  */
 /* If no more bytes are available, ERROR_SUCCESS is still returned   */

 dcl MsiRecordReadStream
     entry (type MSIHANDLE,
            type UINT,
            pointer,       /* buffer to receive bytes from stream */
        type DWORD byaddr) /* in/out buffer byte count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiRecordReadStream');

 /* Clears all data fields in a record to NULL */

 dcl MsiRecordClearData
     entry (type MSIHANDLE)
     returns (type UINT byvalue optional)
     WINAPI external('MsiRecordClearData');

 /* ----------------------------------------------------------------- */
 /* Functions to access a running installation, called from custom    */
 /* actions                                                           */
 /* The install handle is the single argument passed to custom actions*/
 /* ----------------------------------------------------------------- */

 /* Return a handle to the database currently in use by this          */
 /* installer instance                                                */

 dcl MsiGetActiveDatabase
     entry (type MSIHANDLE)
     returns (type MSIHANDLE byvalue optional)
                   /* returns handle to database, ~0 if none active */
     WINAPI external('MsiGetActiveDatabase');

 /* Set the value for an installer property                           */
 /* If the property is not defined, it will be created                */
 /* If the value is null or an empty string, the property will be     */
 /* removed                                                           */
 /* Returns ERROR_SUCCESS, ERROR_INVALID_HANDLE, ERROR_BAD_ARGUMENTS  */

 dcl MsiSetPropertyA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn,
                         /* property identifier, case-sensitive */
            char(*) varz byaddr nonasgn optional)
                         /* property value, null to undefine property */
     returns (type UINT byvalue optional)
     WINAPI external('MsiSetPropertyA');
 dcl MsiSetPropertyW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn,
                         /* property identifier, case-sensitive */
            WCHAR(*) varz byaddr nonasgn optional)
                         /* property value, null to undefine property */
     returns (type UINT byvalue optional)
     WINAPI external('MsiSetPropertyW');

 %dcl MsiSetProperty char ext;
 %if UNICODE ^= '' %then %do;
   %MsiSetProperty='MsiSetPropertyW';
 %end; %else %do;
   %MsiSetProperty='MsiSetPropertyA';
 %end; /* !UNICODE */

 /* Get the value for an installer property                           */
 /* If the property is not defined, it is equivalent to a 0-length    */
 /* value, not error                                                  */
 /* Returns ERROR_SUCCESS, ERROR_MORE_DATA, ERROR_INVALID_HANDLE,     */
 /* ERROR_BAD_ARGUMENTS                                               */

 dcl MsiGetPropertyA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn,
                               /* property identifier, case-sensitive */
            char(*) varz byaddr,
                               /* buffer for returned property value */
            type DWORD byaddr) /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetPropertyA');
 dcl MsiGetPropertyW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn,
                               /* property identifier, case-sensitive */
            WCHAR(*) varz byaddr,
                               /* buffer for returned property value */
            type DWORD byaddr) /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetPropertyW');

 %dcl MsiGetProperty char ext;
 %if UNICODE ^= '' %then %do;
   %MsiGetProperty='MsiGetPropertyW';
 %end; %else %do;
   %MsiGetProperty='MsiGetPropertyA';
 %end; /* !UNICODE */

 /* Return the numeric language for the currently running install     */
 /* Returns 0 if an install not running                               */

 dcl MsiGetLanguage
     entry (type MSIHANDLE)
     returns (type LANGID byvalue optional)
     WINAPI external('MsiGetLanguage');

 /* Return one of the boolean internal installer states               */
 /* Returns FALSE if the handle is not active or if the mode is not   */
 /* implemented                                                       */

 dcl MsiGetMode
     entry (type MSIHANDLE,
            type MSIRUNMODE)
                   /* particular mode for which the state is returned */
     returns (type BOOL byvalue optional)
     WINAPI external('MsiGetMode');

 /* Set an internal install session boolean mode - Note: most modes   */
 /* are read-only                                                     */
 /* Returns ERROR_SUCCESS if the mode can be set to the desired state */
 /* Returns ERROR_ACCESS_DENIED if the mode is not settable           */
 /* Returns ERROR_INVALID_HANDLE if the handle is not an active       */
 /* install session                                                   */

 dcl MsiSetMode
     entry (type MSIHANDLE,
            type MSIRUNMODE,
                       /* particular mode for which state is to be set*/
            type BOOL) /* new state for bit flag */
     returns (type UINT byvalue optional)
     WINAPI external('MsiSetMode');

 /* Format record data using a format string containing field markers */
 /* and/or properties                                                 */
 /* Record field 0 must contain the format string                     */
 /* Other fields must contain data that may be referenced by the      */
 /* format string.                                                    */

 dcl MsiFormatRecordA
     entry (type MSIHANDLE,      /* non-zero for property expansion */
            type MSIHANDLE,
                  /* handle to record, field 0 contains format string */
            char(*) varz byaddr, /* buffer to return formatted string */
            type DWORD byaddr)   /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiFormatRecordA');
 dcl MsiFormatRecordW
     entry (type MSIHANDLE,      /* non-zero for property expansion */
            type MSIHANDLE,
                  /* handle to record, field 0 contains format string */
            WCHAR(*) varz byaddr,/* buffer to return formatted string */
            type DWORD byaddr)   /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiFormatRecordW');

 %dcl MsiFormatRecord char ext;
 %if UNICODE ^= '' %then %do;
   %MsiFormatRecord='MsiFormatRecordW';
 %end; %else %do;
   %MsiFormatRecord='MsiFormatRecordA';
 %end; /* !UNICODE */

 /* Execute another action, either built-in, custom, or UI wizard     */
 /* Returns ERROR_FUNCTION_NOT_CALLED if action not found             */
 /* Returns ERROR_SUCCESS if action completed succesfully             */
 /* Returns ERROR_INSTALL_USEREXIT if user cancelled during action    */
 /* Returns ERROR_INSTALL_FAILURE if action failed                    */
 /* Returns ERROR_INSTALL_SUSPEND if user suspended installation      */
 /* Returns ERROR_MORE_DATA if action wishes to skip remaining actions*/
 /* Returns ERROR_INVALID_HANDLE_STATE if install session not active  */
 /* Returns ERROR_INVALID_DATA if failure calling custom action       */
 /* Returns ERROR_INVALID_HANDLE or ERROR_INVALID_PARAMETER           */
 /*         if arguments invalid                                      */

 dcl MsiDoActionA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn)
                            /* name of action to call, case-sensitive */
     returns (type UINT byvalue optional)
     WINAPI external('MsiDoActionA');
 dcl MsiDoActionW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn)
                            /* name of action to call, case-sensitive */
     returns (type UINT byvalue optional)
     WINAPI external('MsiDoActionW');

 %dcl MsiDoAction char ext;
 %if UNICODE ^= '' %then %do;
   %MsiDoAction='MsiDoActionW';
 %end; %else %do;
   %MsiDoAction='MsiDoActionA';
 %end; /* !UNICODE */

 /* Execute another action sequence, as descibed in the specified     */
 /* table                                                             */
 /* Returns the same error codes as MsiDoAction                       */

 dcl MsiSequenceA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn,
                      /* name of table containing action sequence */
            type INT) /* for future use, must be 0 in MSI 1.0 */
     returns (type UINT byvalue optional)
     WINAPI external('MsiSequenceA');
 dcl MsiSequenceW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn,
                      /* name of table containing action sequence */
            type INT) /* for future use, must be 0 in MSI 1.0 */
     returns (type UINT byvalue optional)
     WINAPI external('MsiSequenceW');

 %dcl MsiSequence char ext;
 %if UNICODE ^= '' %then %do;
   %MsiSequence='MsiSequenceW';
 %end; %else %do;
   %MsiSequence='MsiSequenceA';
 %end; /* !UNICODE */

 /* Send an error record to the installer for processing.             */
 /* If field 0 (template) is not set, field 1 must be set to the      */
 /* error code, corresponding the the error message in the Error      */
 /* database table, and the message will be formatted using the       */
 /* template from the Error table before passing it to the UI handler */
 /* for display.                                                      */
 /* Returns Win32 button codes: IDOK IDCANCEL IDABORT IDRETRY         */
 /* IDIGNORE IDYES IDNO or 0 if no action taken, or -1 if invalid     */
 /* argument or handle                                                */

 dcl MsiProcessMessage
     entry (type MSIHANDLE,
            type INSTALLMESSAGE, /* type of message */
            type MSIHANDLE)
                         /* record containing message format and data */
     returns (fixed bin(31) byvalue optional)
     WINAPI external('MsiProcessMessage');

 /* Evaluate a conditional expression containing property names       */
 /* and values                                                        */

 dcl MsiEvaluateConditionA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn)
     returns (type MSICONDITION byvalue optional)
     WINAPI external('MsiEvaluateConditionA');
 dcl MsiEvaluateConditionW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn)
     returns (type MSICONDITION byvalue optional)
     WINAPI external('MsiEvaluateConditionW');

 %dcl MsiEvaluateCondition char ext;
 %if UNICODE ^= '' %then %do;
   %MsiEvaluateCondition='MsiEvaluateConditionW';
 %end; %else %do;
   %MsiEvaluateCondition='MsiEvaluateConditionA';
 %end; /* !UNICODE */

 /* Get the installed state and requested action state of a feature   */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiGetFeatureStateA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn,
                               /* feature name within product */
            type INSTALLSTATE byaddr,
                               /* returned current install state */
            type INSTALLSTATE byaddr)
                               /* action taken during install session */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetFeatureStateA');
 dcl MsiGetFeatureStateW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn,
                               /* feature name within product */
            type INSTALLSTATE byaddr,
                               /* returned current install state */
            type INSTALLSTATE byaddr)
                               /* action taken during install session */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetFeatureStateW');

 %dcl MsiGetFeatureState char ext;
 %if UNICODE ^= '' %then %do;
   %MsiGetFeatureState='MsiGetFeatureStateW';
 %end; %else %do;
   %MsiGetFeatureState='MsiGetFeatureStateA';
 %end; /* !UNICODE */

 /* Request a feature to be set to a specified state                  */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiSetFeatureStateA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn,
                               /* feature name within product */
            type INSTALLSTATE) /* requested state for feature */
     returns (type UINT byvalue optional)
     WINAPI external('MsiSetFeatureStateA');
 dcl MsiSetFeatureStateW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn,
                               /* feature name within product */
            type INSTALLSTATE) /* requested state for feature */
     returns (type UINT byvalue optional)
     WINAPI external('MsiSetFeatureStateW');

 %dcl MsiSetFeatureState char ext;
 %if UNICODE ^= '' %then %do;
   %MsiSetFeatureState='MsiSetFeatureStateW';
 %end; %else %do;
   %MsiSetFeatureState='MsiSetFeatureStateA';
 %end; /* !UNICODE */

 /* Get the installed state and requested action state of a component */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiGetComponentStateA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn,
                               /* component name within product */
            type INSTALLSTATE byaddr,
                               /* returned current install state */
            type INSTALLSTATE byaddr)
                               /* action taken during install session */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetComponentStateA');
 dcl MsiGetComponentStateW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn,
                               /* component name within product */
            type INSTALLSTATE byaddr,
                               /* returned current install state */
            type INSTALLSTATE byaddr)
                               /* action taken during install session */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetComponentStateW');

 %dcl MsiGetComponentState char ext;
 %if UNICODE ^= '' %then %do;
   %MsiGetComponentState='MsiGetComponentStateW';
 %end; %else %do;
   %MsiGetComponentState='MsiGetComponentStateA';
 %end; /* !UNICODE */

 /* Request a component to be set to a specified state                */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiSetComponentStateA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn,
                               /* component name within product */
            type INSTALLSTATE) /* requested state for component */
     returns (type UINT byvalue optional)
     WINAPI external('MsiSetComponentStateA');
 dcl MsiSetComponentStateW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn,
                               /* component name within product */
            type INSTALLSTATE) /* requested state for component */
     returns (type UINT byvalue optional)
     WINAPI external('MsiSetComponentStateW');

 %dcl MsiSetComponentState char ext;
 %if UNICODE ^= '' %then %do;
   %MsiSetComponentState='MsiSetComponentStateW';
 %end; %else %do;
   %MsiSetComponentState='MsiSetComponentStateA';
 %end; /* !UNICODE */

 /* Return the disk cost for a feature and related features           */
 /* Can specify either current feature state or proposed state        */
 /* Can specify extent of related features to cost                    */
 /* Note that adding costs for several features may produce an        */
 /* excessively large cost due to shared components and parents.      */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiGetFeatureCostA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn, /* name of feature */
            type MSICOSTTREE,            /* portion of tree to cost */
            type INSTALLSTATE,
                          /* requested state, or INSTALLSTATE_UNKNOWN */
            type INT byaddr) /* returned cost, in units of 512 bytes */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetFeatureCostA');
 dcl MsiGetFeatureCostW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn, /* name of feature */
            type MSICOSTTREE,             /* portion of tree to cost */
            type INSTALLSTATE,
                          /* requested state, or INSTALLSTATE_UNKNOWN */
            type INT byaddr) /* returned cost, in units of 512 bytes */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetFeatureCostW');

 %dcl MsiGetFeatureCost char ext;
 %if UNICODE ^= '' %then %do;
   %MsiGetFeatureCost='MsiGetFeatureCostW';
 %end; %else %do;
   %MsiGetFeatureCost='MsiGetFeatureCostA';
 %end; /* !UNICODE */

 /* Set the install level for a full product installation (not a      */
 /* feature request)                                                  */
 /* Setting the value to 0 initialized components and features to the */
 /* default level                                                     */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiSetInstallLevel
     entry (type MSIHANDLE,
            fixed bin(31))
     returns (type UINT byvalue optional)
     WINAPI external('MsiSetInstallLevel');

 /* Get the valid install states for a feature, represented by bit    */
 /* flags                                                             */
 /* For each valid install state, a bit is set of value:              */
 /* (1 << INSTALLSTATE)                                               */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiGetFeatureValidStatesA
     entry (type MSIHANDLE,
            char (*) varz byaddr nonasgn,
            type DWORD byaddr)
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetFeatureValidStatesA');
 dcl MsiGetFeatureValidStatesW
     entry (type MSIHANDLE,
            WCHAR (*) varz byaddr nonasgn,
            type DWORD byaddr)
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetFeatureValidStatesW');

 %dcl MsiGetFeatureValidStates char ext;
 %if UNICODE ^= '' %then %do;
   %MsiGetFeatureValidStates='MsiGetFeatureValidStatesW';
 %end; %else %do;
   %MsiGetFeatureValidStates='MsiGetFeatureValidStatesA';
 %end; /* !UNICODE */

 /* Return the full source path for a folder in the Directory table   */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiGetSourcePathA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn,
               /* folder identifier, primary key into Directory table */
            char(*) varz byaddr,  /* buffer to return full path */
            type DWORD byaddr)    /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetSourcePathA');
 dcl MsiGetSourcePathW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn,
               /* folder identifier, primary key into Directory table */
            WCHAR(*) varz byaddr, /* buffer to return full path */
            type DWORD byaddr)    /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetSourcePathW');

 %dcl MsiGetSourcePath char ext;
 %if UNICODE ^= '' %then %do;
   %MsiGetSourcePath='MsiGetSourcePathW';
 %end; %else %do;
   %MsiGetSourcePath='MsiGetSourcePathA';
 %end; /* !UNICODE */

 /* Return the full target path for a folder in the Directory table   */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiGetTargetPathA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn,
               /* folder identifier, primary key into Directory table */
            char(*) varz byaddr,  /* buffer to return full path */
            type DWORD byaddr)    /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetTargetPathA');
 dcl MsiGetTargetPathW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn,
               /* folder identifier, primary key into Directory table */
            WCHAR(*) varz byaddr, /* buffer to return full path */
            type DWORD byaddr)    /* in/out buffer character count */
     returns (type UINT byvalue optional)
     WINAPI external('MsiGetTargetPathW');

 %dcl MsiGetTargetPath char ext;
 %if UNICODE ^= '' %then %do;
   %MsiGetTargetPath='MsiGetTargetPathW';
 %end; %else %do;
   %MsiGetTargetPath='MsiGetTargetPathA';
 %end; /* !UNICODE */

 /* Set the full target path for a folder in the Directory table      */
 /* Execution of this function sets the error record, accessible via  */
 /* MsiGetLastErrorRecord                                             */

 dcl MsiSetTargetPathA
     entry (type MSIHANDLE,
            char(*) varz byaddr nonasgn,
               /* folder identifier, primary key into Directory table */
            char(*) varz byaddr nonasgn)
               /* full path for folder, ending in directory separator */
     returns (type UINT byvalue optional)
     WINAPI external('MsiSetTargetPathA');
 dcl MsiSetTargetPathW
     entry (type MSIHANDLE,
            WCHAR(*) varz byaddr nonasgn,
               /* folder identifier, primary key into Directory table */
            WCHAR(*) varz byaddr nonasgn)
               /* full path for folder, ending in directory separator */
     returns (type UINT byvalue optional)
     WINAPI external('MsiSetTargetPathW');

 %dcl MsiSetTargetPath char ext;
 %if UNICODE ^= '' %then %do;
   %MsiSetTargetPath='MsiSetTargetPathW';
 %end; %else %do;
   %MsiSetTargetPath='MsiSetTargetPathA';
 %end; /* !UNICODE */

 /* Check to see if sufficent disk space is present for the current   */
 /* installation                                                      */
 /* Returns ERROR_SUCCESS, ERROR_DISK_FULL,                           */
 /* ERROR_INVALID_HANDLE_STATE, or ERROR_INVALID_HANDLE               */

 dcl MsiVerifyDiskSpace
     entry (type MSIHANDLE)
     returns (type UINT byvalue optional)
     WINAPI external('MsiVerifyDiskSpace');

 /* ----------------------------------------------------------------- */
 /* Functions for rendering UI dialogs from the database              */
 /* representations.                                                  */
 /* Purpose is for product development, not for use during            */
 /* installation.                                                     */
 /* ----------------------------------------------------------------- */

 /* Enable UI in preview mode to facilitate authoring of UI dialogs.  */
 /* The preview mode will end when the handle is closed.              */

 dcl MsiEnableUIPreview
     entry (type MSIHANDLE,
            type MSIHANDLE byaddr)
                         /* returned handle for UI preview capability */
     returns (type UINT byvalue optional)
     WINAPI external('MsiEnableUIPreview');

 /* Display any UI dialog as modeless and inactive.                   */
 /* Supplying a null name will remove any current dialog.             */

 dcl MsiPreviewDialogA
     entry (type MSIHANDLE,
            char (*) varz byaddr nonasgn optional)
                               /* dialog to display, Dialog table key */
     returns (type UINT byvalue optional)
     WINAPI external('MsiPreviewDialogA');
 dcl MsiPreviewDialogW
     entry (type MSIHANDLE,
            WCHAR (*) varz byaddr nonasgn optional)
                               /* dialog to display, Dialog table key */
     returns (type UINT byvalue optional)
     WINAPI external('MsiPreviewDialogW');

 %dcl MsiPreviewDialog char ext;
 %if UNICODE ^= '' %then %do;
   %MsiPreviewDialog='MsiPreviewDialogW';
 %end; %else %do;
   %MsiPreviewDialog='MsiPreviewDialogA';
 %end; /* !UNICODE */

 /* Display a billboard within a host control in the displayed dialog.*/
 /* Supplying a null billboard name will remove any billboard         */
 /* displayed.                                                        */

 dcl MsiPreviewBillboardA
     entry (type MSIHANDLE,
            char (*) varz byaddr nonasgn,
                           /* name of control that accepts billboards */
            char (*) varz byaddr nonasgn optional)
                                      /* name of billboard to display */
     returns (type UINT byvalue optional)
     WINAPI external('MsiPreviewBillboardA');
 dcl MsiPreviewBillboardW
     entry (type MSIHANDLE,
            WCHAR (*) varz byaddr nonasgn,
                           /* name of control that accepts billboards */
            WCHAR (*) varz byaddr nonasgn optional)
                                      /* name of billboard to display */
     returns (type UINT byvalue optional)
     WINAPI external('MsiPreviewBillboardW');

 %dcl MsiPreviewBillboard char ext;
 %if UNICODE ^= '' %then %do;
   %MsiPreviewBillboard='MsiPreviewBillboardW';
 %end; %else %do;
   %MsiPreviewBillboard='MsiPreviewBillboardA';
 %end; /* !UNICODE */

 /* Fetch the next sequential record from the view                    */
 /* Result is ERROR_SUCCESS if a row is found, and its handle is      */
 /* returned                                                          */
 /* else ERROR_NO_DATA if no records remain, and a null handle is     */
 /* returned                                                          */
 /* else result is error: ERROR_INVALID_HANDLE_STATE,                 */
 /* ERROR_INVALID_HANDLE, ERROR_GEN_FAILURE                           */

 /* ----------------------------------------------------------------- */
 /* Error handling not associated with any particular object          */
 /* ----------------------------------------------------------------- */

 /* Return a record handle to the last function that generated an     */
 /* error record                                                      */
 /* Only specified functions will set the error record, or clear it   */
 /* if success                                                        */
 /* Field 1 of the record will contain the internal MSI error code    */
 /* Other fields will contain data specific to the particular error   */
 /* The error record is released internally after this function is    */
 /* executed                                                          */

 dcl MsiGetLastErrorRecord
     entry () /* returns 0 if no cached record */
     returns (type MSIHANDLE byvalue optional)
     WINAPI external('MsiGetLastErrorRecord');

 %end; /* _MSIQUERY_H_ */
