 /* ++ BUILD Version: 0001                                            */
 /*      -- Increment this if a change has global effects             */
 /*                                                                   */
 /* Copyright (c) 1997-1999 Microsoft Corporation                     */
 /*                                                                   */
 /* Module Name:                                                      */
 /*                                                                   */
 /*   schedule.cpy                                                    */
 /*                                                                   */
 /* Abstract:                                                         */
 /*                                                                   */
 /*   This file defines a common schedule structure for use by        */
 /*   various NT components.                                          */
 /*                                                                   */
 /* --                                                                */

 %dcl #SCHEDULE_H_ char ext;
 %if #SCHEDULE_H_ = '' %then %do;
 %#SCHEDULE_H_ = 'Y';

 /* The DS and FRS use the same structure to represent different      */
 /* schedules.                                                        */
 /* The DS uses a 15-minute polling schedule. FRS uses a 60-minute    */
 /* start/stop schedule. Hence, the schedule for the system volume is */
 /* a special case because we only have the DS schedule to work from. */
 /* We will work around this problem by treating the 15               */
 /* -minute polling schedule as a 60-minute start/stop schedule.      */
 /* Replication is enabled for any hour that has any of the four      */
 /* 15-minute bits set.                                               */
 /*                                                                   */
 /* When the ReplicationSchedule is not present the default is        */
 /* "always replicate."                                               */

 /* Only the interval schedule is currently implemented.              */
 /* Others are ignored.                                               */
 dcl SCHEDULE_INTERVAL         fixed bin(31) value(0);
     /* schedule as understood by NT5 */
 dcl SCHEDULE_BANDWIDTH        fixed bin(31) value(1);
     /* bandwidth as understood by NT5 */
 dcl SCHEDULE_PRIORITY         fixed bin(31) value(2);
     /* priority as understood by NT5 */

 /* Schedule Header                                                   */

 /* Each schedule blob begins with n array of schedule headers that   */
 /* specify the number and type of schedules contained in the blob.   */

 define structure
   1 SCHEDULE_HEADER,
     2 Type   type ULONG, /* one of the SCHEDULE_ ordinals            */
     2 Offset type ULONG; /* offset from start of schedule structure  */
 define alias PSCHEDULE_HEADER handle SCHEDULE_HEADER;

 /* Schedule */

 define structure
   1 SCHEDULE,
     2 Size              type ULONG, /* inclusive size in bytes */
     2 Bandwidth         type ULONG,
     2 NumberOfSchedules type ULONG,
     2 Schedules (1)     type SCHEDULE_HEADER; /* "refer" */
 define alias PSCHEDULE handle SCHEDULE;
 /* The above structure is followed by the Data buffer and the        */
 /* SCHEDULE_HEADER contains offsets to refer to the appropriate      */
 /* parts in the data buffer.                                         */

 dcl SCHEDULE_DATA_ENTRIES fixed bin (31) value (7 * 24);
     /* 7 days X 24 hours */

 %end; /* _SCHEDULE_H_ */

