/******************************************************************/
/*                                                                */
/*    Licensed Materials - Property of IBM                        */
/*    5639-A83, 5639-A24 (C) Copyright IBM Corp. 1992,1996.       */
/*    All Rights Reserved.                                        */
/*    US Government Users Restricted Rights-- Use, duplication or */
/*    disclosure restricted by GSA ADP Schedule Contract with     */
/*    IBM Corp.                                                   */
/*                                                                */
/*  DISCLAIMER OF WARRANTIES                                      */
/*    The following enclosed code is sample code created by IBM */
/*    Corporation. This sample code is not part of any standard   */
/*    IBM product and is provided to you solely for the purpose of*/
/*    assisting you in the development of your applications.  The */
/*    code is provided "AS IS", without warranty of any kind.     */
/*    IBM shall not be liable for any damages arising out of your */
/*    use of the sample code, even if IBM has been advised of the */
/*    possibility of such damages.                                */
/*                                                                */
/*                                                                */
/* This is the VisualAge C++ program used to call the PL/I procedure  */
/* which has been compiled as object and packaged into a DLL.     */
/* The C program will first load the PL/I DLL and query its first */
/* entry point in one thread.  Then, it will call the entry point */
/* in another thread.  The C function also creates other threads. */
/* The thread that does not call the PL/I procedure will only     */
/* print out a message and will return to the main thread.        */
/* The main thread will use printf function to print out message  */
/* to show the progress of each major step during execution.      */
/* At the end, the PL/I DLL will be freed explicitly to ensure    */
/* proper program termination.                                    */
/*                                                                */
/* Syntax:                                                        */
/*    LD1CALL1 arg1                                               */
/*           where arg1 is the name of the PL/I DLL               */
/*                                                                */
/* If no argument is supplied, LD1CALL1 will assume the DLL of    */
/* "HIWORLD" and will call it.                                    */
/*                                                                */
/******************************************************************/

#include <windows.h>
#include <malloc.h>
#include <process.h>
#include <stdio.h>
#include <stdlib.h>

/*******************************************************************/
/* Global static and automatic variables.                          */
/*******************************************************************/

static int waitthd1 = 1;
static int waitthd2 = 1;
static int waitthd3 = 1;
static int waitsem1 = 1;
int gargc;
char *gargv[4];
HMODULE mh;
unsigned long handle;
FARPROC pa;
char *pc;
char objnamebuf[16];
char pn[]="HIWORLD";

/*******************************************************************/
/* The 1st child thread code starts here.  It only calls the PL/I. */
/*******************************************************************/

void thread2( void *arg2 )
{
   char *upc;

   while (waitsem1)
     {
        Sleep(0);
     } /* endwhile */
   fprintf(stderr,"THREAD2(%s) ===> Hello from the 1st child thread.\n", pc);
   fprintf(stderr,"THREAD2(%s) ===> The beginning of the PL/I thread.\n", pc);
   fprintf(stderr,"THREAD2(%s) ===> GetProcAddr address = %d.\n",pc, pa);
   if (gargc > 2)
     {
       upc = gargv[2];
       while ( (*upc) != '\0')
         {
           (*upc) = toupper( (*upc) );
           upc ++;
         } /* endwhile */
      fprintf(stderr,"THREAD2(%s) ===> Command line arg2= %s.\n",pc,gargv[2]);
      (void (* APIENTRY)(char *))pa(gargv[2]);
     }
   else
     pa();
   if (_freemod(handle)) {
     printf("Error in freeing the %s DLL module\n",pc);
     return ;
   }
   fprintf(stderr,"THREAD2(%s) ===> The end of the PL/I thread.\n", pc);
   fprintf(stderr,"THREAD2(%s) ===> End of the 1st child thread.\n", pc);
   waitthd2 = 0;
}

/*******************************************************************/
/* The 2nd child thread code starts here.  It only loads the PL/I  */
/*******************************************************************/

void thread3 (void *arg3)
{
   char *upc;

   while (waitthd3)
     {
        Sleep(0);
     } /* endwhile */
   if (gargc > 1)
     {
       upc = gargv[1];
       while ( (*upc) != '\0')
         {
           (*upc) = toupper( (*upc) );
           upc ++;
         } /* endwhile */
       pc = gargv[1];    /* Pick up the optional cmd line argument.*/
     }
   else
     pc = pn;         /* Default is to pick up "HIWORLD".       */
   fprintf(stderr,"THREAD3(%s) ===> Hello from the 2nd child thread.\n", pc);
   fprintf(stderr,"%s ===> First call to the routine by _loadmod\n", pc);
   if ( _loadmod( pc, &handle)) {
     printf("Error loading module %s\n",pc);
     fprintf(stderr,"%s ===> This thread will be terminated.\n", pc);
     return ;
   }
   else
     printf("_loadmod completed sucessfully \n");

   pa = GetProcAddress((HMODULE) handle,pc) ;
   if (pa==(FARPROC) 0)
    {
     printf("Error locating address of function %s\n",pa);
     return;
     }
   fprintf(stderr,"%s ===> GetProcAddress  = %d.\n", pc, pa);
   waitsem1 = 0;

   while (waitthd2)
     {
        Sleep(0);
     } /* endwhile */
   fprintf(stderr,"THREAD3(%s) ===> End of the 2nd child thread.\n", pc);
}

/*******************************************************************/
/* The 3rd child thread code starts here....                       */
/*******************************************************************/

void thread4( void *arg4)
{
   while (waitthd1)
     {
        Sleep(0);
     } /* endwhile */
   fprintf(stderr,"THREAD4 ===> Hello world from the 3rd child thread.\n");
}

/*******************************************************************/
/* The main thread code starts here....                            */
/*******************************************************************/

int main (int argc, char *argv[])
{
   /* argv is a ptr to an array of char ptrs.  argv[0] = EXE name.  */
   /* argv[1] = 1st cmd line arguement.  argc = 1 means no argument.*/

   unsigned long tid2;
   unsigned long tid3;
   unsigned long tid4;
   unsigned stack_size=2000000;
   char *pcarg;

   fprintf(stderr,"************* Start of the main thread #1 ************\n");
   if (argc > 1)
     {
       pcarg = argv[1]; /* Pick up the optional cmd line argument.*/
       gargv[1] = argv[1];
     }
   else
     pcarg = NULL;
   if (argc > 2)
     {
       gargv[2] = argv[2];
     }
   gargc = argc;
   gargv[0] = argv[0];

   tid2 = _beginthread(thread2, NULL, stack_size, NULL);

   if ( tid2 == -1 )
     {
       fprintf(stderr,"Unable to start 1st child thread...\n");
       exit(-1);
     }
   else
     {
       fprintf(stderr,"*** The 1st child thread started with ID %d.\n", tid2);
     } /* endif */

   tid3 = _beginthread(thread3, NULL, stack_size, pcarg);

   if ( tid3 == -1 )
     {
       fprintf(stderr,"Unable to start 2nd child thread...\n");
       exit(-1);
     }
   else
     {
       fprintf(stderr,"*** The 2nd child thread started with ID %d.\n", tid3);
       waitthd3 = 0;
     } /* endif */

   tid4 = _beginthread(thread4, NULL, stack_size, NULL);

   if ( tid4 == -1 )
     {
       fprintf(stderr,"Unable to start 3rd child thread...\n");
       exit(-1);
     }
   else
     {
       fprintf(stderr,"*** The 3rd child thread started with ID %d.\n", tid4);
       waitthd1 = 0;
     } /* endif */

   WaitForSingleObject((HANDLE)tid2,INFINITE);
   WaitForSingleObject((HANDLE)tid3,INFINITE);
   WaitForSingleObject((HANDLE)tid4,INFINITE);

   fprintf(stderr,"************** End of the main thread #1 *************\n");
   return 0;
}
